@@ -21,6 +21,7 @@
 #include "ui_itemnotessettings.h"
 
 #include "common/contenttype.h"
+#include "gui/iconfont.h"
 #include "gui/iconwidget.h"
 
 #include <QBoxLayout>
@@ -34,33 +35,66 @@
 #include <QTimer>
 #include <QToolTip>
 #include <QtPlugin>
-#include <QDebug>
+
+#include <cmath>
 
 namespace {
 
 // Limit number of characters for performance reasons.
 const int defaultMaxBytes = 10*1024;
 
-const char mimeEncryptedData[] = "application/x-copyq-item-notes";
+const char mimeNotes[] = "application/x-copyq-item-notes";
+const char mimeIcon[] = "application/x-copyq-item-icon";
 
 const int notesIndent = 16;
 
+QWidget *createIconWidget(const QByteArray &icon, QWidget *parent)
+{
+    if (!icon.isEmpty()) {
+        QPixmap p;
+        if (p.loadFromData(icon)) {
+#if QT_VERSION < 0x050000
+            const auto ratio = 1;
+#else
+            const auto ratio = parent->devicePixelRatio();
+            p.setDevicePixelRatio(ratio);
+#endif
+            const int side = ratio * (iconFontSizePixels() + 2);
+            p = p.scaled(side, side, Qt::KeepAspectRatio, Qt::SmoothTransformation);
+            QLabel *label = new QLabel(parent);
+            const auto m = side / 4;
+            label->setFixedSize( p.size() / ratio + QSize(m, m) );
+            label->setAlignment(Qt::AlignCenter);
+            label->setPixmap(p);
+            return label;
+        }
+    }
+
+    return new IconWidget(IconEditSign, parent);
+}
+
 } // namespace
 
-ItemNotes::ItemNotes(ItemWidget *childItem, const QString &text,
+ItemNotes::ItemNotes(ItemWidget *childItem, const QString &text, const QByteArray &icon,
                      bool notesAtBottom, bool showIconOnly, bool showToolTip)
     : QWidget( childItem->widget()->parentWidget() )
     , ItemWidget(this)
-    , m_notes( showIconOnly ? NULL : new QTextEdit(this) )
-    , m_icon( showIconOnly ? new IconWidget(IconEditSign, this) : NULL )
+    , m_notes(nullptr)
+    , m_icon(nullptr)
     , m_childItem(childItem)
     , m_notesAtBottom(notesAtBottom)
-    , m_timerShowToolTip(NULL)
+    , m_timerShowToolTip(nullptr)
     , m_toolTipText()
 {
     m_childItem->widget()->setObjectName("item_child");
     m_childItem->widget()->setParent(this);
 
+    if (showIconOnly || !icon.isEmpty())
+        m_icon = createIconWidget(icon, this);
+
+    if (!showIconOnly)
+        m_notes = new QTextEdit(this);
+
     QBoxLayout *layout;
 
     if (showIconOnly) {
@@ -69,8 +103,7 @@ ItemNotes::ItemNotes(ItemWidget *childItem, const QString &text,
         layout->addWidget(m_childItem->widget());
     } else {
         m_notes->setObjectName("item_child");
-
-        m_notes->document()->setDefaultFont(font());
+        m_notes->setProperty("CopyQ_item_type", "notes");
 
         m_notes->setReadOnly(true);
         m_notes->setUndoRedoEnabled(false);
@@ -87,10 +120,14 @@ ItemNotes::ItemNotes(ItemWidget *childItem, const QString &text,
 
         layout = new QVBoxLayout(this);
 
-        QHBoxLayout *labelLayout = new QHBoxLayout;
+        auto labelLayout = new QHBoxLayout;
         labelLayout->setMargin(0);
         labelLayout->setContentsMargins(notesIndent, 0, 0, 0);
-        labelLayout->addWidget(m_notes, 0, Qt::AlignLeft);
+
+        if (m_icon)
+            labelLayout->addWidget(m_icon, 0, Qt::AlignLeft | Qt::AlignTop);
+
+        labelLayout->addWidget(m_notes, 1, Qt::AlignLeft);
 
         if (notesAtBottom) {
             layout->addWidget( m_childItem->widget() );
@@ -116,9 +153,11 @@ ItemNotes::ItemNotes(ItemWidget *childItem, const QString &text,
 
 void ItemNotes::setCurrent(bool current)
 {
+    m_isCurrent = current;
+
     ItemWidget::setCurrent(current);
 
-    if (m_timerShowToolTip == NULL)
+    if (m_timerShowToolTip == nullptr)
         return;
 
     QToolTip::hideText();
@@ -133,7 +172,7 @@ void ItemNotes::highlight(const QRegExp &re, const QFont &highlightFont, const Q
 {
     m_childItem->setHighlight(re, highlightFont, highlightPalette);
 
-    if (m_notes != NULL) {
+    if (m_notes != nullptr) {
         QList<QTextEdit::ExtraSelection> selections;
 
         if ( !re.isEmpty() ) {
@@ -171,24 +210,24 @@ void ItemNotes::highlight(const QRegExp &re, const QFont &highlightFont, const Q
 
 QWidget *ItemNotes::createEditor(QWidget *parent) const
 {
-    return m_childItem.isNull() ? NULL : m_childItem->createEditor(parent);
+    return (m_childItem == nullptr) ? nullptr : m_childItem->createEditor(parent);
 }
 
 void ItemNotes::setEditorData(QWidget *editor, const QModelIndex &index) const
 {
-    Q_ASSERT( !m_childItem.isNull() );
-    return m_childItem->setEditorData(editor, index);
+    Q_ASSERT(m_childItem != nullptr);
+    m_childItem->setEditorData(editor, index);
 }
 
 void ItemNotes::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
 {
-    Q_ASSERT( !m_childItem.isNull() );
-    return m_childItem->setModelData(editor, model, index);
+    Q_ASSERT(m_childItem != nullptr);
+    m_childItem->setModelData(editor, model, index);
 }
 
 bool ItemNotes::hasChanges(QWidget *editor) const
 {
-    Q_ASSERT( !m_childItem.isNull() );
+    Q_ASSERT(m_childItem != nullptr);
     return m_childItem->hasChanges(editor);
 }
 
@@ -198,7 +237,7 @@ QObject *ItemNotes::createExternalEditor(const QModelIndex &index, QWidget *pare
                        : ItemWidget::createExternalEditor(index, parent);
 }
 
-void ItemNotes::updateSize(const QSize &maximumSize)
+void ItemNotes::updateSize(QSize maximumSize, int idealWidth)
 {
     setMaximumSize(maximumSize);
 
@@ -206,11 +245,13 @@ void ItemNotes::updateSize(const QSize &maximumSize)
         const int w = maximumSize.width() - 2 * notesIndent - 8;
         QTextDocument *doc = m_notes->document();
         doc->setTextWidth(w);
-        m_notes->setFixedSize( doc->idealWidth() + 16, doc->size().height() );
+        m_notes->setFixedSize(
+                    static_cast<int>(doc->idealWidth()) + 16,
+                    static_cast<int>(doc->size().height()) );
     }
 
-    if ( !m_childItem.isNull() )
-        m_childItem->updateSize(maximumSize);
+    if (m_childItem != nullptr)
+        m_childItem->updateSize(maximumSize, idealWidth);
 
     adjustSize();
     setFixedSize(sizeHint());
@@ -221,20 +262,25 @@ void ItemNotes::paintEvent(QPaintEvent *event)
     QWidget::paintEvent(event);
 
     // Decorate notes.
-    if (m_notes != NULL) {
+    if (m_notes != nullptr) {
         QPainter p(this);
 
         QColor c = p.pen().color();
         c.setAlpha(80);
         p.setBrush(c);
         p.setPen(Qt::NoPen);
-        p.drawRect(m_notes->x() - notesIndent + 4, m_notes->y() + 4,
-                   notesIndent - 4, m_notes->height() - 8);
+        QWidget *w = m_icon ? m_icon : m_notes;
+        const auto height = std::max(w->height(), m_notes->height()) - 8;
+        p.drawRect(w->x() - notesIndent + 4, w->y() + 4,
+                   notesIndent - 4, height);
     }
 }
 
 bool ItemNotes::eventFilter(QObject *, QEvent *event)
 {
+    if ( event->type() == QEvent::Show && m_timerShowToolTip && m_isCurrent )
+        m_timerShowToolTip->start();
+
     return ItemWidget::filterMouseEvents(m_notes, event);
 }
 
@@ -242,6 +288,9 @@ void ItemNotes::showToolTip()
 {
     QToolTip::hideText();
 
+    if ( !isVisible() )
+        return;
+
     QPoint toolTipPosition = QPoint(parentWidget()->contentsRect().width() - 16, height() - 16);
     toolTipPosition = mapToGlobal(toolTipPosition);
 
@@ -252,13 +301,11 @@ ItemNotesLoader::ItemNotesLoader()
 {
 }
 
-ItemNotesLoader::~ItemNotesLoader()
-{
-}
+ItemNotesLoader::~ItemNotesLoader() = default;
 
 QStringList ItemNotesLoader::formatsToSave() const
 {
-    return QStringList(mimeEncryptedData);
+    return QStringList() << mimeNotes << mimeIcon;
 }
 
 QVariantMap ItemNotesLoader::applySettings()
@@ -290,10 +337,12 @@ QWidget *ItemNotesLoader::createSettingsWidget(QWidget *parent)
 ItemWidget *ItemNotesLoader::transform(ItemWidget *itemWidget, const QModelIndex &index)
 {
     const QString text = index.data(contentType::notes).toString();
-    if ( text.isEmpty() )
-        return NULL;
+    const QByteArray icon = index.data(contentType::data).toMap().value(mimeIcon).toByteArray();
+    if ( text.isEmpty() && icon.isEmpty() )
+        return nullptr;
 
-    return new ItemNotes( itemWidget, text,
+    itemWidget->setTagged(true);
+    return new ItemNotes( itemWidget, text, icon,
                           m_settings["notes_at_bottom"].toBool(),
                           m_settings["icon_only"].toBool(),
             m_settings["show_tooltip"].toBool() );
