@@ -20,25 +20,19 @@
 #include "itemsynctests.h"
 
 #include "common/mimetypes.h"
+#include "tests/test_utils.h"
 
 #include <QDir>
 #include <QFile>
-#include <QTemporaryFile>
-#include <QTest>
 
-#define RUN(arguments, stdoutExpected) \
-    TEST( m_test->runClient(arguments, stdoutExpected) );
+#include <memory>
 
 namespace {
 
-typedef QSharedPointer<QFile> FilePtr;
-typedef QStringList Args;
+using FilePtr = std::shared_ptr<QFile>;
 
 const char sep[] = " ;; ";
 
-/// Interval to wait (in ms) for items to synchronize from files.
-const int waitMsFiles = 2500;
-
 QString fileNameForId(int i)
 {
     return QString("copyq_%1.txt").arg(i, 4, 10, QChar('0'));
@@ -46,7 +40,7 @@ QString fileNameForId(int i)
 
 class TestDir {
 public:
-    TestDir(int i, bool createPath = true)
+    explicit TestDir(int i, bool createPath = true)
         : m_dir(ItemSyncTests::testDir(i))
     {
         clear();
@@ -62,7 +56,7 @@ public:
     void clear()
     {
         if (isValid()) {
-            foreach ( const QString &fileName, files() )
+            for ( const auto &fileName : files() )
                 remove(fileName);
             m_dir.rmpath(".");
         }
@@ -86,7 +80,7 @@ public:
 
     FilePtr file(const QString &fileName) const
     {
-        return FilePtr(new QFile(filePath(fileName)));
+        return std::make_shared<QFile>(filePath(fileName));
     }
 
     QString filePath(const QString &fileName) const
@@ -130,7 +124,7 @@ ItemSyncTests::ItemSyncTests(const TestInterfacePtr &test, QObject *parent)
 
 QString ItemSyncTests::testTab(int i)
 {
-    return "ITEMSYNC_TEST_&" + QString::number(i);
+    return ::testTab(i);
 }
 
 QString ItemSyncTests::testDir(int i)
@@ -140,13 +134,12 @@ QString ItemSyncTests::testDir(int i)
 
 void ItemSyncTests::initTestCase()
 {
-    TEST(m_test->init());
-    cleanup();
+    TEST(m_test->initTestCase());
 }
 
 void ItemSyncTests::cleanupTestCase()
 {
-    TEST(m_test->stopServer());
+    TEST(m_test->cleanupTestCase());
 }
 
 void ItemSyncTests::init()
@@ -206,7 +199,9 @@ void ItemSyncTests::itemsToFiles()
     const QString tab1 = testTab(1);
     const Args args = Args() << "tab" << tab1;
 
-    RUN(Args(args) << "add" << "0" << "1" << "2", "");
+    RUN(args << "add" << "A" << "B" << "C", "");
+    RUN(args << "read" << "0" << "1" << "2", "C\nB\nA");
+    RUN(args << "size", "3\n");
 
     QCOMPARE( dir1.files().join(sep),
               fileNameForId(0) + sep + fileNameForId(1) + sep + fileNameForId(2) );
@@ -218,7 +213,7 @@ void ItemSyncTests::filesToItems()
     const QString tab1 = testTab(1);
     const Args args = Args() << "tab" << tab1;
 
-    RUN(Args(args) << "size", "0\n");
+    RUN(args << "size", "0\n");
 
     const QByteArray text1 = "Hello world!";
     createFile(dir1, "test1.txt", text1);
@@ -228,22 +223,19 @@ void ItemSyncTests::filesToItems()
     const QByteArray text2 = "And hello again!";
     TEST(createFile(dir1, "test2.txt", text2));
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "2\n");
+    WAIT_ON_OUTPUT(args << "size", "2\n");
     // Newer files first.
-    RUN(Args(args) << "read" << "0", text2);
-    RUN(Args(args) << "read" << "1", text1);
+    RUN(args << "read" << "0", text2);
+    RUN(args << "read" << "1", text1);
 }
 
-void ItemSyncTests::removeItems()
+void ItemSyncTests::removeOwnItems()
 {
     TestDir dir1(1);
     const QString tab1 = testTab(1);
     const Args args = Args() << "separator" << "," << "tab" << tab1;
 
-    RUN(Args(args) << "add" << "A" << "B" << "C" << "D", "");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,C,B,A");
+    RUN(args << "add" << "A" << "B" << "C" << "D", "");
 
     const QString fileA = fileNameForId(0);
     const QString fileB = fileNameForId(1);
@@ -258,16 +250,60 @@ void ItemSyncTests::removeItems()
               );
 
     // Move to test tab and select second and third item.
-    RUN(Args(args) << "keys" << "RIGHT", "");
-    RUN(Args(args) << "keys" << "HOME" << "DOWN" << "SHIFT+DOWN", "");
-    RUN(Args(args) << "selectedtab", tab1.toUtf8() + "\n");
-    RUN(Args(args) << "currentitem", "2\n");
-    RUN(Args(args) << "selecteditems", "1\n2\n");
+    RUN("setCurrentTab" << tab1, "");
+    RUN(args << "selectItems" << "1" << "2", "true\n");
+    RUN(args << "testSelected", tab1.toUtf8() + " 2 1 2\n");
+
+    // Remove selected items.
+    RUN(args << "keys" << m_test->shortcutToRemove(), "");
+    RUN(args << "read" << "0" << "1" << "2" << "3", "D,A,,");
+    QCOMPARE( dir1.files().join(sep),
+              fileA
+              + sep + fileD
+              );
+
+    // Removing own items works from script.
+    RUN(args << "remove" << "1", "");
+    RUN(args << "read" << "0" << "1" << "2" << "3", "D,,,");
+    QCOMPARE( dir1.files().join(sep), fileD );
+}
+
+void ItemSyncTests::removeNotOwnedItems()
+{
+    TestDir dir1(1);
+    const QString tab1 = testTab(1);
+    const Args args = Args() << "separator" << "," << "tab" << tab1;
+
+    const QString fileA = "test1.txt";
+    const QString fileB = "test2.txt";
+    const QString fileC = "test3.txt";
+    const QString fileD = "test4.txt";
+
+    TEST(createFile(dir1, fileA, "A"));
+    WAIT_ON_OUTPUT(args << "size", "1\n");
+    TEST(createFile(dir1, fileB, "B"));
+    WAIT_ON_OUTPUT(args << "size", "2\n");
+    TEST(createFile(dir1, fileC, "C"));
+    WAIT_ON_OUTPUT(args << "size", "3\n");
+    TEST(createFile(dir1, fileD, "D"));
+    WAIT_ON_OUTPUT(args << "size", "4\n");
+
+    QCOMPARE( dir1.files().join(sep),
+              fileA
+              + sep + fileB
+              + sep + fileC
+              + sep + fileD
+              );
+
+    // Move to test tab and select second and third item.
+    RUN("setCurrentTab" << tab1, "");
+    RUN(args << "selectItems" << "1" << "2", "true\n");
+    RUN(args << "testSelected", tab1.toUtf8() + " 2 1 2\n");
 
     // Don't accept the "Remove Items?" dialog.
-    RUN(Args(args) << "keys" << m_test->shortcutToRemove(), "");
-    RUN(Args(args) << "keys" << "ESCAPE", "");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,C,B,A");
+    RUN(args << "keys" << m_test->shortcutToRemove(), "");
+    RUN(args << "keys" << "ESCAPE", "");
+    RUN(args << "read" << "0" << "1" << "2" << "3", "D,C,B,A");
     QCOMPARE( dir1.files().join(sep),
               fileA
               + sep + fileB
@@ -276,17 +312,21 @@ void ItemSyncTests::removeItems()
               );
 
     // Accept the "Remove Items?" dialog.
-    RUN(Args(args) << "keys" << m_test->shortcutToRemove(), "");
-    RUN(Args(args) << "keys" << "ENTER", "");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,A,,");
+    RUN(args << "keys" << m_test->shortcutToRemove(), "");
+    RUN(args << "keys" << "ENTER", "");
+    RUN(args << "read" << "0" << "1" << "2" << "3", "D,A,,");
+    QCOMPARE( dir1.files().join(sep),
+              fileA
+              + sep + fileD
+              );
+
+    // Removing not owned items from script doesn't work.
+    RUN_EXPECT_ERROR(args << "remove" << "1", CommandException);
     QCOMPARE( dir1.files().join(sep),
               fileA
               + sep + fileD
               );
 
-    SKIP("Removing files with \"remove\" command doesn't work!");
-    RUN(Args(args) << "remove" << "1", "");
-    QCOMPARE( dir1.files().join(sep), fileD );
 }
 
 void ItemSyncTests::removeFiles()
@@ -295,9 +335,7 @@ void ItemSyncTests::removeFiles()
     const QString tab1 = testTab(1);
     const Args args = Args() << "separator" << "," << "tab" << tab1;
 
-    RUN(Args(args) << "add" << "A" << "B" << "C" << "D", "");
-    RUN(Args(args) << "size", "4\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,C,B,A");
+    RUN(args << "add" << "A" << "B" << "C" << "D", "");
 
     const QString fileA = fileNameForId(0);
     const QString fileB = fileNameForId(1);
@@ -316,18 +354,14 @@ void ItemSyncTests::removeFiles()
     QCOMPARE(file->readAll().data(), QByteArray("C").data());
     file->remove();
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "3\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2", "D,B,A");
+    WAIT_ON_OUTPUT(args << "size", "3\n");
+    RUN(args << "read" << "0" << "1" << "2", "D,B,A");
 
     dir1.file(fileB)->remove();
     dir1.file(fileA)->remove();
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "1\n");
-    RUN(Args(args) << "read" << "0", "D");
+    WAIT_ON_OUTPUT(args << "size", "1\n");
+    RUN(args << "read" << "0", "D");
 }
 
 void ItemSyncTests::modifyItems()
@@ -336,9 +370,7 @@ void ItemSyncTests::modifyItems()
     const QString tab1 = testTab(1);
     const Args args = Args() << "separator" << "," << "tab" << tab1;
 
-    RUN(Args(args) << "add" << "A" << "B" << "C" << "D", "");
-    RUN(Args(args) << "size", "4\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,C,B,A");
+    RUN(args << "add" << "A" << "B" << "C" << "D", "");
 
     const QString fileC = fileNameForId(2);
     FilePtr file = dir1.file(fileC);
@@ -346,9 +378,9 @@ void ItemSyncTests::modifyItems()
     QCOMPARE(file->readAll().data(), QByteArray("C").data());
     file->close();
 
-    RUN(Args(args) << "keys" << "RIGHT" << "HOME" << "DOWN" << "F2" << ":XXX" << "F2", "");
-    RUN(Args(args) << "size", "4\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,XXX,B,A");
+    RUN(args << "keys" << "RIGHT" << "HOME" << "DOWN" << "F2" << ":XXX" << "F2", "");
+    RUN(args << "size", "4\n");
+    RUN(args << "read" << "0" << "1" << "2" << "3", "D,XXX,B,A");
 
     file = dir1.file(fileC);
     QVERIFY(file->open(QIODevice::ReadOnly));
@@ -362,9 +394,7 @@ void ItemSyncTests::modifyFiles()
     const QString tab1 = testTab(1);
     const Args args = Args() << "separator" << "," << "tab" << tab1;
 
-    RUN(Args(args) << "add" << "A" << "B" << "C" << "D", "");
-    RUN(Args(args) << "size", "4\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,C,B,A");
+    RUN(args << "add" << "A" << "B" << "C" << "D", "");
 
     const QString fileA = fileNameForId(0);
     const QString fileB = fileNameForId(1);
@@ -384,10 +414,8 @@ void ItemSyncTests::modifyFiles()
     file->write("X");
     file->close();
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "4\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2" << "3", "D,CX,B,A");
+    WAIT_ON_OUTPUT(args << "read" << "0" << "1" << "2" << "3", "D,CX,B,A");
+    RUN(args << "size", "4\n");
 }
 
 void ItemSyncTests::notes()
@@ -397,13 +425,13 @@ void ItemSyncTests::notes()
 
     const Args args = Args() << "separator" << ";" << "tab" << tab1;
 
-    RUN(Args(args) << "add" << "TEST1", "");
+    RUN(args << "add" << "TEST1", "");
 
-    RUN(Args(args) << "keys" << "LEFT", "");
-    RUN(Args(args) << "keys" << "CTRL+N" << ":TEST2" << "F2", "");
-    RUN(Args(args) << "keys" << "CTRL+N" << ":TEST3" << "F2", "");
-    RUN(Args(args) << "size", "3\n");
-    RUN(Args(args) << "read" << "0" << "1" << "2", "TEST3;TEST2;TEST1");
+    RUN(args << "keys" << "LEFT"
+        << "CTRL+N" << ":TEST2" << "F2"
+        << "CTRL+N" << ":TEST3" << "F2", "");
+    RUN(args << "size", "3\n");
+    RUN(args << "read" << "0" << "1" << "2", "TEST3;TEST2;TEST1");
 
     const QString fileTest1 = fileNameForId(0);
     const QString fileTest2 = fileNameForId(1);
@@ -413,8 +441,8 @@ void ItemSyncTests::notes()
 
     QCOMPARE( dir1.files().join(sep), files1.join(sep) );
 
-    RUN(Args(args) << "keys" << "HOME" << "DOWN" << "SHIFT+F2" << ":NOTE1" << "F2", "");
-    RUN(Args(args) << "read" << mimeItemNotes << "0" << "1" << "2", ";NOTE1;");
+    RUN(args << "keys" << "HOME" << "DOWN" << "SHIFT+F2" << ":NOTE1" << "F2", "");
+    RUN(args << "read" << mimeItemNotes << "0" << "1" << "2", ";NOTE1;");
 
     // One new file for notes.
     const QStringList files2 = dir1.files();
@@ -431,18 +459,14 @@ void ItemSyncTests::notes()
     file->write("+NOTE2");
     file->close();
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "read" << mimeItemNotes << "0" << "1" << "2", ";NOTE1+NOTE2;");
-    RUN(Args(args) << "size", "3\n");
+    WAIT_ON_OUTPUT(args << "read" << mimeItemNotes << "0" << "1" << "2", ";NOTE1+NOTE2;");
+    RUN(args << "size", "3\n");
 
     // Remove notes.
     QVERIFY(file->remove());
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "read" << mimeItemNotes << "0" << "1" << "2", ";;");
-    RUN(Args(args) << "size", "3\n");
+    WAIT_ON_OUTPUT(args << "read" << mimeItemNotes << "0" << "1" << "2", ";;");
+    RUN(args << "size", "3\n");
 }
 
 void ItemSyncTests::customFormats()
@@ -454,42 +478,34 @@ void ItemSyncTests::customFormats()
     const QByteArray data1 = "Custom format content";
     createFile(dir1, "test1.xxx", data1);
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "1\n");
-    RUN(Args(args) << "keys" << "LEFT", "");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-xxx" << "0", data1);
+    WAIT_ON_OUTPUT(args << "size", "1\n");
+    RUN(args << "keys" << "LEFT", "");
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-xxx" << "0", data1);
 
     const QByteArray data2 = "Other custom format content";
     createFile(dir1, "test2.yyy", data2);
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "2\n");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0", data2);
+    WAIT_ON_OUTPUT(args << "size", "2\n");
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0", data2);
 
     const QByteArray data3 = "Last custom format content";
     createFile(dir1, "test3.zzz", data3);
 
-    QTest::qSleep(waitMsFiles);
+    WAIT_ON_OUTPUT(args << "size", "3\n");
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0", data3);
 
-    RUN(Args(args) << "size", "3\n");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0", data3);
-
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-xxx" << "0" << "1" << "2",
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-xxx" << "0" << "1" << "2",
         ";;" + data1);
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << "1" << "2",
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << "1" << "2",
         data3 + ";" + data2 + ";");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << "1" << COPYQ_MIME_PREFIX "test-xxx" << "2",
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << "1" << COPYQ_MIME_PREFIX "test-xxx" << "2",
         data3 + ";" + data2 + ";" + data1);
 
     // Remove
     dir1.remove("test2.yyy");
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "2\n");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << COPYQ_MIME_PREFIX "test-xxx" << "1",
+    WAIT_ON_OUTPUT(args << "size", "2\n");
+    RUN(args << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << COPYQ_MIME_PREFIX "test-xxx" << "1",
         data3 + ";" + data1);
 
     // Modify file
@@ -499,17 +515,16 @@ void ItemSyncTests::customFormats()
     file->write(data4);
     file->close();
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "2\n");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << COPYQ_MIME_PREFIX "test-xxx" << "1",
+    WAIT_ON_OUTPUT(
+        Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << COPYQ_MIME_PREFIX "test-xxx" << "1",
         data3 + ";" + data1 + data4);
+    RUN(args << "size", "2\n");
 
     // Create item with custom data
     const QByteArray data5 = "New item data!";
-    RUN(Args(args) << "write" << COPYQ_MIME_PREFIX "test-zzz" << data5, "");
+    RUN(args << "write" << COPYQ_MIME_PREFIX "test-zzz" << data5, "");
 
-    RUN(Args(args) << "size", "3\n");
+    RUN(args << "size", "3\n");
 
     const QString fileData = QString(fileNameForId(0)).replace("txt", "zzz");
 
@@ -524,9 +539,8 @@ void ItemSyncTests::customFormats()
     file->write(data6);
     file->close();
 
-    QTest::qSleep(waitMsFiles);
-
-    RUN(Args(args) << "size", "3\n");
-    RUN(Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << "1" << COPYQ_MIME_PREFIX "test-xxx" << "2",
+    WAIT_ON_OUTPUT(
+        Args(args) << "read" << COPYQ_MIME_PREFIX "test-zzz" << "0" << "1" << COPYQ_MIME_PREFIX "test-xxx" << "2",
         data5 + data6 + ";" + data3 + ";" + data1 + data4);
+    RUN(args << "size", "3\n");
 }
