@@ -23,7 +23,11 @@
 #include "common/contenttype.h"
 #include "common/mimetypes.h"
 
+#include <QAbstractTextDocumentLayout>
+#include <QCoreApplication>
 #include <QContextMenuEvent>
+#include <QCursor>
+#include <QMimeData>
 #include <QModelIndex>
 #include <QMouseEvent>
 #include <QScrollBar>
@@ -35,7 +39,15 @@
 namespace {
 
 // Limit number of characters for performance reasons.
-const int defaultMaxBytes = 100*1024;
+const int maxCharacters = 100 * 1024;
+
+// Limit line length for performance reasons.
+const int maxLineLength = 1024;
+const int maxLineLengthInPreview = 16 * maxLineLength;
+
+// Limit line count for performance reasons.
+const int maxLineCount = 4 * 1024;
+const int maxLineCountInPreview = 16 * maxLineCount;
 
 const char optionUseRichText[] = "use_rich_text";
 const char optionMaximumLines[] = "max_lines";
@@ -43,6 +55,14 @@ const char optionMaximumHeight[] = "max_height";
 
 const char mimeRichText[] = "text/richtext";
 
+// Some applications insert \0 teminator at the end of text data.
+// It needs to be removed because QTextBrowser can render the character.
+void removeTrailingNull(QString *text)
+{
+    if ( text->endsWith(QChar(0)) )
+        text->chop(1);
+}
+
 bool getRichText(const QModelIndex &index, QString *text)
 {
     if ( index.data(contentType::hasHtml).toBool() ) {
@@ -57,10 +77,6 @@ bool getRichText(const QModelIndex &index, QString *text)
     const QByteArray data = dataMap[mimeRichText].toByteArray();
     *text = QString::fromUtf8(data);
 
-    // Remove trailing null character.
-    if ( text->endsWith(QChar(0)) )
-        text->resize(text->size() - 1);
-
     return true;
 }
 
@@ -74,10 +90,24 @@ bool getText(const QModelIndex &index, QString *text)
     return false;
 }
 
+QString normalizeText(QString text)
+{
+    removeTrailingNull(&text);
+    return text.left(maxCharacters);
+}
+
+void insertEllipsis(QTextCursor *tc)
+{
+    tc->insertHtml( " &nbsp;"
+                    "<span style='background:rgba(0,0,0,30);border-radius:4px'>"
+                    "&nbsp;&hellip;&nbsp;"
+                    "</span>" );
+}
+
 } // namespace
 
-ItemText::ItemText(const QString &text, bool isRichText, int maxLines, int maximumHeight, QWidget *parent)
-    : QTextBrowser(parent)
+ItemText::ItemText(const QString &text, const QString &richText, int maxLines, int maxLineLength, int maximumHeight, QWidget *parent)
+    : QTextEdit(parent)
     , ItemWidget(this)
     , m_textDocument()
     , m_maximumHeight(maximumHeight)
@@ -86,47 +116,63 @@ ItemText::ItemText(const QString &text, bool isRichText, int maxLines, int maxim
 
     setReadOnly(true);
     setUndoRedoEnabled(false);
-    setOpenExternalLinks(true);
+    setTextInteractionFlags(
+                textInteractionFlags() | Qt::LinksAccessibleByMouse);
 
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     setFrameStyle(QFrame::NoFrame);
 
     setContextMenuPolicy(Qt::NoContextMenu);
 
-    viewport()->installEventFilter(this);
+    if ( !richText.isEmpty() ) {
+        m_textDocument.setHtml(richText);
+        // Use plain text instead if rendering HTML fails or result is empty.
+        m_isRichText = !m_textDocument.isEmpty();
+    }
 
-    if (isRichText)
-        m_textDocument.setHtml( text.left(defaultMaxBytes) );
-    else
-        m_textDocument.setPlainText( text.left(defaultMaxBytes) );
+    if (!m_isRichText)
+        m_textDocument.setPlainText(text);
 
     m_textDocument.setDocumentMargin(0);
 
-    setProperty("CopyQ_no_style", isRichText);
-
     if (maxLines > 0) {
         QTextBlock block = m_textDocument.findBlockByLineNumber(maxLines);
         if (block.isValid()) {
             QTextCursor tc(&m_textDocument);
             tc.setPosition(block.position() - 1);
             tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+
+            m_elidedFragment = tc.selection();
             tc.removeSelectedText();
-            tc.insertHtml( " &nbsp;"
-                           "<span style='background:rgba(0,0,0,30);border-radius:4px'>"
-                           "&nbsp;&hellip;&nbsp;"
-                           "</span>");
+
+            m_ellipsisPosition = tc.position();
+            insertEllipsis(&tc);
+        }
+    }
+
+    if (maxLineLength > 0) {
+        for ( auto block = m_textDocument.begin(); block.isValid(); block = block.next() ) {
+            if ( block.length() > maxLineLength ) {
+                QTextCursor tc(&m_textDocument);
+                tc.setPosition(block.position() + maxLineLength);
+                tc.setPosition(block.position() + block.length() - 1, QTextCursor::KeepAnchor);
+                insertEllipsis(&tc);
+            }
         }
     }
 
     setDocument(&m_textDocument);
+
+    connect( this, SIGNAL(selectionChanged()),
+             this, SLOT(onSelectionChanged()) );
 }
 
 void ItemText::highlight(const QRegExp &re, const QFont &highlightFont, const QPalette &highlightPalette)
 {
-    QList<QTextBrowser::ExtraSelection> selections;
+    QList<QTextEdit::ExtraSelection> selections;
 
     if ( !re.isEmpty() ) {
-        QTextBrowser::ExtraSelection selection;
+        QTextEdit::ExtraSelection selection;
         selection.format.setBackground( highlightPalette.base() );
         selection.format.setForeground( highlightPalette.text() );
         selection.format.setFont(highlightFont);
@@ -157,15 +203,27 @@ void ItemText::highlight(const QRegExp &re, const QFont &highlightFont, const QP
     update();
 }
 
-void ItemText::updateSize(const QSize &maximumSize)
+void ItemText::updateSize(QSize maximumSize, int idealWidth)
 {
-    const int w = maximumSize.width();
     const int scrollBarWidth = verticalScrollBar()->isVisible() ? verticalScrollBar()->width() : 0;
     setMaximumHeight( maximumSize.height() );
-    setFixedWidth(w);
-    m_textDocument.setTextWidth(w - scrollBarWidth);
+    setFixedWidth(idealWidth);
+    m_textDocument.setTextWidth(idealWidth - scrollBarWidth);
 
-    const int h = m_textDocument.size().height();
+    QTextOption option = m_textDocument.defaultTextOption();
+    const QTextOption::WrapMode wrapMode = maximumSize.width() > idealWidth
+            ? QTextOption::NoWrap : QTextOption::WrapAtWordBoundaryOrAnywhere;
+    if (wrapMode != option.wrapMode()) {
+        option.setWrapMode(wrapMode);
+        m_textDocument.setDefaultTextOption(option);
+    }
+
+    const QRectF rect = m_textDocument.documentLayout()->frameBoundingRect(m_textDocument.rootFrame());
+    setFixedWidth( static_cast<int>(rect.right()) );
+
+    QTextCursor tc(&m_textDocument);
+    tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+    const auto h = static_cast<int>( cursorRect(tc).bottom() + 4 * logicalDpiY() / 96.0 );
     setFixedHeight(0 < m_maximumHeight && m_maximumHeight < h ? m_maximumHeight : h);
 }
 
@@ -174,26 +232,78 @@ bool ItemText::eventFilter(QObject *, QEvent *event)
     return ItemWidget::filterMouseEvents(this, event);
 }
 
+QMimeData *ItemText::createMimeDataFromSelection() const
+{
+    const auto data = QTextEdit::createMimeDataFromSelection();
+    if (!data)
+        return nullptr;
+
+    // Copy only plain text if rich text is not available.
+    if (!m_isRichText) {
+        const auto text = data->text();
+        data->clear();
+        data->setText(text);
+    }
+
+    const auto owner = qApp->property("CopyQ_session_name").toString();
+    data->setData( mimeOwner, owner.toUtf8() );
+
+    return data;
+}
+
+void ItemText::onSelectionChanged()
+{
+    // Expand the ellipsis if selected.
+    if ( m_ellipsisPosition == -1 || textCursor().selectionEnd() <= m_ellipsisPosition )
+        return;
+
+    QTextCursor tc(&m_textDocument);
+    tc.setPosition(m_ellipsisPosition);
+    m_ellipsisPosition = -1;
+    tc.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+
+    tc.insertFragment(m_elidedFragment);
+    m_elidedFragment = QTextDocumentFragment();
+}
+
 ItemTextLoader::ItemTextLoader()
 {
 }
 
-ItemTextLoader::~ItemTextLoader()
-{
-}
+ItemTextLoader::~ItemTextLoader() = default;
 
-ItemWidget *ItemTextLoader::create(const QModelIndex &index, QWidget *parent) const
+ItemWidget *ItemTextLoader::create(const QModelIndex &index, QWidget *parent, bool preview) const
 {
+    if ( index.data(contentType::isHidden).toBool() )
+        return nullptr;
+
+    QString richText;
+    const bool isRichText = m_settings.value(optionUseRichText, true).toBool()
+            && getRichText(index, &richText);
+
     QString text;
-    bool isRichText = m_settings.value(optionUseRichText, true).toBool()
-            && getRichText(index, &text);
+    const bool isPlainText = getText(index, &text);
 
-    if ( !isRichText && !getText(index, &text) )
-        return NULL;
+    if (!isRichText && !isPlainText)
+        return nullptr;
 
-    const int maxLines = m_settings.value(optionMaximumLines, 0).toInt();
-    const int maxHeight = m_settings.value(optionMaximumHeight, 0).toInt();
-    return new ItemText(text, isRichText, maxLines, maxHeight, parent);
+    richText = normalizeText(richText);
+    text = normalizeText(text);
+
+    ItemText *item = nullptr;
+    // Always limit text size for performance reasons.
+    if (preview) {
+        item = new ItemText(text, richText, maxLineCountInPreview, maxLineLengthInPreview, 0, parent);
+    } else {
+        int maxLines = m_settings.value(optionMaximumLines, maxLineCount).toInt();
+        if (maxLines <= 0 || maxLines > maxLineCount)
+            maxLines = maxLineCount;
+        const int maxHeight = m_settings.value(optionMaximumHeight, 0).toInt();
+        item = new ItemText(text, richText, maxLines, maxLineLength, maxHeight, parent);
+        item->viewport()->installEventFilter(item);
+    }
+
+    return item;
 }
 
 QStringList ItemTextLoader::formatsToSave() const
