@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,6 +21,7 @@
 
 #include "common/log.h"
 #include "common/settings.h"
+#include "common/textdata.h"
 #include "platform/platformnativeinterface.h"
 #ifdef Q_OS_UNIX
 #   include "platform/unix/unixsignalhandler.h"
@@ -33,15 +34,7 @@
 #include <QTranslator>
 #include <QVariant>
 
-#ifndef COPYQ_TRANSLATION_PREFIX
-#   ifdef Q_OS_WIN
-#       define COPYQ_TRANSLATION_PREFIX QCoreApplication::applicationDirPath() + "/translations"
-#   elif defined(Q_OS_MAC)
-#       define COPYQ_TRANSLATION_PREFIX QCoreApplication::applicationDirPath() + "/../Resources/translations"
-#   else
-#       define COPYQ_TRANSLATION_PREFIX ""
-#   endif
-#endif
+#include <memory>
 
 namespace {
 
@@ -51,7 +44,11 @@ namespace {
  */
 bool isTesting()
 {
+#if QT_VERSION < 0x050100
     return !qgetenv("COPYQ_TEST_ID").isEmpty();
+#else
+    return !qEnvironmentVariableIsEmpty("COPYQ_TEST_ID");
+#endif
 }
 
 /**
@@ -69,62 +66,16 @@ void initTests()
     QCoreApplication::setOrganizationName(session);
     QCoreApplication::setApplicationName(session);
 
-    const QString testId = QString::fromUtf8( qgetenv("COPYQ_TEST_ID") );
+    const QString testId = getTextData( qgetenv("COPYQ_TEST_ID") );
     qApp->setProperty("CopyQ_test_id", testId);
 }
-
-/**
- * Read base64-encoded settings from "COPYQ_TEST_SETTINGS" enviroment variable if not empty.
- *
- * The settings are initially taken from "CopyQ_test_settings" property of test object returned by
- * ItemLoaderInterface::tests().
- *
- * This function does nothing if isTesting() returns false.
- */
-void initTestsSettings()
-{
-    if ( !isTesting() )
-        return;
-
-    const QByteArray settingsData = qgetenv("COPYQ_TEST_SETTINGS");
-    if ( settingsData.isEmpty() )
-        return;
-
-    // Reset settings on first run of each test case.
-    Settings settings;
-    settings.clear();
-
-    QVariant testSettings;
-    const QByteArray data = QByteArray::fromBase64(settingsData);
-    QDataStream input(data);
-    input >> testSettings;
-    const QVariantMap testSettingsMap = testSettings.toMap();
-
-    const QString testId = qApp->property("CopyQ_test_id").toString();
-    bool pluginsTest = testId != "CORE";
-
-    if (pluginsTest) {
-        settings.beginGroup("Plugins");
-        settings.beginGroup(testId);
-    }
-
-    foreach (const QString &key, testSettingsMap.keys())
-        settings.setValue(key, testSettingsMap[key]);
-
-    if (pluginsTest) {
-        settings.endGroup();
-        settings.endGroup();
-    }
-
-    settings.setValue("CopyQ_test_id", testId);
-}
 #endif // HAS_TESTS
 
 void installTranslator(const QString &filename, const QString &directory)
 {
-    QScopedPointer<QTranslator> translator( new QTranslator(qApp) );
+    std::unique_ptr<QTranslator> translator( new QTranslator(qApp) );
     if ( translator->load(filename, directory) )
-        QCoreApplication::installTranslator(translator.take());
+        QCoreApplication::installTranslator(translator.release());
 }
 
 void installTranslator()
@@ -135,30 +86,30 @@ void installTranslator()
         return;
 #endif
 
-    QString locale = Settings().value("Options/language").toString();
+    QString locale = QSettings().value("Options/language").toString();
     if (locale.isEmpty())
         locale = QLocale::system().name();
 
-    QStringList translationDirectories;
 #ifdef COPYQ_TRANSLATION_PREFIX
-    translationDirectories.prepend(COPYQ_TRANSLATION_PREFIX);
+    const QString translationPrefix = COPYQ_TRANSLATION_PREFIX;
+#else
+    const QString translationPrefix = createPlatformNativeInterface()->translationPrefix();
 #endif
 
+    QStringList translationDirectories;
+    translationDirectories.prepend(translationPrefix);
+
     // 1. Qt translations
-#ifdef COPYQ_TRANSLATION_PREFIX
-    installTranslator("qt_" + locale, COPYQ_TRANSLATION_PREFIX);
-#endif
+    installTranslator("qt_" + locale, translationPrefix);
     installTranslator("qt_" + locale, QLibraryInfo::location(QLibraryInfo::TranslationsPath));
 
     // 2. installed translations
-#ifdef COPYQ_TRANSLATION_PREFIX
-    installTranslator("copyq_" + locale, COPYQ_TRANSLATION_PREFIX);
-#endif
+    installTranslator("copyq_" + locale, translationPrefix);
 
     // 3. custom translations
     const QByteArray customPath = qgetenv("COPYQ_TRANSLATION_PREFIX");
     if ( !customPath.isEmpty() ) {
-        const QString customDir = QDir::fromNativeSeparators( QString::fromUtf8(customPath) );
+        const QString customDir = QDir::fromNativeSeparators( getTextData(customPath) );
         installTranslator("copyq_" + locale, customDir);
         translationDirectories.prepend(customDir);
     }
@@ -177,9 +128,13 @@ void installTranslator()
 
 } // namespace
 
-App::App(QCoreApplication *application, const QString &sessionName)
+App::App(
+        const QString &threadName,
+        QCoreApplication *application,
+        const QString &sessionName)
     : m_app(application)
     , m_exitCode(0)
+    , m_started(false)
     , m_closed(false)
 {
     QString session("copyq");
@@ -194,28 +149,35 @@ App::App(QCoreApplication *application, const QString &sessionName)
     QCoreApplication::setOrganizationName(session);
     QCoreApplication::setApplicationName(session);
 
+    if ( !threadName.isEmpty() )
+        setCurrentThreadName(threadName);
+
 #ifdef HAS_TESTS
     initTests();
 #endif
 
 #ifdef Q_OS_UNIX
-    if ( !UnixSignalHandler::create(m_app.data()) )
+    if ( !UnixSignalHandler::create(m_app) )
         log( QString("Failed to create handler for Unix signals!"), LogError );
 #endif
-
-    createPlatformNativeInterface()->loadSettings();
-
-    Settings(); // restore settings
-
-    installTranslator();
-
-#ifdef HAS_TESTS
-    initTestsSettings();
-#endif
 }
 
 App::~App()
 {
+    QCoreApplication::processEvents();
+    App::exit();
+    delete m_app;
+}
+
+void App::restoreSettings(bool canModifySettings)
+{
+    m_app->setProperty("CopyQ_server", canModifySettings);
+
+    createPlatformNativeInterface()->loadSettings();
+
+    Settings::restore();
+
+    installTranslator();
 }
 
 int App::exec()
@@ -225,6 +187,8 @@ int App::exec()
         return m_exitCode;
     }
 
+    m_started = true;
+
     return m_app->exec();
 }
 
@@ -233,6 +197,9 @@ void App::exit(int exitCode)
     if ( wasClosed() )
         return;
 
+    if (!m_started)
+        ::exit(exitCode);
+
     QCoreApplication::exit(exitCode);
     m_exitCode = exitCode;
     m_closed = true;
