@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -32,10 +32,11 @@
 #include <QVariantMap>
 #include <QWidget>
 
-class Arguments;
 class ClientSocket;
+class ItemFactory;
 class RemoteProcess;
 class QxtGlobalShortcut;
+class QApplication;
 class QSessionManager;
 
 /**
@@ -50,7 +51,7 @@ class ClipboardServer : public QObject, public App
     Q_OBJECT
 
 public:
-    ClipboardServer(int &argc, char **argv, const QString &sessionName = QString());
+    ClipboardServer(QApplication *app, const QString &sessionName);
     ~ClipboardServer();
 
     /** Stop monitor application. */
@@ -71,7 +72,7 @@ public:
 
 public slots:
     /** Load @a item data to clipboard. */
-    void changeClipboard(const QVariantMap &data);
+    void changeClipboard(const QVariantMap &data, QClipboard::Mode mode);
 
     /** Load settings. */
     void loadSettings();
@@ -83,22 +84,20 @@ signals:
     void terminateClientThreads();
 
 protected:
-    bool eventFilter(QObject *object, QEvent *ev);
+    bool eventFilter(QObject *object, QEvent *ev) override;
 
 private slots:
     /**
      * Execute command in different thread.
      */
-    void doCommand(
-            const Arguments &args, //!< Contains command and its arguments.
-            ClientSocket *client = NULL //!< For sending responses.
+    void doCommand(const ClientSocketPtr &client = nullptr //!< For sending responses.
             );
 
     /** New message from monitor process. */
     void newMonitorMessage(const QByteArray &message);
 
     /** An error occurred on monitor connection. */
-    void monitorConnectionError();
+    void monitorConnectionError(const QString &error);
 
     /** Shortcut was pressed on host system. */
     void shortcutActivated(QxtGlobalShortcut *shortcut);
@@ -115,22 +114,28 @@ private slots:
      */
     void onCommitData(QSessionManager &sessionManager);
 
+    /**
+     * Save session name for next time.
+     */
+    void onSaveState(QSessionManager &sessionManager);
+
     /** Quit application, but ask to cancel exit if there are any active commands. */
     void maybeQuit();
 
-    void onIgnoreKeysTimeout();
-
 private:
     /** Ask to cancel application exit if there are any active commands. */
     bool askToQuit();
 
+    void terminateThreads();
+
+    bool hasRunningCommands() const;
+
     MainWindow* m_wnd;
     RemoteProcess *m_monitor;
-    bool m_checkclip;
     QMap<QxtGlobalShortcut*, Command> m_shortcutActions;
-    QWidget m_shortcutBlocker;
     QThreadPool m_clientThreads;
     QTimer m_ignoreKeysTimer;
+    ItemFactory *m_itemFactory;
 };
 
 #endif // CLIPBOARDSERVER_H
