@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -27,3 +27,12 @@ const char mimeItems[] = COPYQ_MIME_PREFIX "item";
 const char mimeItemNotes[] = COPYQ_MIME_PREFIX "item-notes";
 const char mimeOwner[] = COPYQ_MIME_PREFIX "owner";
 const char mimeClipboardMode[] = COPYQ_MIME_PREFIX "clipboard-mode";
+const char mimeCurrentTab[] = COPYQ_MIME_PREFIX "current-tab";
+const char mimeSelectedItems[] = COPYQ_MIME_PREFIX "selected-items";
+const char mimeCurrentItem[] = COPYQ_MIME_PREFIX "current-item";
+const char mimeHidden[] = COPYQ_MIME_PREFIX "hidden";
+const char mimeShortcut[] = COPYQ_MIME_PREFIX "shortcut";
+const char mimeColor[] = COPYQ_MIME_PREFIX "color";
+const char mimeOutputTab[] = COPYQ_MIME_PREFIX "output-tab";
+const char mimeSyncToClipboard[] = COPYQ_MIME_PREFIX "sync-to-clipboard";
+const char mimeSyncToSelection[] = COPYQ_MIME_PREFIX "sync-to-selection";
