@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -29,5 +29,14 @@ extern const char mimeItems[];
 extern const char mimeItemNotes[];
 extern const char mimeOwner[];
 extern const char mimeClipboardMode[];
+extern const char mimeCurrentTab[];
+extern const char mimeSelectedItems[];
+extern const char mimeCurrentItem[];
+extern const char mimeHidden[];
+extern const char mimeShortcut[];
+extern const char mimeColor[];
+extern const char mimeOutputTab[];
+extern const char mimeSyncToClipboard[];
+extern const char mimeSyncToSelection[];
 
 #endif // MIMETYPES_H
