@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -44,9 +44,12 @@ bool needsUpdate(const Settings &newSettings, const QSettings &oldSettings)
 
 void copySettings(const QSettings &from, QSettings *to)
 {
+    Q_ASSERT(from.group().isEmpty());
+    Q_ASSERT(to->group().isEmpty());
+
     to->clear();
 
-    foreach ( const QString &key, from.allKeys() )
+    for ( const auto &key : from.allKeys() )
         to->setValue(key, from.value(key));
 
     to->sync();
@@ -80,37 +83,59 @@ bool Settings::isEmpty(const QSettings &settings)
     return settings.childGroups().isEmpty();
 }
 
+bool Settings::canModifySettings()
+{
+    Q_ASSERT(qApp);
+    return qApp->property("CopyQ_server").toBool();
+}
+
 Settings::Settings()
-    : QSettings(
+    : m_settings(
           QSettings::defaultFormat(),
           QSettings::UserScope,
           QCoreApplication::organizationName(),
           QCoreApplication::applicationName() + "-bak" )
+    , m_changed(false)
 {
     Q_ASSERT( isMainThread() );
-
-    if ( isLastSaveUnfinished() ) {
-        log("Restoring application settings", LogWarning);
-
-        if ( isEmpty(*this) ) {
-            log("Cannot restore application settings", LogError);
-        } else {
-            QSettings settings;
-            copySettings(*this, &settings);
-        }
-
-        endSave();
-    } else if ( needsUpdate(*this, QSettings()) ) {
-        copySettings(QSettings(), this);
-    }
 }
 
 Settings::~Settings()
 {
-    sync();
+    // Only main application is allowed to change settings.
+    if (canModifySettings() && m_changed) {
+        m_settings.sync();
 
-    beginSave();
-    QSettings settings;
-    copySettings(*this, &settings);
-    endSave();
+        beginSave();
+        QSettings to;
+        while ( !m_settings.group().isEmpty() )
+            m_settings.endGroup();
+        copySettings(m_settings, &to);
+        endSave();
+    }
+}
+
+void Settings::restore()
+{
+    if (!canModifySettings())
+        return;
+
+    Settings appSettings;
+
+    if ( isLastSaveUnfinished() ) {
+        log("Restoring application settings", LogWarning);
+
+        if ( appSettings.isEmpty() ) {
+            log("Cannot restore application settings", LogError);
+        } else {
+            QSettings settings;
+            copySettings(appSettings.m_settings, &settings);
+        }
+
+        endSave();
+    } else {
+        const QSettings settings;
+        if ( needsUpdate(appSettings, settings) )
+            copySettings(settings, &appSettings.m_settings);
+    }
 }
