@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,11 +19,14 @@
 
 #include "actionhandler.h"
 
+#include "common/appconfig.h"
 #include "common/action.h"
 #include "common/common.h"
 #include "common/contenttype.h"
+#include "common/display.h"
 #include "common/log.h"
 #include "common/mimetypes.h"
+#include "common/textdata.h"
 #include "gui/actiondialog.h"
 #include "gui/processmanagerdialog.h"
 #include "gui/clipboardbrowser.h"
@@ -33,26 +36,26 @@
 #include <QDateTime>
 #include <QModelIndex>
 
+#include <cmath>
+
 ActionHandler::ActionHandler(MainWindow *mainWindow)
     : QObject(mainWindow)
     , m_wnd(mainWindow)
-    , m_lastActionId(0)
-    , m_actionData()
+    , m_actionCounter(0)
     , m_activeActionDialog(new ProcessManagerDialog(mainWindow))
-    , m_hasRunningAction(false)
 {
     Q_ASSERT(mainWindow);
 }
 
 ActionDialog *ActionHandler::createActionDialog(const QStringList &tabs)
 {
-    ActionDialog *actionDialog = new ActionDialog(m_wnd);
+    auto actionDialog = new ActionDialog(m_wnd);
     actionDialog->setAttribute(Qt::WA_DeleteOnClose, true);
     actionDialog->setOutputTabs(tabs, QString());
     actionDialog->setCommand(m_lastActionDialogCommand);
 
-    connect( actionDialog, SIGNAL(accepted(Action*,QVariantMap)),
-             this, SLOT(action(Action*,QVariantMap)) );
+    connect( actionDialog, SIGNAL(accepted(Action*)),
+             this, SLOT(action(Action*)) );
     connect( actionDialog, SIGNAL(closed(ActionDialog*)),
              this, SLOT(actionDialogClosed(ActionDialog*)) );
 
@@ -61,14 +64,7 @@ ActionDialog *ActionHandler::createActionDialog(const QStringList &tabs)
 
 bool ActionHandler::hasRunningAction() const
 {
-    return !m_actionData.isEmpty();
-}
-
-QByteArray ActionHandler::getActionData(const QByteArray actionId, const QString &format) const
-{
-    const QVariantMap dataMap = m_actionData.value(actionId);
-    return format == "?" ? QStringList(dataMap.keys()).join("\n").toUtf8() + '\n'
-                         : dataMap.value(format).toByteArray();
+    return m_actionCounter > 0;
 }
 
 void ActionHandler::showProcessManagerDialog()
@@ -81,20 +77,35 @@ void ActionHandler::addFinishedAction(const QString &name)
     m_activeActionDialog->actionFinished(name);
 }
 
-void ActionHandler::action(Action *action, const QVariantMap &data)
+QVariantMap ActionHandler::actionData(int id) const
+{
+    const auto action = m_actions.value(id);
+    return action ? action->data() : QVariantMap();
+}
+
+void ActionHandler::setActionData(int id, const QVariantMap &data)
+{
+    const auto action = m_actions.value(id);
+    if (action)
+        action->setData(data);
+}
+
+void ActionHandler::action(Action *action)
 {
     action->setParent(this);
 
+    const auto id = ++m_lastActionId;
+    action->setId(id);
+    m_actions.insert(id, action);
+
     m_lastAction = action;
 
-    connect( action, SIGNAL(newItems(QStringList, QString)),
-             this, SLOT(addItems(QStringList, QString)) );
-    connect( action, SIGNAL(newItems(QStringList, QModelIndex)),
-             this, SLOT(addItems(QStringList, QModelIndex)) );
-    connect( action, SIGNAL(newItem(QByteArray, QString, QString)),
-             this, SLOT(addItem(QByteArray, QString, QString)) );
-    connect( action, SIGNAL(newItem(QByteArray, QString, QModelIndex)),
-             this, SLOT(addItem(QByteArray, QString, QModelIndex)) );
+    connect( action, SIGNAL(newItems(QStringList,QString,QString)),
+             this, SLOT(addItems(QStringList,QString,QString)) );
+    connect( action, SIGNAL(newItem(QByteArray,QString,QString)),
+             this, SLOT(addItem(QByteArray,QString,QString)) );
+    connect( action, SIGNAL(changeItem(QByteArray,QString,QModelIndex)),
+             this, SLOT(changeItem(QByteArray,QString,QModelIndex)) );
     connect( action, SIGNAL(actionStarted(Action*)),
              this, SLOT(actionStarted(Action*)) );
     connect( action, SIGNAL(actionFinished(Action*)),
@@ -102,11 +113,7 @@ void ActionHandler::action(Action *action, const QVariantMap &data)
     connect( action, SIGNAL(actionError(Action*)),
              this, SLOT(closeAction(Action*)) );
 
-    // Set data for action (can be accessed using 'data' command).
-    const QByteArray actionId = QByteArray::number(++m_lastActionId);
-    qputenv("COPYQ_ACTION_ID", actionId);
-    m_actionData.insert(actionId, data);
-    action->setProperty("COPYQ_ACTION_ID", actionId);
+    ++m_actionCounter;
 
     if ( !action->outputFormat().isEmpty() && action->outputTab().isEmpty() )
         action->setOutputTab(m_currentTabName);
@@ -119,27 +126,27 @@ void ActionHandler::action(Action *action, const QVariantMap &data)
 void ActionHandler::actionStarted(Action *action)
 {
     m_activeActionDialog->actionStarted(action);
-
-    if (!m_hasRunningAction) {
-        m_hasRunningAction = true;
-        emit hasRunningActionChanged();
-    }
+    emit runningActionsCountChanged();
 }
 
 void ActionHandler::closeAction(Action *action)
 {
+    m_actions.remove(action->id());
+
     QString msg;
 
     QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::Information;
 
-    if ( action->actionFailed() || action->exitStatus() != QProcess::NormalExit ) {
-        msg += tr("Error: %1\n").arg(action->errorString()) + action->errorOutput();
+    if ( action->actionFailed() ) {
+        msg = tr("Error: %1\n").arg(action->errorString()) + action->errorOutput();
         icon = QSystemTrayIcon::Critical;
     } else if ( action->exitCode() != 0 ) {
-        msg += tr("Exit code: %1\n").arg(action->exitCode()) + action->errorOutput();
-        icon = QSystemTrayIcon::Warning;
+        if ( !action->ignoreExitCode() ) {
+            msg = tr("Exit code: %1\n").arg(action->exitCode()) + "\n" + action->errorOutput();
+            icon = QSystemTrayIcon::Warning;
+        }
     } else if ( !action->inputFormats().isEmpty() ) {
-        QModelIndex index = action->index();
+        const QModelIndex index = action->index();
         ClipboardBrowser *c = m_wnd->browserForItem(index);
         if (c) {
             QStringList removeFormats = action->inputFormats();
@@ -150,17 +157,35 @@ void ActionHandler::closeAction(Action *action)
         }
     }
 
-    if ( !msg.isEmpty() )
-        m_wnd->showMessage( tr("Command %1").arg(quoteString(action->command())), msg, icon );
+    if ( !msg.isEmpty() ) {
+        const int maxWidthPoints =
+                AppConfig().option<Config::notification_maximum_width>();
+        const QString command = action->command()
+                .replace("copyq eval --", "copyq:");
+        const QString name = action->name().isEmpty()
+                ? QString(command).replace('\n', " ")
+                : action->name();
+        const QString format = tr("Command %1").arg(quoteString("%1"));
+        const QString title = elideText(name, QFont(), format, pointsToPixels(maxWidthPoints));
+
+        // Print command with line numbers.
+        int lineNumber = 0;
+        const auto lines = command.split("\n");
+        const auto lineNumberWidth = std::log10(lines.size()) + 1;
+        for (const auto &line : lines)
+            msg.append(QString("\n%1. %2").arg(++lineNumber, lineNumberWidth).arg(line));
+
+        log(title + "\n" + msg);
+        m_wnd->showMessage(title, msg, icon);
+    }
 
     m_activeActionDialog->actionFinished(action);
-    m_actionData.remove( action->property("COPYQ_ACTION_ID").toByteArray() );
+    Q_ASSERT(m_actionCounter > 0);
+    --m_actionCounter;
+
+    emit runningActionsCountChanged();
+
     action->deleteLater();
-
-    m_hasRunningAction = hasRunningAction();
-
-    if (!m_hasRunningAction)
-        emit hasRunningActionChanged();
 }
 
 void ActionHandler::actionDialogClosed(ActionDialog *dialog)
@@ -168,47 +193,35 @@ void ActionHandler::actionDialogClosed(ActionDialog *dialog)
     m_lastActionDialogCommand = dialog->command();
 }
 
-void ActionHandler::addItems(const QStringList &items, const QString &tabName)
+void ActionHandler::addItems(const QStringList &items, const QString &format, const QString &tabName)
 {
-    ClipboardBrowser *c = tabName.isEmpty() ? m_wnd->browser() : m_wnd->createTab(tabName);
-    ClipboardBrowser::Lock lock(c);
-    foreach (const QString &item, items)
-        c->add(item);
+    ClipboardBrowser *c = tabName.isEmpty() ? m_wnd->browser() : m_wnd->tab(tabName);
+    for (const auto &item : items)
+        c->add( createDataMap(format, item) );
 
     if (m_lastAction) {
         if (m_lastAction == sender())
             c->setCurrent(items.size() - 1);
-        m_lastAction = NULL;
+        m_lastAction = nullptr;
     }
 }
 
-void ActionHandler::addItems(const QStringList &items, const QModelIndex &index)
-{
-    ClipboardBrowser *c = m_wnd->browserForItem(index);
-    if (c == NULL)
-        return;
-
-    QVariantMap dataMap;
-    dataMap.insert( mimeText, items.join(QString()).toUtf8() );
-    c->model()->setData(index, dataMap, contentType::updateData);
-}
-
 void ActionHandler::addItem(const QByteArray &data, const QString &format, const QString &tabName)
 {
-    ClipboardBrowser *c = tabName.isEmpty() ? m_wnd->browser() : m_wnd->createTab(tabName);
+    ClipboardBrowser *c = tabName.isEmpty() ? m_wnd->browser() : m_wnd->tab(tabName);
     c->add( createDataMap(format, data) );
 
     if (m_lastAction) {
         if (m_lastAction == sender())
             c->setCurrent(0);
-        m_lastAction = NULL;
+        m_lastAction = nullptr;
     }
 }
 
-void ActionHandler::addItem(const QByteArray &data, const QString &format, const QModelIndex &index)
+void ActionHandler::changeItem(const QByteArray &data, const QString &format, const QModelIndex &index)
 {
     ClipboardBrowser *c = m_wnd->browserForItem(index);
-    if (c == NULL)
+    if (c == nullptr)
         return;
 
     QVariantMap dataMap;
