@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -53,8 +53,6 @@ public:
 
     bool hasRunningAction() const;
 
-    QByteArray getActionData(const QByteArray actionId, const QString &format) const;
-
     /** Open dialog with active commands. */
     void showProcessManagerDialog();
 
@@ -62,13 +60,16 @@ public:
 
     void addFinishedAction(const QString &name);
 
+    QVariantMap actionData(int id) const;
+    void setActionData(int id, const QVariantMap &data);
+
 public slots:
     /** Execute action. */
-    void action(Action *action, const QVariantMap &data);
+    void action(Action *action);
 
 signals:
-    /** Emitted when last action finishes or first action starts. */
-    void hasRunningActionChanged();
+    /** Emitted new action starts or ends. */
+    void runningActionsCountChanged();
 
 private slots:
     /** Called after action was started (creates menu item to kill it). */
@@ -79,20 +80,19 @@ private slots:
 
     void actionDialogClosed(ActionDialog *dialog);
 
-    void addItems(const QStringList &items, const QString &tabName);
-    void addItems(const QStringList &items, const QModelIndex &index);
+    void addItems(const QStringList &items, const QString &format, const QString &tabName);
     void addItem(const QByteArray &data, const QString &format, const QString &tabName);
-    void addItem(const QByteArray &data, const QString &format, const QModelIndex &index);
+    void changeItem(const QByteArray &data, const QString &format, const QModelIndex &index);
 
 private:
     MainWindow *m_wnd;
     QPointer<Action> m_lastAction;
-    int m_lastActionId;
-    QMap<QByteArray, QVariantMap> m_actionData;
+    int m_actionCounter;
     ProcessManagerDialog *m_activeActionDialog;
     QString m_currentTabName;
     Command m_lastActionDialogCommand;
-    bool m_hasRunningAction;
+    QHash<int, Action*> m_actions;
+    int m_lastActionId = -1;
 };
 
 #endif // ACTIONHANDLER_H
