@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,78 +21,52 @@
 #define CLIPBOARDBROWSER_H
 
 #include "common/command.h"
+#include "gui/clipboardbrowsershared.h"
 #include "gui/configtabshortcuts.h"
+#include "gui/theme.h"
+#include "item/clipboardmodel.h"
+#include "item/itemdelegate.h"
 #include "item/itemwidget.h"
 
 #include <QListView>
 #include <QPointer>
-#include <QScopedPointer>
-#include <QSharedPointer>
 #include <QTimer>
 #include <QVariantMap>
 
-class ClipboardModel;
-class ItemDelegate;
+#include <memory>
+
 class ItemEditorWidget;
+class ItemFactory;
 class QProgressBar;
 class QPushButton;
 
-enum SelectAction {
-    NoSelectAction,
-    MoveToTop = 1 << 1,
-    MoveToClipboard = 1 << 2
-};
-Q_DECLARE_FLAGS(SelectActions, SelectAction)
-Q_DECLARE_OPERATORS_FOR_FLAGS(SelectActions)
-
-struct ClipboardBrowserShared {
-    ClipboardBrowserShared();
-
-    void loadFromConfiguration();
-
-    QString editor;
-    int maxItems;
-    bool textWrap;
-    bool viMode;
-    bool saveOnReturnKey;
-    bool moveItemOnReturnKey;
-    int minutesToExpire;
-};
-typedef QSharedPointer<ClipboardBrowserShared> ClipboardBrowserSharedPtr;
-
-QVariantMap itemData(const QModelIndex &index);
-
 /** List view of clipboard items. */
 class ClipboardBrowser : public QListView
 {
     Q_OBJECT
 
     public:
-        /**
-         * Disable ClipboardBrowser's automatic visual and clipboard updates.
-         */
-        class Lock {
-            public:
-                Lock(ClipboardBrowser *self) : c(self) { c->lock(); }
-                ~Lock() { if (!c.isNull()) c->unlock(); }
+        ClipboardBrowser(
+                const QString &tabName,
+                const ClipboardBrowserSharedPtr &sharedData,
+                QWidget *parent = nullptr);
 
-            private:
-                QPointer<ClipboardBrowser> c;
-        };
-
-        explicit ClipboardBrowser(QWidget *parent = NULL,
-                                  const ClipboardBrowserSharedPtr &sharedData = ClipboardBrowserSharedPtr());
         /** Close all external editors and save items if needed. */
         ~ClipboardBrowser();
-        /** Load settings. */
-        void loadSettings();
 
         /**
-         * Select item with given @a hash and move it to clipboard.
+         * Move item with given @a hash to clipboard.
          *
          * @return true only if item exists
          */
-        bool select(uint itemHash, SelectActions selectActions);
+        bool moveToClipboard(uint itemHash);
+
+        /**
+         * Move item with given @a hash to the top of the list.
+         *
+         * @return true only if item exists
+         */
+        bool moveToTop(uint itemHash);
 
         /** Sort selected items. */
         void sortItems(const QModelIndexList &indexes);
@@ -101,68 +75,50 @@ class ClipboardBrowser : public QListView
         void reverseItems(const QModelIndexList &indexes);
 
         /** Index of item in given row. */
-        QModelIndex index(int i) const { return model()->index(i,0); }
+        QModelIndex index(int i) const { return m.index(i,0); }
 
         /** Returns concatenation of selected items. */
         const QString selectedText() const;
 
-        /** Update clipboard content according to first item in list. */
-        void updateClipboard(int row = 0);
-
-        /** Invalidate item cache. */
-        void invalidateItemCache();
-
-        /** Toggle automatic clipboard update. */
-        void setAutoUpdate(bool update);
-
-        /** Return true if automatic clipboard update is on. */
-        bool autoUpdate() { return m_update; }
-
         /**
-         * Set ID. Used to save items. If ID is empty saving is disabled.
+         * Set tab name.
+         *
+         * This is ID used for saving items.
+         *
+         * If ID is empty saving is disabled.
          */
-        void setTabName(const QString &id);
+        void setTabName(const QString &tabName);
         const QString &tabName() const { return m_tabName; }
 
         /**
-         * Return true if editing is active.
+         * Return true only if the internal editor widget is open.
          */
-        bool editing() const;
+        bool isInternalEditorOpen() const;
 
         /**
-         * Return true if items are loaded (loadItems() called or tabName() is empty).
+         * Return true only if an external editor is open.
          */
-        bool isLoaded() const;
+        bool isExternalEditorOpen() const;
 
         /**
-         * Close editor if unless user don't want to discard changed (show message box).
+         * Close internal and external editor
+         * unless user don't want to discard changed (show message box).
          *
-         * @return true only if editor was closed
+         * @return true only if editors were closed
          */
-        bool maybeCloseEditor();
-
-        /**
-         * Handle key for Vi mode.
-         */
-        bool handleViKey(QKeyEvent *event);
-
-        /**
-         * Get data of selected item, NULL if none or multiple items selected.
-         */
-        QVariantMap getSelectedItemData() const;
+        bool maybeCloseEditors();
 
         /**
          * Override to disable default QAbstractItemView search.
          */
-        void keyboardSearch(const QString &) {}
+        void keyboardSearch(const QString &) override {}
 
-        /** Return true if user defined a selection and it shouldn't change programmatically. */
-        bool hasUserSelection() const;
+        QVariantMap copyIndex(const QModelIndex &index) const;
 
-        QVariantMap copyIndexes(const QModelIndexList &indexes, bool serializeItems = true) const;
+        QVariantMap copyIndexes(const QModelIndexList &indexes) const;
 
-        /** Remove items and return row number of last removed item. */
-        int removeIndexes(const QModelIndexList &indexes);
+        /** Remove items and return smallest row number (new current item if selection was removed). */
+        int removeIndexes(const QModelIndexList &indexes, QString *error = nullptr);
 
         /** Paste items. */
         void paste(const QVariantMap &data, int destinationRow);
@@ -170,6 +126,9 @@ class ClipboardBrowser : public QListView
         /** Render preview image with items. */
         QPixmap renderItemPreview(const QModelIndexList &indexes, int maxWidth, int maxHeight);
 
+        /** Removes items from end of list without notifying plugins. */
+        bool allocateSpaceForNewItems(int newItemCount);
+
         /** Add new item to the browser. */
         bool add(
                 const QString &txt, //!< Text of new item.
@@ -181,45 +140,35 @@ class ClipboardBrowser : public QListView
          * @a item is automatically deleted after it's no longer needed.
          */
         bool add(
-                const QVariantMap &item, //!< Data for new item.
+                const QVariantMap &data, //!< Data for new item.
                 int row = 0 //!< Target row for the new item (negative to append item).
                 );
 
+        /**
+         * Add item and remove duplicates.
+         */
+        void addUnique(const QVariantMap &data);
+
         /** Number of items in list. */
-        int length() const { return model()->rowCount(); }
+        int length() const { return m.rowCount(); }
 
         /** Receive key event. */
         void keyEvent(QKeyEvent *event) { keyPressEvent(event); }
         /** Move item to clipboard. */
         void moveToClipboard(const QModelIndex &ind);
-        /** Move item to clipboard. */
-        void moveToClipboard(int i);
+        /** Move items to clipboard. */
+        void moveToClipboard(const QModelIndexList &indexes);
         /** Show only items matching the regular expression. */
         void filterItems(const QRegExp &re);
-        /** Show all items. */
-        void clearFilter() { filterItems( QRegExp() ); }
         /** Open editor. */
-        bool openEditor(const QByteArray &data, const QString &mime = QString("text/plain"),
-                        const QString &editorCommand = QString());
+        bool openEditor(const QByteArray &textData, bool changeClipboard = false);
         /** Open editor for an item. */
         bool openEditor(const QModelIndex &index);
         /** Add items. */
         void addItems(const QStringList &items);
 
-        void removeRow(int row);
-
         /** Set current item. */
-        void setCurrent(
-                int row, //!< Row of the item.
-                bool cycle = false, //!< If true @a row is relative number of rows from top.
-                bool selection = false //!< Makes selection.
-                );
-
-        /**
-         * Load items from configuration even if model is disabled.
-         * @see loadItems
-         */
-        void loadItemsAgain();
+        void setCurrent(int row, bool keepSelection = false, bool setCurrentOnly = false);
 
         /**
          * Save items to configuration if needed.
@@ -236,20 +185,38 @@ class ClipboardBrowser : public QListView
          * Create and edit new item.
          */
         void editNew(
-                const QString &text = QString() //!< Text of new item.
+                const QString &text = QString(), //!< Text of new item.
+                bool changeClipboard = false //!< Change clipboard if item is modified.
                 );
 
         /** Edit item in given @a row. */
         void editRow(int row);
 
-    public slots:
+        void otherItemLoader(bool next);
+
+        void move(int key);
+
+        QWidget *currentItemPreview();
+
+        void findNext();
+
+        void findPrevious();
+
+        ItemWidget *itemWidget(const QModelIndex & index);
+
         /**
          * Load items from configuration.
          * This function does nothing if model is disabled (e.g. loading failed previously).
          * @see setID, saveItems, purgeItems
          */
-        void loadItems();
+        bool loadItems();
 
+        /**
+         * Return true only if row is filtered and should be hidden.
+         */
+        bool isFiltered(int row) const;
+
+    public slots:
         /**
          * Save items to configuration.
          * @see setID, loadItems, purgeItems
@@ -268,16 +235,6 @@ class ClipboardBrowser : public QListView
         /** Open editor with text of all selected items or for single selected item. */
         bool openEditor();
 
-        /**
-         * Select next item and copy it to clipboard.
-         */
-        void copyNextItemToClipboard();
-
-        /**
-         * Select previous item and copy it to clipboard.
-         */
-        void copyPreviousItemToClipboard();
-
         /** Remove selected unhidden items. */
         void remove();
 
@@ -285,10 +242,14 @@ class ClipboardBrowser : public QListView
         void showItemContent();
 
         /** Item modified in external editor. */
-        void itemModified(const QByteArray &bytes, const QString &mime);
+        void itemModified(const QByteArray &bytes, const QString &mime, const QModelIndex &index);
         /** Called if editor was closed. */
         void closeExternalEditor(QObject *editor);
 
+        void emitItemCount();
+
+        bool eventFilter(QObject *watched, QEvent *event) override;
+
     signals:
         /** Action dialog requested. */
         void requestActionDialog(const QVariantMap &data);
@@ -299,107 +260,90 @@ class ClipboardBrowser : public QListView
         /** Request clipboard change. */
         void changeClipboard(const QVariantMap &data);
 
-        /** Add item to another tab (invoked by an automatic command). */
-        void addToTab(const QVariantMap &data, const QString &tabName);
-
         /** Emitted on error. */
         void error(const QString &errorString);
 
+        void editingFinished();
+
         void itemCountChanged(const QString &tabName, int count);
 
         void showContextMenu(const QPoint &position);
 
-        void updateContextMenu();
+        void updateContextMenu(const ClipboardBrowser *self);
+
+        void searchRequest();
+        void searchHideRequest();
+
+        void closeExternalEditors();
 
     protected:
-        void keyPressEvent(QKeyEvent *event);
-        void contextMenuEvent(QContextMenuEvent *);
-        void resizeEvent(QResizeEvent *event);
-        void showEvent(QShowEvent *event);
-        void hideEvent(QHideEvent *event);
-        void currentChanged(const QModelIndex &current, const QModelIndex &previous);
-        void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);
-        void focusInEvent(QFocusEvent *event);
+        void keyPressEvent(QKeyEvent *event) override;
+        void contextMenuEvent(QContextMenuEvent *) override;
+        void resizeEvent(QResizeEvent *event) override;
+        void showEvent(QShowEvent *event) override;
+        void currentChanged(const QModelIndex &current, const QModelIndex &previous) override;
+        void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected) override;
+        void focusInEvent(QFocusEvent *event) override;
 
-        void dragEnterEvent(QDragEnterEvent *event);
-        void dragLeaveEvent(QDragLeaveEvent *event);
-        void dragMoveEvent(QDragMoveEvent *event);
-        void dropEvent(QDropEvent *event);
+        void dragEnterEvent(QDragEnterEvent *event) override;
+        void dragLeaveEvent(QDragLeaveEvent *event) override;
+        void dragMoveEvent(QDragMoveEvent *event) override;
+        void dropEvent(QDropEvent *event) override;
 
-        void paintEvent(QPaintEvent *e);
+        void paintEvent(QPaintEvent *e) override;
 
-        void mousePressEvent(QMouseEvent *event);
-        void mouseReleaseEvent(QMouseEvent *event);
-        void mouseMoveEvent(QMouseEvent *event);
+        void mousePressEvent(QMouseEvent *event) override;
+        void mouseReleaseEvent(QMouseEvent *event) override;
+        void mouseMoveEvent(QMouseEvent *event) override;
 
     private slots:
-        void onModelDataChanged();
-
         void onDataChanged(const QModelIndex &a, const QModelIndex &b);
 
+        void onRowsInserted(const QModelIndex &parent, int first, int last);
+
         void onItemCountChanged();
 
-        void onTabNameChanged(const QString &tabName);
-
-        /** Delayed update. */
-        void updateCurrentPage();
-
-        /** Immediate update if possible. */
-        void doUpdateCurrentPage();
-
-        void expire();
-
         void onEditorDestroyed();
 
         void onEditorSave();
 
         void onEditorCancel();
 
-        void onModelUnloaded();
+        void onEditorNeedsChangeClipboard();
 
-        void filterItems();
+        void onEditorNeedsChangeClipboard(const QByteArray &bytes, const QString &mime);
 
-    private:
         /**
          * Save items to configuration after an interval.
          */
         void delayedSaveItems();
 
-        bool isFiltered(int row) const;
+        /**
+         * Update item and editor sizes.
+         */
+        void updateSizes();
+
+    private:
+        bool isLoaded() const;
 
         /**
          * Hide row if filtered out, otherwise show.
          * @return true only if hidden
          */
         bool hideFiltered(int row);
+        bool hideFiltered(const QModelIndex &index);
 
         /**
          * Connects signals and starts external editor.
          */
-        bool startEditor(QObject *editor);
+        bool startEditor(QObject *editor, bool changeClipboard = false);
 
-        /**
-         * Select next/previous item and copy it to clipboard.
-         */
-        void copyItemToClipboard(int d);
+        void setEditorWidget(ItemEditorWidget *editor, bool changeClipboard = false);
 
-        /**
-         * Preload items in given range (relative to current scroll offset).
-         */
-        void preload(int minY, int maxY);
-
-        void setEditorWidget(ItemEditorWidget *widget);
-
-        void editItem(const QModelIndex &index, bool editNotes = false);
+        void editItem(const QModelIndex &index, bool editNotes = false, bool changeClipboard = false);
 
         void updateEditorGeometry();
 
-        bool canExpire();
-
-        void restartExpiring();
-
-        void stopExpiring();
-
         void updateCurrentItem();
 
         /**
@@ -408,9 +352,7 @@ class ClipboardBrowser : public QListView
          */
         QModelIndex indexNear(int offset) const;
 
-        void updateSearchProgress();
-
-        int getDropRow(const QPoint &position);
+        int getDropRow(QPoint position);
 
         void connectModelAndDelegate();
 
@@ -418,39 +360,43 @@ class ClipboardBrowser : public QListView
 
         void updateItemMaximumSize();
 
-        void lock();
-        void unlock();
+        void processDragAndDropEvent(QDropEvent *event);
 
-        void refilterItems();
+        /// Removes indexes without notifying or asking plugins.
+        int dropIndexes(const QModelIndexList &indexes);
 
-        ItemLoaderInterfacePtr m_itemLoader;
+        void focusEditedIndex();
+
+        int findNextVisibleRow(int row);
+        int findPreviousVisibleRow(int row);
+
+        void preload(int pixelsAboveCurrent, int pixelsBelowCurrent, const QModelIndex &current);
+        void preload(int pixels, bool above, const QModelIndex &current);
+
+        void updateCurrentIndex();
+
+        void moveToTop(const QModelIndex &index);
+
+        void maybeEmitEditingFinished();
+
+        ItemSaverPtr m_itemSaver;
         QString m_tabName;
-        int m_lastFiltered;
-        bool m_update;
-        ClipboardModel *m;
-        ItemDelegate *d;
+        ClipboardModel m;
+        ItemDelegate d;
         QTimer m_timerSave;
-        QTimer m_timerScroll;
-        QTimer m_timerUpdate;
-        QTimer m_timerFilter;
-        QTimer m_timerExpire;
+        QTimer m_timerEmitItemCount;
+        QTimer m_timerUpdateSizes;
+        bool m_resizing = false;
 
-        bool m_invalidateCache;
-        bool m_expireAfterEditing;
-
-        ItemEditorWidget *m_editor;
+        QPointer<ItemEditorWidget> m_editor;
+        int m_externalEditorsOpen = 0;
 
         ClipboardBrowserSharedPtr m_sharedData;
 
-        QPushButton *m_loadButton;
-        QProgressBar *m_searchProgress;
-
         int m_dragTargetRow;
         QPoint m_dragStartPosition;
 
-        int m_spinLock;
-        bool m_updateLock;
-        QScopedPointer<class ScrollSaver> m_scrollSaver;
+        int m_filterRow = -1;
 };
 
 #endif // CLIPBOARDBROWSER_H
