@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -31,7 +31,7 @@ class CommandEdit : public QWidget
     Q_OBJECT
 
 public:
-    explicit CommandEdit(QWidget *parent = NULL);
+    explicit CommandEdit(QWidget *parent = nullptr);
     ~CommandEdit();
 
     void setCommand(const QString &command) const;
@@ -41,18 +41,15 @@ public:
 
     QFont commandFont() const;
 
+    void resizeToContent();
+
 signals:
     void changed();
 
-protected:
-    void resizeEvent(QResizeEvent *event);
-
 private slots:
     void on_plainTextEditCommand_textChanged();
 
 private:
-    void updateCommandEditSize();
-
     Ui::CommandEdit *ui;
 };
 
