@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -22,24 +22,24 @@
 
 #include <QWidget>
 
-class QPushButton;
-class QTextBrowser;
+class QToolButton;
+class QDialog;
 
 class CommandHelpButton : public QWidget
 {
     Q_OBJECT
 public:
-    explicit CommandHelpButton(QWidget *parent = NULL);
+    explicit CommandHelpButton(QWidget *parent = nullptr);
 
 public slots:
-    void setHelpVisible(bool visible);
+    void showHelp();
 
 signals:
     void hidden();
 
 private:
-    QPushButton *m_button;
-    QTextBrowser *m_help;
+    QToolButton *m_button;
+    QDialog *m_help;
 };
 
 #endif // COMMANDHELPBUTTON_H
