@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,73 +20,105 @@
 #include "commandwidget.h"
 #include "ui_commandwidget.h"
 
+#include "common/appconfig.h"
 #include "common/command.h"
 #include "common/mimetypes.h"
-#include "gui/configurationmanager.h"
+#include "common/shortcuts.h"
 #include "gui/iconfactory.h"
 #include "gui/icons.h"
 #include "gui/shortcutdialog.h"
+#include "gui/tabicons.h"
 #include "item/itemfactory.h"
+#include "platform/platformnativeinterface.h"
 
 #include <QAction>
 #include <QFontMetrics>
 #include <QMenu>
-#include <QSyntaxHighlighter>
 
 namespace {
 
-QStringList serializeShortcuts(const QList<QKeySequence> &shortcuts)
-{
-    QStringList shortcutTexts;
+const char globalShortcutsDisabled[] = "DISABLED";
 
-    foreach (const QKeySequence &shortcut, shortcuts)
-        shortcutTexts.append(shortcut.toString(QKeySequence::PortableText));
+const QIcon iconClipboard() { return getIcon("", IconPaste); }
+const QIcon iconMenu() { return getIcon("", IconBars); }
+const QIcon iconShortcut() { return getIcon("", IconKeyboard); }
+
+QStringList serializeShortcuts(const QList<QKeySequence> &shortcuts, bool enabled = true)
+{
+    if ( shortcuts.isEmpty() )
+        return QStringList();
+
+    QStringList shortcutTexts;
+    shortcutTexts.reserve( shortcuts.size() + (enabled ? 1 : 0) );
+
+    for (const auto &shortcut : shortcuts)
+        shortcutTexts.append(portableShortcutText(shortcut));
+
+    if (!enabled)
+        shortcutTexts.append(globalShortcutsDisabled);
 
     return shortcutTexts;
 }
 
-void deserializeShortcuts(const QStringList &serializedShortcuts, ShortcutButton *shortcutButton)
+void deserializeShortcuts(
+        const QStringList &serializedShortcuts, ShortcutButton *shortcutButton,
+        QCheckBox *checkBoxEnabled = nullptr
+        )
 {
     shortcutButton->resetShortcuts();
 
-    foreach (const QString &shortcutText, serializedShortcuts)
-        shortcutButton->addShortcut(shortcutText);
-}
+    bool enabled = !serializedShortcuts.isEmpty();
 
-void setComboBoxItems(const QStringList &items, QComboBox *w)
-{
-    const QString text = w->currentText();
-    w->clear();
-    w->addItem(QString());
-    w->addItems(items);
-    w->setEditText(text);
+    for (const auto &shortcutText : serializedShortcuts) {
+        if (shortcutText == globalShortcutsDisabled)
+            enabled = false;
+        else
+            shortcutButton->addShortcut(shortcutText);
+    }
+
+    if (checkBoxEnabled)
+        checkBoxEnabled->setChecked(enabled);
 }
 
 } // namespace
 
 CommandWidget::CommandWidget(QWidget *parent)
     : QWidget(parent)
-    , ui(NULL)
-    , m_cmd()
+    , ui(new Ui::CommandWidget)
 {
+    ui->setupUi(this);
+
+    ui->widgetAdvanced->hide();
+
+    updateWidgets();
+
+#ifdef NO_GLOBAL_SHORTCUTS
+    ui->checkBoxGlobalShortcut->hide();
+    ui->shortcutButtonGlobalShortcut->hide();
+#else
+    ui->checkBoxGlobalShortcut->setIcon(iconShortcut());
+#endif
+
+    ui->groupBoxCommand->setFocusProxy(ui->commandEdit);
+
+    ui->checkBoxAutomatic->setIcon(iconClipboard());
+    ui->checkBoxInMenu->setIcon(iconMenu());
+
+    // Add tab names to combo boxes.
+    initTabComboBox(ui->comboBoxCopyToTab);
+    initTabComboBox(ui->comboBoxOutputTab);
+
+    if ( !createPlatformNativeInterface()->canGetWindowTitle() )
+        ui->lineEditWindow->hide();
 }
 
 CommandWidget::~CommandWidget()
 {
-    if (ui) {
-        delete ui;
-#if !defined(COPYQ_WS_X11) && !defined(Q_OS_WIN)
-        ui->lineEditWindow->hide();
-        ui->labelWindow->hide();
-#endif
-    }
+    delete ui;
 }
 
 Command CommandWidget::command() const
 {
-    if (!ui)
-        return m_cmd;
-
     Command c;
     c.name   = ui->lineEditName->text();
     c.re     = QRegExp( ui->lineEditMatch->text() );
@@ -105,7 +137,9 @@ Command CommandWidget::command() const
     c.enable = true;
     c.icon   = ui->buttonIcon->currentIcon();
     c.shortcuts = serializeShortcuts( ui->shortcutButton->shortcuts() );
-    c.globalShortcuts = serializeShortcuts( ui->shortcutButtonGlobalShortcut->shortcuts() );
+    c.globalShortcuts = serializeShortcuts(
+                ui->shortcutButtonGlobalShortcut->shortcuts(),
+                ui->checkBoxGlobalShortcut->isChecked() );
     c.tab    = ui->comboBoxCopyToTab->currentText();
     c.outputTab = ui->comboBoxOutputTab->currentText();
 
@@ -114,11 +148,6 @@ Command CommandWidget::command() const
 
 void CommandWidget::setCommand(const Command &c)
 {
-    if (!ui) {
-        m_cmd = c;
-        return;
-    }
-
     ui->lineEditName->setText(c.name);
     ui->lineEditMatch->setText( c.re.pattern() );
     ui->lineEditWindow->setText( c.wndre.pattern() );
@@ -135,19 +164,32 @@ void CommandWidget::setCommand(const Command &c)
     ui->checkBoxHideWindow->setChecked(c.hideWindow);
     ui->buttonIcon->setCurrentIcon(c.icon);
     deserializeShortcuts(c.shortcuts, ui->shortcutButton);
-    deserializeShortcuts(c.globalShortcuts, ui->shortcutButtonGlobalShortcut);
+    deserializeShortcuts(
+                c.globalShortcuts,
+                ui->shortcutButtonGlobalShortcut,
+                ui->checkBoxGlobalShortcut);
     ui->comboBoxCopyToTab->setEditText(c.tab);
     ui->comboBoxOutputTab->setEditText(c.outputTab);
 }
 
-QString CommandWidget::currentIcon() const
+void CommandWidget::setFormats(const QStringList &formats)
 {
-    return ui ? ui->buttonIcon->currentIcon() : m_cmd.icon;
+    setComboBoxItems(ui->comboBoxInputFormat, formats);
+    setComboBoxItems(ui->comboBoxOutputFormat, formats);
+}
+
+void CommandWidget::resizeEvent(QResizeEvent *event)
+{
+    QWidget::resizeEvent(event);
+    ui->commandEdit->resizeToContent();
 }
 
 void CommandWidget::showEvent(QShowEvent *event)
 {
-    init();
+    AppConfig appConfig;
+    const bool showAdvanced = appConfig.option<Config::show_advanced_command_settings>();
+    ui->checkBoxShowAdvanced->setChecked(showAdvanced);
+
     QWidget::showEvent(event);
 }
 
@@ -161,6 +203,15 @@ void CommandWidget::on_buttonIcon_currentIconChanged(const QString &iconString)
     emit iconChanged(iconString);
 }
 
+void CommandWidget::on_checkBoxShowAdvanced_stateChanged(int state)
+{
+    const bool showAdvanced = state == Qt::Checked;
+    AppConfig appConfig;
+    appConfig.setOption(Config::show_advanced_command_settings::name(), showAdvanced);
+    ui->widgetAdvanced->setVisible(showAdvanced);
+    updateWidgets();
+}
+
 void CommandWidget::on_checkBoxAutomatic_stateChanged(int)
 {
     updateWidgets();
@@ -172,6 +223,11 @@ void CommandWidget::on_checkBoxInMenu_stateChanged(int)
     updateWidgets();
 }
 
+void CommandWidget::on_checkBoxGlobalShortcut_stateChanged(int)
+{
+    updateWidgets();
+}
+
 void CommandWidget::on_shortcutButtonGlobalShortcut_shortcutAdded(const QKeySequence &)
 {
     updateWidgets();
@@ -187,58 +243,29 @@ void CommandWidget::on_commandEdit_changed()
     updateWidgets();
 }
 
-void CommandWidget::init()
-{
-    if (ui)
-        return;
-
-    QScopedPointer<Ui::CommandWidget> uiGuard(new Ui::CommandWidget);
-    uiGuard->setupUi(this);
-    ui = uiGuard.take();
-
-    updateWidgets();
-
-#ifdef NO_GLOBAL_SHORTCUTS
-    ui->shortcutButtonGlobalShortcut->hide();
-    ui->labelGlobalShortcut->hide();
-#else
-    ui->shortcutButtonGlobalShortcut->setExpectModifier(true);
-#endif
-
-    ui->groupBoxCommand->setFocusProxy(ui->commandEdit);
-
-    ConfigurationManager *cm = ConfigurationManager::instance();
-
-    // Add tab names to combo boxes.
-    QStringList tabs = cm->value("tabs").toStringList();
-    setComboBoxItems(tabs, ui->comboBoxCopyToTab);
-    setComboBoxItems(tabs, ui->comboBoxOutputTab);
-
-    // Add formats to combo boxex.
-    QStringList formats = cm->itemFactory()->formatsToSave();
-    formats.prepend(mimeText);
-    formats.removeDuplicates();
-
-    setComboBoxItems(formats, ui->comboBoxInputFormat);
-    setComboBoxItems(formats, ui->comboBoxOutputFormat);
-
-    setCommand(m_cmd);
-}
-
 void CommandWidget::updateWidgets()
 {
-    Q_ASSERT(ui);
     bool inMenu = ui->checkBoxInMenu->isChecked();
     bool copyOrExecute = inMenu || ui->checkBoxAutomatic->isChecked();
 #ifdef NO_GLOBAL_SHORTCUTS
     bool globalShortcut = false;
 #else
-    bool globalShortcut = ui->shortcutButtonGlobalShortcut->shortcutCount() > 0;
+    bool globalShortcut =
+            ui->checkBoxGlobalShortcut->isChecked()
+            && ui->shortcutButtonGlobalShortcut->shortcutCount() > 0;
 #endif
 
+    ui->widgetGlobalShortcut->setVisible(ui->checkBoxGlobalShortcut->isChecked());
+    ui->widgetMenuShortcut->setVisible(inMenu);
+
     ui->groupBoxMatchItems->setVisible(copyOrExecute);
     ui->groupBoxCommand->setVisible(copyOrExecute || globalShortcut);
     ui->groupBoxAction->setVisible(copyOrExecute);
     ui->groupBoxInMenu->setVisible(inMenu);
     ui->groupBoxCommandOptions->setHidden(!copyOrExecute || ui->commandEdit->isEmpty());
+
+    ui->widgetSpacer->setVisible(
+                ui->widgetAdvanced->isHidden() || ui->groupBoxCommand->isHidden());
+
+    ui->commandEdit->resizeToContent();
 }
