@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,8 +20,6 @@
 #ifndef COMMANDWIDGET_H
 #define COMMANDWIDGET_H
 
-#include "common/command.h"
-
 #include <QWidget>
 
 namespace Ui {
@@ -29,6 +27,7 @@ class CommandWidget;
 }
 
 class QComboBox;
+struct Command;
 
 /** Widget (set of widgets) for creating or modifying Command object. */
 class CommandWidget : public QWidget
@@ -36,17 +35,17 @@ class CommandWidget : public QWidget
     Q_OBJECT
 
 public:
-    explicit CommandWidget(QWidget *parent = NULL);
+    explicit CommandWidget(QWidget *parent = nullptr);
     ~CommandWidget();
 
     /** Return command for the widget. */
     Command command() const;
 
     /** Set current command. */
-    void setCommand(const Command &command);
+    void setCommand(const Command &c);
 
-    /** Return command icon. */
-    QString currentIcon() const;
+    /** Set formats for format selection combo boxes. */
+    void setFormats(const QStringList &formats);
 
 signals:
     void iconChanged(const QString &iconString);
@@ -56,17 +55,22 @@ signals:
     void automaticChanged(bool automatic);
 
 protected:
-    void showEvent(QShowEvent *event);
+    void resizeEvent(QResizeEvent *event) override;
+    void showEvent(QShowEvent *event) override;
 
 private slots:
     void on_lineEditName_textChanged(const QString &name);
 
     void on_buttonIcon_currentIconChanged(const QString &iconString);
 
+    void on_checkBoxShowAdvanced_stateChanged(int state);
+
     void on_checkBoxAutomatic_stateChanged(int);
 
     void on_checkBoxInMenu_stateChanged(int);
 
+    void on_checkBoxGlobalShortcut_stateChanged(int);
+
     void on_shortcutButtonGlobalShortcut_shortcutAdded(const QKeySequence &shortcut);
 
     void on_shortcutButtonGlobalShortcut_shortcutRemoved(const QKeySequence &shortcut);
@@ -79,7 +83,6 @@ private:
     void updateWidgets();
 
     Ui::CommandWidget *ui;
-    Command m_cmd;
 };
 
 #endif // COMMANDWIDGET_H
