@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,7 +20,6 @@
 #include "gui/itemorderlist.h"
 #include "ui_itemorderlist.h"
 
-#include "gui/configurationmanager.h"
 #include "gui/iconfactory.h"
 #include "gui/icons.h"
 
@@ -29,15 +28,25 @@
 #include <QMimeData>
 #include <QScrollBar>
 
+namespace {
+
+void deleteWidget(const QPointer<QWidget> &object)
+{
+    if (object)
+        object->deleteLater();
+}
+
+} // namespace
+
 ItemOrderList::ItemOrderList(QWidget *parent)
     : QWidget(parent)
     , ui(new Ui::ItemOrderList)
 {
     ui->setupUi(this);
     ui->pushButtonRemove->hide();
-    ui->toolButtonAdd->hide();
+    ui->pushButtonAdd->hide();
     setFocusProxy(ui->listWidgetItems);
-    setCurrentItemWidget(NULL);
+    setCurrentItemWidget(nullptr);
 }
 
 ItemOrderList::~ItemOrderList()
@@ -45,62 +54,66 @@ ItemOrderList::~ItemOrderList()
     delete ui;
 }
 
-void ItemOrderList::setAddMenu(QMenu *menu)
+void ItemOrderList::setAddRemoveButtonsVisible(bool visible)
 {
-    Q_ASSERT(menu != NULL);
-    ui->pushButtonRemove->show();
-    ui->toolButtonAdd->show();
-    ui->toolButtonAdd->setMenu(menu);
+    ui->pushButtonRemove->setVisible(visible);
+    ui->pushButtonAdd->setVisible(visible);
 }
 
 void ItemOrderList::clearItems()
 {
     ui->listWidgetItems->clear();
-    foreach ( QWidget *w, m_itemWidgets.values() )
-        ui->stackedWidget->removeWidget(w);
-    m_itemWidgets.clear();
+    for (const auto &pair : m_items)
+        deleteWidget(pair.widget);
+    m_items.clear();
 }
 
-void ItemOrderList::appendItem(const QString &label, bool checked, bool highlight, const QIcon &icon, QWidget *widget)
+void ItemOrderList::appendItem(const QString &label, bool checked, bool highlight, const QIcon &icon, const ItemPtr &item)
 {
-    insertItem(label, checked, highlight, icon, widget, -1);
+    insertItem(label, checked, highlight, icon, item, -1);
 }
 
 void ItemOrderList::insertItem(const QString &label, bool checked, bool highlight, const QIcon &icon,
-                               QWidget *widget, int targetRow)
+                               const ItemPtr &item, int targetRow)
 {
     QListWidget *list = ui->listWidgetItems;
-    QListWidgetItem *item = new QListWidgetItem(icon, label);
+    auto listItem = new QListWidgetItem(icon, label);
     const int row = targetRow >= 0 ? qMin(list->count(), targetRow) : list->count();
-    list->insertItem(row, item);
-    item->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
-    setItemHighlight(item, highlight);
+    list->insertItem(row, listItem);
+    listItem->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
+    setItemHighlight(listItem, highlight);
 
-    m_itemWidgets[item] = widget;
-    ui->stackedWidget->addWidget(widget);
+    m_items[listItem] = ItemWidgetPair(item);
 
     // Resize list to minimal size.
-    const int w = list->sizeHintForColumn(0)
-                + list->verticalScrollBar()->sizeHint().width() + 4;
-    list->setMaximumWidth(w);
+    if ( !isVisible() ) {
+        const int w = list->sizeHintForColumn(0)
+                    + list->verticalScrollBar()->sizeHint().width() + 4;
+        list->resize( w, list->height() );
+    }
 
-    if ( list->currentItem() == NULL )
+    if ( list->currentItem() == nullptr )
         list->setCurrentRow(row);
 }
 
-QWidget *ItemOrderList::itemWidget(int row) const
+QWidget *ItemOrderList::widget(int row) const
 {
-    return m_itemWidgets[item(row)];
+    return m_items[listItem(row)].widget;
+}
+
+QVariant ItemOrderList::data(int row) const
+{
+    return m_items[listItem(row)].item->data();
 }
 
 int ItemOrderList::itemCount() const
 {
-    return ui->listWidgetItems->count();
+    return m_items.size();
 }
 
 bool ItemOrderList::isItemChecked(int row) const
 {
-    return item(row)->checkState() == Qt::Checked;
+    return listItem(row)->checkState() == Qt::Checked;
 }
 
 int ItemOrderList::currentRow() const
@@ -110,43 +123,42 @@ int ItemOrderList::currentRow() const
 
 void ItemOrderList::setCurrentItem(int row)
 {
-    QListWidgetItem *currentItem = item(row);
+    QListWidgetItem *currentItem = listItem(row);
     ui->listWidgetItems->setCurrentItem(currentItem, QItemSelectionModel::ClearAndSelect);
-    QWidget *widget = m_itemWidgets[currentItem];
-    widget->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+    QWidget *widget = createWidget(currentItem);
     widget->setFocus();
 }
 
 void ItemOrderList::setCurrentItemIcon(const QIcon &icon)
 {
     QListWidgetItem *current = ui->listWidgetItems->currentItem();
-    if(current != NULL)
+    if(current != nullptr)
         current->setIcon(icon);
 }
 
 void ItemOrderList::setCurrentItemLabel(const QString &label)
 {
     QListWidgetItem *current = ui->listWidgetItems->currentItem();
-    if(current != NULL)
+    if(current != nullptr)
         current->setText(label);
 }
 
 void ItemOrderList::setCurrentItemHighlight(bool highlight)
 {
     QListWidgetItem *current = ui->listWidgetItems->currentItem();
-    if(current != NULL)
+    if(current != nullptr)
         setItemHighlight(current, highlight);
 }
 
 QString ItemOrderList::itemLabel(int row) const
 {
-    return item(row)->text();
+    return listItem(row)->text();
 }
 
 QList<int> ItemOrderList::selectedRows() const
 {
     QList<int> rows;
-    foreach (QListWidgetItem *item, ui->listWidgetItems->selectedItems())
+    for (auto item : ui->listWidgetItems->selectedItems())
         rows.append(ui->listWidgetItems->row(item));
     return rows;
 }
@@ -154,12 +166,12 @@ QList<int> ItemOrderList::selectedRows() const
 void ItemOrderList::setSelectedRows(const QList<int> &selectedRows)
 {
     ui->listWidgetItems->clearSelection();
-    ui->listWidgetItems->setCurrentItem(NULL);
+    ui->listWidgetItems->setCurrentItem(nullptr);
 
-    foreach (int row, selectedRows) {
+    for (int row : selectedRows) {
         if (row >= 0 && row < rowCount()) {
             QListWidgetItem *item = ui->listWidgetItems->item(row);
-            if ( ui->listWidgetItems->currentItem() == NULL )
+            if ( ui->listWidgetItems->currentItem() == nullptr )
                 ui->listWidgetItems->setCurrentItem(item);
             else
                 item->setSelected(true);
@@ -209,8 +221,8 @@ void ItemOrderList::dropEvent(QDropEvent *event)
 
 void ItemOrderList::showEvent(QShowEvent *event)
 {
-    if ( ui->toolButtonAdd->icon().isNull() ) {
-        ui->toolButtonAdd->setIcon( getIcon("list-add", IconPlus) );
+    if ( ui->pushButtonAdd->icon().isNull() ) {
+        ui->pushButtonAdd->setIcon( getIcon("list-add", IconPlus) );
         ui->pushButtonRemove->setIcon( getIcon("list-remove", IconMinus) );
         ui->pushButtonDown->setIcon( getIcon("go-down", IconArrowDown) );
         ui->pushButtonUp->setIcon( getIcon("go-up", IconArrowUp) );
@@ -247,18 +259,21 @@ void ItemOrderList::on_pushButtonDown_clicked()
 
 void ItemOrderList::on_pushButtonRemove_clicked()
 {
-    foreach (QListWidgetItem *item, ui->listWidgetItems->selectedItems())
+    for (auto item : ui->listWidgetItems->selectedItems()) {
+        ItemWidgetPair pair = m_items.take(item);
+        deleteWidget(pair.widget);
         delete item;
+    }
 }
 
-void ItemOrderList::on_toolButtonAdd_triggered(QAction *action)
+void ItemOrderList::on_pushButtonAdd_clicked()
 {
-    emit addButtonClicked(action);
+    emit addButtonClicked();
 }
 
 void ItemOrderList::on_listWidgetItems_currentItemChanged(QListWidgetItem *current, QListWidgetItem *)
 {
-    setCurrentItemWidget( m_itemWidgets.value(current, NULL) );
+    setCurrentItemWidget( current ? createWidget(current) : nullptr );
 }
 
 void ItemOrderList::on_listWidgetItems_itemSelectionChanged()
@@ -268,7 +283,7 @@ void ItemOrderList::on_listWidgetItems_itemSelectionChanged()
     emit itemSelectionChanged();
 }
 
-QListWidgetItem *ItemOrderList::item(int row) const
+QListWidgetItem *ItemOrderList::listItem(int row) const
 {
     Q_ASSERT(row >= 0 && row < itemCount());
     return ui->listWidgetItems->item(row);
@@ -276,11 +291,16 @@ QListWidgetItem *ItemOrderList::item(int row) const
 
 void ItemOrderList::setCurrentItemWidget(QWidget *widget)
 {
-    if (widget == NULL) {
-        ui->stackedWidget->hide();
-    } else {
-        ui->stackedWidget->setCurrentWidget(widget);
-        ui->stackedWidget->show();
+    // Reparent current widget so it's safely deleted.
+    QWidget *currentWidget = ui->scrollArea->takeWidget();
+    if (currentWidget != nullptr) {
+        currentWidget->setParent(this);
+        currentWidget->hide();
+    }
+
+    if (widget != nullptr) {
+        ui->scrollArea->setWidget(widget);
+        widget->show();
     }
 }
 
@@ -290,3 +310,11 @@ void ItemOrderList::setItemHighlight(QListWidgetItem *item, bool highlight)
     font.setBold(highlight);
     item->setFont(font);
 }
+
+QWidget *ItemOrderList::createWidget(QListWidgetItem *item)
+{
+    ItemWidgetPair &pair = m_items[item];
+    if (!pair.widget)
+        pair.widget = pair.item->createWidget(ui->scrollArea);
+    return pair.widget;
+}
