@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,25 +20,38 @@
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include "gui/clipboardbrowser.h"
-#include "gui/configtabshortcuts.h"
+#include "common/commandtester.h"
+#include "gui/clipboardbrowsershared.h"
+#include "gui/menuitems.h"
+#include "gui/notificationbutton.h"
 
 #include "platform/platformnativeinterface.h"
 
+#include <QAction>
+#include <QClipboard>
 #include <QMainWindow>
 #include <QPointer>
 #include <QSystemTrayIcon>
+#include <QTimer>
+#include <QModelIndex>
 
+class Action;
 class ActionHandler;
+class ClipboardBrowser;
+class ClipboardBrowserPlaceholder;
+class CommandAction;
 class CommandDialog;
 class ConfigurationManager;
+class ItemFactory;
 class NotificationDaemon;
-class QAction;
-class QModelIndex;
+class Theme;
 class TrayMenu;
 struct Command;
 struct MainWindowOptions;
 
+Q_DECLARE_METATYPE(QPersistentModelIndex)
+Q_DECLARE_METATYPE(QList<QPersistentModelIndex>)
+
 namespace Ui
 {
     class MainWindow;
@@ -51,6 +64,13 @@ enum ItemActivationCommand {
     ActivatePastes = 0x4
 };
 
+enum class ImportOptions {
+    /// Select what to import/export in dialog.
+    Select,
+    /// Import/export everything without asking.
+    All
+};
+
 struct MainWindowOptions {
     MainWindowOptions()
         : confirmExit(true)
@@ -60,14 +80,17 @@ struct MainWindowOptions {
         , trayTabName()
         , trayItems(5)
         , trayImages(true)
+        , trayMenuOpenOnLeftClick(false)
         , itemPopupInterval(0)
         , clipboardNotificationLines(0)
         , transparency(0)
         , transparencyFocused(0)
         , hideTabs(false)
+        , hideMainWindow(false)
         , itemActivationCommands(ActivateCloses)
         , clearFirstTab(false)
         , trayItemPaste(true)
+        , clipboardTab()
     {}
 
     bool activateCloses() const { return itemActivationCommands & ActivateCloses; }
@@ -81,6 +104,7 @@ struct MainWindowOptions {
     QString trayTabName;
     int trayItems;
     bool trayImages;
+    bool trayMenuOpenOnLeftClick;
     int itemPopupInterval;
     int clipboardNotificationLines;
     int transparency;
@@ -88,11 +112,23 @@ struct MainWindowOptions {
 
     bool hideTabs;
 
+    bool hideMainWindow;
+
     int itemActivationCommands;
 
     bool clearFirstTab;
 
     bool trayItemPaste;
+
+    QString clipboardTab;
+};
+
+class Callable {
+public:
+    Callable() = default;
+    Callable(Callable &&) = default;
+    virtual ~Callable() = default;
+    virtual void operator()() = 0;
 };
 
 /**
@@ -114,7 +150,7 @@ class MainWindow : public QMainWindow
     Q_OBJECT
 
 public:
-    explicit MainWindow(QWidget *parent = NULL);
+    explicit MainWindow(ItemFactory *itemFactory, QWidget *parent = nullptr);
     ~MainWindow();
 
     /** Return true if in browse mode (i.e. search field is hidden). */
@@ -122,9 +158,13 @@ public:
 
     bool hasRunningAction() const;
 
-    QWidget *trayMenu();
-
-    QByteArray getActionData(const QByteArray &actionId, const QString &format);
+    /**
+     * Try to close command dialog and return true on success.
+     *
+     * Note that dialog won't be closed if it has unsaved changes
+     * and user canceles the closing.
+     */
+    bool maybeCloseCommandDialog();
 
     /**
      * Return browser widget in given tab @a index.
@@ -138,7 +178,7 @@ public:
      */
     ClipboardBrowser *browser();
 
-    /** Return browser containing item or NULL. */
+    /** Return browser containing item or nullptr. */
     ClipboardBrowser *browserForItem(const QModelIndex &index);
 
     /**
@@ -148,34 +188,43 @@ public:
     int findTabIndex(const QString &name);
 
     /**
-     * Create tab with given @a name if it doesn't exist.
+     * Tries to find tab with exact or similar name (ignores
+     * key hints '&') or creates new one.
      * @return Existing or new tab with given @a name.
      */
-    ClipboardBrowser *createTab(
+    ClipboardBrowser *tab(
             const QString &name //!< Name of the new tab.
             );
 
     /**
      * Show/hide tray menu. Return true only if menu is shown.
      */
-    bool toggleMenu(ClipboardBrowser *browser = NULL);
+    bool toggleMenu();
+    bool toggleMenu(const QString &tabName, int itemCount, QPoint position);
 
     /** Switch between browse and search mode. */
-    void enterBrowseMode(bool browsemode = true);
+    void enterBrowseMode();
+
+    void enterSearchMode();
+
+    void enterSearchMode(const QString &txt);
 
     /** Show notification. */
-    void showMessage(
-            const QString &title, //!< Message title.
+    void showMessage(const QString &title, //!< Message title.
             const QString &msg, //!< Message text.
-            QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::Information,
+            ushort icon = QSystemTrayIcon::Information,
             //!< Type of popup.
             int msec = 8000, //!< Show interval.
-            int notificationId = -1 //!< ID of notification.
-            );
+            const QString &notificationId = QString(), //!< ID of notification.
+            const NotificationButtons &buttons = NotificationButtons());
 
     /** Show popup with icon. */
-    void showMessage(const QString &title, const QString &msg, ushort icon, int msec,
-                     int notificationId);
+    void showMessage(const QString &title,
+            const QString &msg,
+            const QString &icon,
+            int msec,
+            const QString &notificationId = QString(),
+            const NotificationButtons &buttons = NotificationButtons());
 
     /** Show clipboard content in notification. */
     void showClipboardMessage(const QVariantMap &data);
@@ -184,6 +233,10 @@ public:
     void showWindow();
     /** Hide window to tray or minimize if tray is not available. */
     void hideWindow();
+    /** Minimize window (hide if option is set). */
+    void minimizeWindow();
+    /** Set current tab. */
+    bool setCurrentTab(int index);
     /** Show window and given tab and give focus to the tab. */
     void showBrowser(int index);
     /** Enter browse mode and reset search. */
@@ -196,7 +249,7 @@ public:
     /** Remove tab. */
     void removeTab(
             bool ask, //!< Ask before removing.
-            int tab_index //!< Tab index or current tab.
+            int tabIndex //!< Tab index or current tab.
             );
     /** Set icon for tab or tab group. */
     void setTabIcon(const QString &tabName);
@@ -209,7 +262,7 @@ public:
      */
     bool saveTab(
             const QString &fileName,
-            int tab_index = -1 //!< Tab index or current tab.
+            int tabIndex = -1 //!< Tab index or current tab.
             );
 
     /** Save all unsaved tabs. */
@@ -221,10 +274,17 @@ public:
      */
     bool loadTab(const QString &fileName);
 
-    /** Set next or first tab as current. */
-    void nextTab();
-    /** Set previous or last tab as current. */
-    void previousTab();
+    /**
+     * Import tabs, settings etc.
+     * @return True only if all data were successfully loaded.
+     */
+    bool importData(const QString &fileName, ImportOptions options);
+
+    /**
+     * Export tabs, settings etc.
+     * @return True only if all data were successfully saved.
+     */
+    bool exportAllData(const QString &fileName);
 
     /** Called after clipboard content changes. */
     void clipboardChanged(const QVariantMap &data);
@@ -235,19 +295,25 @@ public:
     /** Return true only if monitoring is enabled. */
     bool isMonitoringEnabled() const;
 
-    /** Return true if clipboard storing was disabled. */
-    bool isClipboardStoringDisabled() const { return m_clipboardStoringDisabled; }
-
-    /** Ignore current clipboard content. */
-    void ignoreCurrentClipboard();
-
-    bool isClipboardIgnored() const { return m_ignoreCurrentClipboard; }
+    /** Abort execution of automatic commands. */
+    void abortAutomaticCommands();
 
     QStringList tabs() const;
 
-    QString selectedTab() const;
+    /** Update the first item in the first tab. */
+    void updateFirstItem(QVariantMap data);
 
-    QList<int> selectedItems() const;
+    /// Used by config() command.
+    QVariant config(const QStringList &nameValue);
+
+    QVariantMap actionData(int id) const;
+    void setActionData(int id, const QVariantMap &data);
+
+    void setCommands(const QList<Command> &commands);
+
+    void setSessionIconColor(QColor color);
+
+    QColor sessionIconColor() const;
 
 public slots:
     /** Close main window and exit the application. */
@@ -256,6 +322,12 @@ public slots:
     /** Load settings. */
     void loadSettings();
 
+    /** Open help. */
+    void openHelp();
+
+    /** Open log dialog. */
+    void openLogDialog();
+
     /** Open about dialog. */
     void openAboutDialog();
 
@@ -265,8 +337,11 @@ public slots:
     /** Open dialog with active commands. */
     void showProcessManagerDialog();
 
-    /** Open action dialog with given input @a text. */
-    WId openActionDialog(const QVariantMap &data);
+    /** Open action dialog with given input data. */
+    void openActionDialog(const QVariantMap &data);
+
+    /** Open action dialog with input data from selected items. */
+    void openActionDialog();
 
     /** Open preferences dialog. */
     void openPreferences();
@@ -280,10 +355,10 @@ public slots:
     void reverseSelectedItems();
 
     /**
-     * Load saved items to new tab.
-     * Show file dialog and focus the new tab.
+     * Import tabs, settings etc. (select file in dialog).
+     * @return True only if all data were successfully loaded.
      */
-    bool loadTab();
+    bool importData();
 
     /** Create new item in current tab. */
     void editNewItem();
@@ -295,15 +370,23 @@ public slots:
     /** Activate current item. */
     void activateCurrentItem();
 
+    /** Set current tab. */
+    void setCurrentTab(const ClipboardBrowser *browser);
+
     /** Show window and given tab and give focus to the tab. */
     void showBrowser(const ClipboardBrowser *browser);
 
     /** Show error popup message. */
     void showError(const QString &msg);
 
+    /** Open command dialog and add commands. */
+    void addCommands(const QList<Command> &commands);
+
     /** Execute command on given input data. */
-    void action(const QVariantMap &data, const Command &cmd,
-                const QModelIndex &outputIndex = QModelIndex());
+    Action *action(
+            const QVariantMap &data,
+            const Command &cmd,
+            const QModelIndex &outputIndex = QModelIndex());
 
     /** Add @a data to tab with given name (create if tab doesn't exist). */
     void addToTab(
@@ -313,10 +396,16 @@ public slots:
             const QString &tabName
             );
 
-    /** Add @a new clipboard to the first tab. */
-    void addToTabFromClipboard(const QVariantMap &data);
+    /**
+     * Run automatic commands and add @a new clipboard to the first tab
+     * if commands didn't remove or transform the data.
+     */
+    void runAutomaticCommands(QVariantMap data);
 
     /** Set clipboard. */
+    void setClipboard(const QVariantMap &data, QClipboard::Mode mode);
+
+    /** Set clipboard and synchronize selection if needed. */
     void setClipboard(const QVariantMap &data);
 
     /** Show/hide main window. Return true only if window is shown. */
@@ -340,21 +429,66 @@ public slots:
 
     /**
      * Add tab with given name if doesn't exist and focus the tab.
-     * @return New or existing tab with given name.
      */
-    ClipboardBrowser *addTab(const QString &name);
+    void addTab(const QString &name);
 
     /** Toggle monitoring (i.e. adding new clipboard content to the first tab). */
     void toggleClipboardStoring();
 
-    /** Save all items in tab. Show file dialog. */
-    bool saveTab(
-            int tab_index = -1 //!< Tab index or current tab.
-            );
+    /**
+     * Export tabs, settings etc. (select in file dialog).
+     * @return True only if all data were successfully saved.
+     */
+    bool exportData();
+
+    /** Set next or first tab as current. */
+    void nextTab();
+    /** Set previous or last tab as current. */
+    void previousTab();
+
+    /** Set window title and tray tool tip from data. */
+    void updateTitle(const QVariantMap &data);
+
+    bool canUpdateTitleFromScript() const;
+
+    /** Set text for filtering items. */
+    void setFilter(const QString &text);
+
+    void hideSearchBar();
+
+    void invoke(Callable *callable);
+
+#ifdef HAS_TESTS
+    /**
+     * Send key clicks to currently focused widget.
+     *
+     * This function will block if it opens a modal dialog.
+     * So use sendKeyClicks() instead.
+     *
+     * Increments key clicks sequence number returned by lastReceivedKeyClicks().
+     */
+    void keyClicks(const QString &keys, int delay);
+
+    /**
+     * Send key clicks to focused widget.
+     * @return Key clicks sequence number.
+     */
+    uint sendKeyClicks(const QString &keys, int delay);
+
+    /**
+     * @return Last key clicks sequence number received by widgets.
+     */
+    uint lastReceivedKeyClicks();
+
+    /**
+     * Remove all tab and reset configuration (work only in test session).
+     */
+    void resetTestSession(const QString &clipboardTabName);
+#endif
 
 signals:
     /** Request clipboard change. */
-    void changeClipboard(const QVariantMap &data);
+    void changeClipboard(const QVariantMap &data, QClipboard::Mode mode);
 
     void tabGroupSelected(bool selected);
 
@@ -362,41 +496,51 @@ signals:
 
     void commandsSaved();
 
+    void stopItemMenuCommandTester();
+    void stopTrayMenuCommandTester();
+
+    void configurationChanged();
+
 protected:
-    void keyPressEvent(QKeyEvent *event);
-    void keyReleaseEvent(QKeyEvent *event);
-    bool event(QEvent *event);
+    void keyPressEvent(QKeyEvent *event) override;
+    void keyReleaseEvent(QKeyEvent *event) override;
+    bool event(QEvent *event) override;
 
     /** Hide (minimize to tray) window on close. */
-    void closeEvent(QCloseEvent *event);
+    void closeEvent(QCloseEvent *event) override;
 
-    void showEvent(QShowEvent *event);
+    void showEvent(QShowEvent *event) override;
+
+    bool focusNextPrevChild(bool next) override;
 
 #if QT_VERSION < 0x050000
 #   ifdef COPYQ_WS_X11
-    bool x11Event(XEvent *event);
+    bool x11Event(XEvent *event) override;
 #   elif defined(Q_OS_WIN)
-    bool winEvent(MSG *message, long *result);
+    bool winEvent(MSG *message, long *result) override;
 #   elif defined(Q_OS_MAC)
-    bool macEvent(EventHandlerCallRef caller, EventRef event);
+    bool macEvent(EventHandlerCallRef caller, EventRef event) override;
 #   endif
 #else
-    bool nativeEvent(const QByteArray &eventType, void *message, long *result);
+    bool nativeEvent(const QByteArray &eventType, void *message, long *result) override;
 #endif
 
 private slots:
+    ClipboardBrowser *getTabForMenu();
     ClipboardBrowser *getTabForTrayMenu();
     void updateTrayMenuItems();
+    void addMenuItems(const QString &searchText);
+    void addTrayMenuItems(const QString &searchText);
     void trayActivated(QSystemTrayIcon::ActivationReason reason);
-    void onTrayActionTriggered(uint clipboardItemHash, bool omitPaste);
-    void enterSearchMode(const QString &txt);
-    void findNext(int where = 1);
-    void findPrevious();
+    void onMenuActionTriggered(uint itemHash, bool omitPaste);
+    void onTrayActionTriggered(uint itemHash, bool omitPaste);
+    void findNextOrPrevious();
     void tabChanged(int current, int previous);
     void saveTabPositions();
+    void doSaveTabPositions();
     void tabsMoved(const QString &oldPrefix, const QString &newPrefix);
-    void tabMenuRequested(const QPoint &pos, int tab);
-    void tabMenuRequested(const QPoint &pos, const QString &groupPath);
+    void tabMenuRequested(QPoint pos, int tab);
+    void tabMenuRequested(QPoint pos, const QString &groupPath);
     void tabCloseRequested(int tab);
     void onFilterChanged(const QRegExp &re);
     void createTrayIfSupported();
@@ -406,32 +550,68 @@ private slots:
 
     /** Update tray and window icon depending on current state. */
     void updateIcon();
-    void updateIconTimeout();
+    void updateIconSnipTimeout();
 
     void updateContextMenuTimeout();
 
+    void updateItemPreview();
+    void setItemPreviewVisible(bool visible);
+
+    /** Update icon snip animation. */
+    void updateIconSnip();
+
     void onAboutToQuit();
 
     void onCommandDialogSaved();
 
     void onSaveCommand(const Command &command);
 
-    void onCommandActionTriggered(const Command &command, const QVariantMap &data);
+    void onItemCommandActionTriggered(CommandAction *commandAction, const QString &triggeredShortcut);
+    void onClipboardCommandActionTriggered(CommandAction *commandAction, const QString &triggeredShortcut);
 
-    void on_tabWidget_dropItems(const QString &tabName, QDropEvent *event);
+    void on_tabWidget_dropItems(const QString &tabName, const QMimeData *data);
 
-    void showContextMenu(const QPoint &position);
+    void showContextMenu(QPoint position);
 
-    void updateContextMenu();
+    void updateContextMenu(const ClipboardBrowser *browser);
 
-    void action();
+    void automaticCommandTestFinished(const Command &command, bool passed);
+
+    QAction *enableActionForCommand(QMenu *menu, const Command &command, bool enable);
+    void addCommandsToItemMenu(const Command &command, bool passed);
+    void addCommandsToTrayMenu(const Command &command, bool passed);
+
+    void nextItemFormat();
+    void previousItemFormat();
+
+    void moveUp();
+    void moveDown();
+    void moveToTop();
+    void moveToBottom();
+
+    void onBrowserCreated(ClipboardBrowser *browser);
+
+    void onNotificationButtonClicked(const NotificationButton &button);
 
 private:
+    enum TabNameMatching {
+        MatchExactTabName,
+        MatchSimilarTabName
+    };
+
+    ClipboardBrowserPlaceholder *createTab(const QString &name, TabNameMatching nameMatch);
+
+    int findTabIndexExactMatch(const QString &name);
+
+    void clearTitle() { updateTitle(QVariantMap()); }
+
     /** Create menu bar and tray menu with items. Called once. */
     void createMenu();
 
     /** Create context menu for @a tab. It will be automatically deleted after closed. */
-    void popupTabBarMenu(const QPoint &pos, const QString &tab);
+    void popupTabBarMenu(QPoint pos, const QString &tab);
+
+    void updateContextMenu();
 
     void updateNotifications();
 
@@ -441,13 +621,13 @@ private:
     void updateMonitoringActions();
 
     /** Return browser widget in given tab @a index. */
-    ClipboardBrowser *getBrowser(int index) const;
+    ClipboardBrowserPlaceholder *getPlaceholder(int index) const;
 
     /** Return browser widget in current tab. */
-    ClipboardBrowser *getBrowser() const;
+    ClipboardBrowserPlaceholder *getPlaceholder() const;
 
-    /** Call updateFocusWindows() after a small delay. */
-    void delayedUpdateFocusWindows();
+    /** Call updateFocusWindows() after a small delay if main window or menu is not active. */
+    void delayedUpdateForeignFocusWindows();
 
     /** Show/hide tab bar. **/
     void setHideTabs(bool hide);
@@ -456,31 +636,26 @@ private:
      * Return true if window should be minimized instead of closed/hidden.
      *
      * If tray icon is not available, window should be minimized so that it can be opened with
-     * mouse. Some window managers (dwm, wmfs etc.) doesn't support minimizing, in that case
-     * window can be hidden instead.
+     * mouse.
      */
     bool closeMinimizes() const;
 
-    /**
-     * Execute all matching automatic commands on @a data.
-     * If Command::remove of an applied command is true the method immediately returns false.
-     */
-    bool executeAutomaticCommands(const QVariantMap &data);
-
     /** Return notification daemon (create if doesn't exist). */
     NotificationDaemon *notificationDaemon();
 
-    ClipboardBrowser *createTab(const QString &name, bool *needSave);
+    QAction *createAction(int id, const char *slot, QMenu *menu);
 
-    QAction *createAction(Actions::Id id, const char *slot, QMenu *menu);
-
-    QAction *addTrayAction(Actions::Id id);
+    QAction *addTrayAction(int id);
 
     void updateTabIcon(const QString &newName, const QString &oldName);
 
-    QAction *addItemAction(Actions::Id id, QObject *receiver, const char *slot);
+    QAction *addItemAction(int id, QObject *receiver, const char *slot);
 
-    void addCommandsToMenu(QMenu *menu, const QVariantMap &data);
+    QList<Command> commandsForMenu(const QVariantMap &data, const QString &tabName);
+    void addCommandsToItemMenu(ClipboardBrowser *c);
+    void addCommandsToTrayMenu(const QVariantMap &clipboardData);
+
+    bool isItemMenuDefaultActionValid() const;
 
     void updateToolBar();
 
@@ -488,10 +663,35 @@ private:
 
     void updateTrayTooltip();
 
-    void updateTrayIcon();
-
     void initTray();
 
+    void runAutomaticCommand(const Command &command);
+
+    bool isWindowVisible() const;
+
+    void onEscape();
+
+    /** Disable shortcuts for all default actions. */
+    void setDisabledShortcuts(const QList<QKeySequence> &shortcuts);
+
+    void updateActionShortcuts(int id);
+    void updateActionShortcuts();
+
+    void pasteClipboard(const PlatformWindowPtr &window);
+
+    QAction *actionForMenuItem(int id, QWidget *parent, Qt::ShortcutContext context);
+
+    void addMenuItems(TrayMenu *menu, ClipboardBrowser *c, int maxItemCount, const QString &searchText);
+    void onMenuActionTriggered(ClipboardBrowser *c, uint itemHash, bool omitPaste);
+    QWidget *toggleMenu(TrayMenu *menu, QPoint pos);
+    QWidget *toggleMenu(TrayMenu *menu);
+
+    bool exportData(const QString &fileName, const QStringList &tabs, bool exportConfiguration, bool exportCommands);
+    bool exportDataV3(QDataStream *out, const QStringList &tabs, bool exportConfiguration, bool exportCommands);
+    bool importDataV3(QDataStream *in, ImportOptions options);
+
+    const Theme &theme() const;
+
     ConfigurationManager *cm;
     Ui::MainWindow *ui;
 
@@ -515,20 +715,44 @@ private:
     QTimer m_timerShowWindow;
     QTimer m_timerTrayAvailable;
     QTimer m_timerTrayIconSnip;
-    QTimer m_timerMinimizing;
+    QTimer m_timerSaveTabPositions;
 
     NotificationDaemon *m_notifications;
 
-    bool m_minimizeUnsupported;
-
     ActionHandler *m_actionHandler;
 
     QVariantMap m_clipboardData;
-    bool m_ignoreCurrentClipboard;
 
-    ClipboardBrowser *m_trayTab;
+    TrayMenu *m_menu;
+    QString m_menuTabName;
+    int m_menuMaxItemCount;
 
     QPointer<CommandDialog> m_commandDialog;
+
+    CommandTester m_itemMenuCommandTester;
+    CommandTester m_trayMenuCommandTester;
+    CommandTester m_automaticCommandTester;
+
+    QPointer<Action> m_currentAutomaticCommand;
+    bool m_canUpdateTitleFromScript;
+
+    bool m_iconSnip;
+
+    bool m_wasMaximized;
+
+    bool m_showItemPreview;
+
+    QList<QKeySequence> m_disabledShortcuts;
+
+    QVector< QPointer<QAction> > m_actions;
+    MenuItems m_menuItems;
+
+#ifdef HAS_TESTS
+    /// Key clicks sequence number last returned by sendKeyClicks().
+    uint m_sentKeyClicks = 0;
+    /// Key clicks sequence number received by widgets.
+    uint m_receivedKeyClicks = 0;
+#endif
 };
 
 #endif // MAINWINDOW_H
