@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,123 +19,121 @@
 
 #include "gui/notification.h"
 
+#include "common/appconfig.h"
 #include "common/common.h"
-#include "gui/configurationmanager.h"
+#include "common/display.h"
+#include "common/textdata.h"
 #include "gui/iconfactory.h"
 #include "gui/icons.h"
 
 #include <QApplication>
 #include <QClipboard>
+#include <QDialog>
 #include <QDialogButtonBox>
 #include <QGridLayout>
+#include <QHBoxLayout>
 #include <QIcon>
 #include <QLabel>
 #include <QMap>
 #include <QMouseEvent>
 #include <QPainter>
 #include <QPushButton>
-#include <QScopedPointer>
 #include <QTextEdit>
 #include <QVBoxLayout>
 
+#include <memory>
+
 namespace {
 
-void showNotificationInspectDialog(const QString &messageTitle, const QString &message)
+class NotificationButtonWidget : public QPushButton
 {
-    QScopedPointer<QDialog> dialog(new QDialog);
-    dialog->setObjectName("InspectNotificationDialog");
-    dialog->setWindowTitle( Notification::tr("CopyQ Inspect Notification") );
+    Q_OBJECT
 
-    ConfigurationManager *cm = ConfigurationManager::instance();
-    cm->registerWindowGeometry( dialog.data() );
+public:
+    NotificationButtonWidget(const NotificationButton &button, QWidget *parent)
+        : QPushButton(button.name, parent)
+        , m_button(button)
+    {
+        connect( this, SIGNAL(clicked()),
+                 this, SLOT(onClicked()) );
+    }
 
-    if ( cm->value("always_on_top").toBool() )
-        dialog->setWindowFlags( dialog->windowFlags() ^ Qt::WindowStaysOnTopHint );
+signals:
+    void clicked(const NotificationButton &button);
 
-    IconFactory *iconFactory = cm->iconFactory();
-    dialog->setWindowIcon( iconFactory->appIcon() );
+private slots:
+    void onClicked()
+    {
+        emit clicked(m_button);
+    }
 
-    QTextEdit *editor = new QTextEdit(dialog.data());
-    editor->setReadOnly(true);
-    editor->setText(
-        "<h3>" + escapeHtml(messageTitle) + "</h3>"
-        "<p>" + escapeHtml(message) + "</p>"
-        );
-
-    QDialogButtonBox *buttons = new QDialogButtonBox(
-                QDialogButtonBox::Close, Qt::Horizontal, dialog.data() );
-    QObject::connect( buttons, SIGNAL(rejected()), dialog.data(), SLOT(close()) );
-
-    QPushButton *copyButton = new QPushButton( Notification::tr("&Copy"), buttons );
-    const QIcon icon = iconFactory->getIcon("clipboard", IconPaste);
-    copyButton->setIcon(icon);
-    QObject::connect( copyButton, SIGNAL(clicked()), editor, SLOT(selectAll()) );
-    QObject::connect( copyButton, SIGNAL(clicked()), editor, SLOT(copy()) );
-    buttons->addButton(copyButton, QDialogButtonBox::ActionRole);
-
-    QVBoxLayout *layout = new QVBoxLayout( dialog.data() );
-    layout->addWidget(editor);
-    layout->addWidget(buttons);
-
-    dialog->adjustSize();
-    dialog->setAttribute(Qt::WA_DeleteOnClose);
-    dialog.take()->show();
-}
+private:
+    NotificationButton m_button;
+};
 
 } // namespace
 
-Notification::Notification(int id)
+Notification::Notification(const QString &id, const QString &title, const NotificationButtons &buttons)
     : m_id(id)
-    , m_body(NULL)
-    , m_titleLabel(NULL)
-    , m_iconLabel(NULL)
-    , m_msgLabel(NULL)
+    , m_body(nullptr)
+    , m_titleLabel(nullptr)
+    , m_iconLabel(nullptr)
+    , m_msgLabel(nullptr)
     , m_opacity(1.0)
-    , m_icon(0)
 {
-    QVBoxLayout *bodyLayout = new QVBoxLayout(this);
+    auto bodyLayout = new QVBoxLayout(this);
     bodyLayout->setMargin(8);
     m_body = new QWidget(this);
     bodyLayout->addWidget(m_body);
     bodyLayout->setSizeConstraint(QLayout::SetMaximumSize);
 
-    QGridLayout *layout = new QGridLayout(m_body);
+    auto layout = new QGridLayout(m_body);
     layout->setMargin(0);
 
-    m_titleLabel = new QLabel(this);
-    layout->addWidget(m_titleLabel, 0, 0, 1, 2, Qt::AlignCenter);
-    m_titleLabel->setTextFormat(Qt::PlainText);
-    setTitle( QString() );
-
     m_iconLabel = new QLabel(this);
-    layout->addWidget(m_iconLabel, 1, 0, Qt::AlignTop);
+    m_iconLabel->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
 
     m_msgLabel = new QLabel(this);
-    layout->addWidget(m_msgLabel, 1, 1, Qt::AlignAbsolute);
+    m_msgLabel->setAlignment(Qt::AlignTop | Qt::AlignAbsolute);
+
+    if ( !title.isEmpty() ) {
+        m_titleLabel = new QLabel(this);
+        m_titleLabel->setObjectName("NotificationTitle");
+        m_titleLabel->setTextFormat(Qt::PlainText);
+        m_titleLabel->setText(title);
+
+        layout->addWidget(m_iconLabel, 0, 0);
+        layout->addWidget(m_titleLabel, 0, 1, Qt::AlignCenter);
+        layout->addWidget(m_msgLabel, 1, 0, 1, 2);
+    } else {
+        layout->addWidget(m_iconLabel, 0, 0, Qt::AlignTop);
+        layout->addWidget(m_msgLabel, 0, 1);
+    }
+
+
+    if ( !buttons.isEmpty() ) {
+        auto buttonLayout = new QHBoxLayout();
+        buttonLayout->addStretch();
+        layout->addLayout(buttonLayout, 2, 0, 1, 2);
+
+        for (const auto &button : buttons) {
+            const auto buttonWidget = new NotificationButtonWidget(button, this);
+            connect( buttonWidget, SIGNAL(clicked(NotificationButton)),
+                     this, SLOT(onButtonClicked(NotificationButton)) );
+            buttonLayout->addWidget(buttonWidget);
+        }
+    }
 
     setWindowFlags(Qt::ToolTip);
     setWindowOpacity(m_opacity);
-}
-
-void Notification::setTitle(const QString &title)
-{
-    m_titleLabel->setText(title);
-    if (title.isEmpty()) {
-        m_titleLabel->hide();
-    } else {
-        m_titleLabel->show();
-        m_titleLabel->adjustSize();
-    }
+    setAttribute(Qt::WA_ShowWithoutActivating);
 }
 
 void Notification::setMessage(const QString &msg, Qt::TextFormat format)
 {
     m_msgLabel->setTextFormat(format);
     m_msgLabel->setText(msg);
-    m_msgLabel->adjustSize();
-
-    m_textToCopy = (format == Qt::PlainText) ? msg : QString();
-
+    m_msgLabel->setVisible( !msg.isEmpty() );
 }
 
 void Notification::setPixmap(const QPixmap &pixmap)
@@ -143,7 +141,7 @@ void Notification::setPixmap(const QPixmap &pixmap)
     m_msgLabel->setPixmap(pixmap);
 }
 
-void Notification::setIcon(ushort icon)
+void Notification::setIcon(const QString &icon)
 {
     m_icon = icon;
 }
@@ -167,9 +165,11 @@ void Notification::setOpacity(qreal opacity)
 void Notification::updateIcon()
 {
     const QColor color = getDefaultIconColor(*this);
-    IconFactory *iconFactory = ConfigurationManager::instance()->iconFactory();
-    const int height = m_msgLabel->fontMetrics().lineSpacing() * 1.2;
-    const QPixmap pixmap = iconFactory->createPixmap(m_icon, color, height);
+    const auto height = static_cast<int>( m_msgLabel->fontMetrics().lineSpacing() * 1.2 );
+    const auto iconId = toIconId(m_icon);
+    const auto pixmap = iconId == 0
+            ? QPixmap(m_icon)
+            : createPixmap(iconId, color, height);
     m_iconLabel->setPixmap(pixmap);
     m_iconLabel->resize(pixmap.size());
 }
@@ -180,14 +180,8 @@ void Notification::adjust()
     adjustSize();
 }
 
-void Notification::mousePressEvent(QMouseEvent *event)
+void Notification::mousePressEvent(QMouseEvent *)
 {
-    if (event->button() != Qt::LeftButton) {
-        if ( !m_textToCopy.isEmpty() )
-            showNotificationInspectDialog(m_titleLabel->text(), m_textToCopy);
-        return;
-    }
-
     m_timer.stop();
 
     emit closeNotification(this);
@@ -240,3 +234,11 @@ void Notification::onTimeout()
 {
     emit closeNotification(this);
 }
+
+void Notification::onButtonClicked(const NotificationButton &button)
+{
+    emit buttonClicked(button);
+    emit closeNotification(this);
+}
+
+#include "notification.moc"
