@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -35,21 +35,14 @@ class PluginWidget : public QWidget
     Q_OBJECT
 
 public:
-    explicit PluginWidget(const ItemLoaderInterfacePtr &loader, QWidget *parent = 0);
+    explicit PluginWidget(const ItemLoaderPtr &loader, QWidget *parent = nullptr);
     ~PluginWidget();
 
-    ItemLoaderInterfacePtr loader() const { return m_loader; }
-
-    void applySettings(QSettings *settings, bool isPluginEnabled);
-
-protected:
-    void showEvent(QShowEvent *event);
+    const ItemLoaderPtr &loader() const { return m_loader; }
 
 private:
-    void init();
-
     Ui::PluginWidget *ui;
-    ItemLoaderInterfacePtr m_loader;
+    ItemLoaderPtr m_loader;
 };
 
 #endif // PLUGINWIDGET_H
