@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,7 +19,6 @@
 
 #include "gui/shortcutbutton.h"
 
-#include "gui/configurationmanager.h"
 #include "gui/iconfactory.h"
 #include "gui/icons.h"
 #include "gui/shortcutdialog.h"
@@ -28,12 +27,17 @@
 #include <QPushButton>
 #include <QVariant>
 
+namespace {
+
+const char propertyShortcut[] = "CopyQ_shortcut";
+
+} // namespace
+
 ShortcutButton::ShortcutButton(QWidget *parent)
     : QWidget(parent)
     , m_defaultShortcut()
     , m_layout(new QHBoxLayout(this))
     , m_buttonAddShortcut(new QPushButton(this))
-    , m_expectModifier(false)
 {
     m_layout->setMargin(0);
     m_layout->setSpacing(2);
@@ -44,6 +48,7 @@ ShortcutButton::ShortcutButton(QWidget *parent)
     const int h = m_buttonAddShortcut->sizeHint().height();
     m_buttonAddShortcut->setMaximumSize(h, h);
     m_layout->addWidget(m_buttonAddShortcut);
+    setFocusProxy(m_buttonAddShortcut);
 
     connect( m_buttonAddShortcut, SIGNAL(clicked()),
              this, SLOT(onButtonAddShortcutClicked()) );
@@ -56,11 +61,15 @@ void ShortcutButton::addShortcut(const QKeySequence &shortcut)
     if ( shortcut.isEmpty() || shortcuts().contains(shortcut) )
         return;
 
-    QPushButton *button = new QPushButton(this);
-    m_layout->insertWidget( shortcutCount(), button, 1 );
+    auto button = new QPushButton(this);
+    const int buttonIndex = shortcutCount();
+    m_layout->insertWidget(buttonIndex, button, 1);
+
+    setTabOrder(m_buttonAddShortcut, button);
+
     connect( button, SIGNAL(clicked()),
              this, SLOT(onShortcutButtonClicked()) );
-    button->setText( shortcut.toString(QKeySequence::NativeText) );
+    setButtonShortcut(button, shortcut);
     emit shortcutAdded(shortcut);
 }
 
@@ -135,50 +144,57 @@ void ShortcutButton::showEvent(QShowEvent *event)
 void ShortcutButton::onShortcutButtonClicked()
 {
     QPushButton *button = qobject_cast<QPushButton*>(sender());
-    Q_ASSERT(button != NULL);
+    Q_ASSERT(button != nullptr);
     addShortcut(button);
 }
 
 void ShortcutButton::onButtonAddShortcutClicked()
 {
-    addShortcut(NULL);
+    addShortcut(nullptr);
 }
 
 void ShortcutButton::addShortcut(QPushButton *shortcutButton)
 {
     QWidget *parent = this;
     // Destroy shortcut dialog, if its shortcut button is deleted.
-    if (shortcutButton != NULL)
+    if (shortcutButton != nullptr)
         parent = shortcutButton;
 
-    ShortcutDialog *dialog = new ShortcutDialog(parent);
-    dialog->setExpectModifier(m_expectModifier);
+    auto dialog = new ShortcutDialog(parent);
+    if (dialog->exec() == QDialog::Rejected)
+        return;
 
-    if (dialog->exec() == QDialog::Accepted) {
-        const QKeySequence shortcut = dialog->shortcut();
-        const QString text = shortcut.toString(QKeySequence::NativeText);
-        if ( shortcut.isEmpty() || shortcuts().contains(shortcut) ) {
-            if (shortcutButton == NULL || shortcutButton->text() != text ) {
-                if (shortcutButton != NULL) {
-                    const QString shortcut = shortcutButton->text();
-                    delete shortcutButton;
-                    emit shortcutRemoved(shortcut);
-                }
-            }
-        } else {
-            if (shortcutButton != NULL) {
-                if ( shortcutButton->text() != text ) {
-                    emit shortcutRemoved(shortcutButton->text());
-                    shortcutButton->setText(text);
-                    emit shortcutAdded(text);
-                }
-            } else {
-                addShortcut(text);
-            }
+    const QKeySequence newShortcut = dialog->shortcut();
+    const QKeySequence oldShortcut = shortcutButton
+            ? shortcutForButton(*shortcutButton)
+            : QKeySequence();
+
+    if (oldShortcut == newShortcut)
+        return;
+
+    // Remove shortcut button if shortcut is removed, unrecognized or already set.
+    if ( newShortcut.isEmpty() || shortcuts().contains(newShortcut) ) {
+        if (shortcutButton) {
+            delete shortcutButton;
+            emit shortcutRemoved(oldShortcut);
         }
+    } else if (shortcutButton) {
+        emit shortcutRemoved(oldShortcut);
+        setButtonShortcut(shortcutButton, newShortcut);
+        emit shortcutAdded(newShortcut);
+    } else {
+        addShortcut(newShortcut);
     }
 }
 
+void ShortcutButton::setButtonShortcut(QPushButton *shortcutButton, const QKeySequence &shortcut)
+{
+    QString label = shortcut.toString(QKeySequence::NativeText);
+    label.replace( QChar('&'), QString("&&") );
+    shortcutButton->setText(label);
+    shortcutButton->setProperty(propertyShortcut, shortcut);
+}
+
 QWidget *ShortcutButton::shortcutButton(int index) const
 {
     return m_layout->itemAt(index)->widget();
@@ -186,5 +202,5 @@ QWidget *ShortcutButton::shortcutButton(int index) const
 
 QKeySequence ShortcutButton::shortcutForButton(const QWidget &w) const
 {
-    return QKeySequence(w.property("text").toString(), QKeySequence::NativeText);
+    return w.property(propertyShortcut).value<QKeySequence>();
 }
