@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,7 +21,7 @@
 #include "ui_shortcutdialog.h"
 
 #include "common/log.h"
-#include "gui/configurationmanager.h"
+#include "common/shortcuts.h"
 #include "gui/icons.h"
 #include "platform/platformnativeinterface.h"
 
@@ -30,7 +30,7 @@
 
 namespace {
 
-bool isNonModifierKey(int key)
+bool isModifierKey(int key)
 {
     switch(key) {
     case Qt::Key_Control:
@@ -43,9 +43,9 @@ bool isNonModifierKey(int key)
     case Qt::Key_Hyper_L:
     case Qt::Key_Hyper_R:
     case Qt::Key_unknown:
-        return false;
-    default:
         return true;
+    default:
+        return false;
     }
 }
 
@@ -56,7 +56,6 @@ ShortcutDialog::ShortcutDialog(QWidget *parent)
     , ui(new Ui::ShortcutDialog)
     , m_shortcut()
     , m_metaPressed(false)
-    , m_expectModifier(false)
 {
     ui->setupUi(this);
 
@@ -66,6 +65,8 @@ ShortcutDialog::ShortcutDialog(QWidget *parent)
     connect(resetButton, SIGNAL(clicked()), SLOT(onResetButtonClicked()));
 
     ui->lineEditShortcut->installEventFilter(this);
+
+    setAttribute(Qt::WA_InputMethodEnabled, false);
 }
 
 ShortcutDialog::~ShortcutDialog()
@@ -81,47 +82,49 @@ QKeySequence ShortcutDialog::shortcut() const
 bool ShortcutDialog::eventFilter(QObject *object, QEvent *event)
 {
     if (object != ui->lineEditShortcut)
-        return false;
+        return QDialog::eventFilter(object, event);
 
     if (event->type() == QEvent::KeyPress) {
         QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
         COPYQ_LOG(QString("Shortcut key press: %1").arg(keyEvent->key()));
 
         const int key = createPlatformNativeInterface()->keyCode(*keyEvent);
-        Qt::KeyboardModifiers mods = getModifiers(*keyEvent);
+        const int mods = getModifiers(*keyEvent);
 
         if (mods == Qt::NoModifier) {
             if (key == Qt::Key_Tab)
-                return false;
+                return QDialog::eventFilter(object, event);
 
             if (key == Qt::Key_Escape) {
                 reject();
                 return true;
             }
-
-            if (m_expectModifier)
-                return true;
         }
 
         event->accept();
-        processKey(key, mods);
 
-        if ( isNonModifierKey(key) )
+        if ( isModifierKey(keyEvent->key()) ) {
+            processKey(0, mods);
+        } else {
+            processKey(key, mods);
             accept();
+        }
 
-        return false;
-    } else if (event->type() == QEvent::KeyRelease) {
+        return QDialog::eventFilter(object, event);
+    }
+
+    if (event->type() == QEvent::KeyRelease) {
         QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
         COPYQ_LOG(QString("Shortcut key release: %1").arg(keyEvent->key()));
 
-        Qt::KeyboardModifiers mods = getModifiers(*keyEvent);
+        const int mods = getModifiers(*keyEvent);
 
         processKey(0, mods);
 
         return true;
     }
 
-    return false;
+    return QDialog::eventFilter(object, event);
 }
 
 void ShortcutDialog::onResetButtonClicked()
@@ -130,32 +133,25 @@ void ShortcutDialog::onResetButtonClicked()
     accept();
 }
 
-void ShortcutDialog::processKey(int key, Qt::KeyboardModifiers mods)
+void ShortcutDialog::processKey(int key, int mods)
 {
-    int keys = 0;
-    if ( isNonModifierKey(key) )
-        keys = key;
+    m_shortcut = QKeySequence(mods | key);
 
-    if (mods & Qt::ControlModifier)
-        keys += Qt::CTRL;
-    if (mods & Qt::ShiftModifier)
-        keys += Qt::SHIFT;
-    if (mods & Qt::AltModifier)
-        keys += Qt::ALT;
-    if (mods & Qt::MetaModifier)
-        keys += Qt::META;
+    // WORKAROUND: Qt has convert some keys to upper case which
+    //             breaks some shortcuts on some keyboard layouts.
+    m_shortcut = QKeySequence(portableShortcutText(m_shortcut));
 
-    m_shortcut = QKeySequence(keys);
-    QString shortcut = m_shortcut.toString(QKeySequence::NativeText);
-    COPYQ_LOG(QString("Shortcut: %1").arg(m_shortcut.toString()));
+    const QString shortcut = m_shortcut.toString();
+    COPYQ_LOG(QString("Shortcut: %1").arg(shortcut));
 
     ui->lineEditShortcut->setText(shortcut);
 }
 
-Qt::KeyboardModifiers ShortcutDialog::getModifiers(const QKeyEvent &event)
+int ShortcutDialog::getModifiers(const QKeyEvent &event)
 {
     int key = event.key();
-    Qt::KeyboardModifiers mods = event.modifiers();
+    const Qt::KeyboardModifiers mods = event.modifiers();
+    int result = 0;
 
     if (key == Qt::Key_Meta || key == Qt::Key_Super_L || key == Qt::Key_Super_R
             || key == Qt::Key_Hyper_L || key == Qt::Key_Hyper_R)
@@ -163,10 +159,15 @@ Qt::KeyboardModifiers ShortcutDialog::getModifiers(const QKeyEvent &event)
         m_metaPressed = (event.type() == QEvent::KeyPress);
         COPYQ_LOG(QString("Shortcut \"Meta\" key %1.").arg(m_metaPressed ? "pressed" : "released"));
     }
-    if (m_metaPressed)
-        mods |= Qt::MetaModifier;
-    else
-        mods &= ~Qt::MetaModifier;
 
-    return mods;
+    if (mods & Qt::ShiftModifier)
+        result |= Qt::SHIFT;
+    if (mods & Qt::ControlModifier)
+        result |= Qt::CTRL;
+    if (mods & Qt::AltModifier)
+        result |= Qt::ALT;
+    if (m_metaPressed || mods & Qt::MetaModifier)
+        result |= Qt::META;
+
+    return result;
 }
