@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,39 +20,56 @@
 #include "tabtree.h"
 
 #include "common/common.h"
+#include "common/display.h"
 #include "common/mimetypes.h"
-#include "gui/configurationmanager.h"
 #include "gui/iconfactory.h"
 #include "gui/iconfont.h"
+#include "gui/tabicons.h"
 
+#include <QApplication>
 #include <QLabel>
 #include <QList>
 #include <QMimeData>
 #include <QMouseEvent>
 #include <QScrollBar>
 #include <QHBoxLayout>
+#include <QTreeWidgetItemIterator>
 
 namespace {
 
 enum {
-    DataIndex = Qt::UserRole,
-    DataText,
+    DataText = Qt::UserRole,
     DataItemCount
 };
 
 void updateItemSize(QTreeWidgetItem *item)
 {
+    QSize size(0, 0);
+
     QWidget *w = item->treeWidget()->itemWidget(item, 0);
-    item->setSizeHint( 0, w ? w->minimumSizeHint() : QSize(0,0) );
+
+    if (w) {
+        size = w->minimumSizeHint();
+
+        if (!item->icon(0).isNull()) {
+            const QSize iconSize = item->treeWidget()->iconSize();
+            size = QSize(
+                        size.width() + iconSize.width() + 8,
+                        qMax(size.height(), iconSize.height())
+                        );
+        }
+    }
+
+    item->setSizeHint(0, size);
 }
 
 void setItemWidgetSelected(QTreeWidgetItem *item)
 {
-    if (item == NULL)
+    if (item == nullptr)
         return;
 
     QTreeWidget *parent = item->treeWidget();
-    if (parent == NULL)
+    if (parent == nullptr)
         return;
 
     QWidget *w = parent->itemWidget(item, 0);
@@ -64,7 +81,7 @@ void setItemWidgetSelected(QTreeWidgetItem *item)
 
         bool selected = parent->currentItem() == item;
 
-        foreach (QWidget *child, w->findChildren<QWidget *>()) {
+        for (auto child : w->findChildren<QWidget *>()) {
             child->setProperty("CopyQ_selected", selected);
             style->unpolish(child);
             style->polish(child);
@@ -76,7 +93,7 @@ void setItemWidgetSelected(QTreeWidgetItem *item)
 
 QTreeWidgetItem *findLastTreeItem(const QTreeWidget &tree, QStringList *pathComponents)
 {
-    QTreeWidgetItem *parentItem = NULL;
+    QTreeWidgetItem *parentItem = nullptr;
 
     if ( !pathComponents->isEmpty() ) {
         const QString &text = pathComponents->first();
@@ -89,12 +106,12 @@ QTreeWidgetItem *findLastTreeItem(const QTreeWidget &tree, QStringList *pathComp
         }
     }
 
-    if (parentItem != NULL) {
+    if (parentItem != nullptr) {
         pathComponents->pop_front();
 
         while ( !pathComponents->isEmpty() ) {
             const QString &text = pathComponents->first();
-            QTreeWidgetItem *item = NULL;
+            QTreeWidgetItem *item = nullptr;
 
             for (int i = 0; i < parentItem->childCount(); ++i) {
                 if ( parentItem->child(i)->data(0, DataText).toString() == text ) {
@@ -103,7 +120,7 @@ QTreeWidgetItem *findLastTreeItem(const QTreeWidget &tree, QStringList *pathComp
                 }
             }
 
-            if (item == NULL)
+            if (item == nullptr)
                 break;
 
             parentItem = item;
@@ -114,14 +131,9 @@ QTreeWidgetItem *findLastTreeItem(const QTreeWidget &tree, QStringList *pathComp
     return parentItem;
 }
 
-bool canDrop(const QMimeData &data)
-{
-    return data.hasFormat(mimeItems) || data.hasText() || data.hasImage() || data.hasUrls();
-}
-
 QTreeWidgetItem *dropItemsTarget(const QDropEvent &event, const QTreeWidget &parent)
 {
-    return canDrop( *event.mimeData() ) ? parent.itemAt( event.pos() ) : NULL;
+    return canDropToTab(event) ? parent.itemAt( event.pos() ) : nullptr;
 }
 
 int itemLabelPadding()
@@ -143,30 +155,38 @@ public:
     explicit ItemLabel(QTreeWidgetItem *item)
         : QWidget(item->treeWidget())
         , m_treeWidget(item->treeWidget())
-        , m_labelItemCount(NULL)
+        , m_label(createLabel("tab_tree_item", this))
+        , m_labelItemCount(nullptr)
         , m_layout(new QHBoxLayout(this))
     {
-        const QString text = item->data(0, DataText).toString();
-        const QString itemCount = item->data(0, DataItemCount).toString();
+        m_label->setBuddy(m_treeWidget);
+        m_label->installEventFilter(this);
 
-        QLabel *label = createLabel("tab_tree_item", this);
-        label->setText(text);
-        label->setBuddy(m_treeWidget);
-        label->installEventFilter(this);
-
-        m_layout->addWidget(label);
+        m_layout->addWidget(m_label);
         m_layout->setMargin(0);
         m_layout->addStretch(1);
 
-        if ( !itemCount.isEmpty() )
-            setItemCountLabel(itemCount);
+        updateFromItem(item);
+    }
+
+    void updateFromItem(QTreeWidgetItem *item)
+    {
+        const QString text = item->data(0, DataText).toString();
+        const QString itemCount = item->data(0, DataItemCount).toString();
+        setText(text);
+        setItemCountLabel(itemCount);
+    }
+
+    void setText(const QString &text)
+    {
+        m_label->setText(text);
     }
 
     void setItemCountLabel(const QString &itemCount)
     {
         if (itemCount.isEmpty()) {
             delete m_labelItemCount;
-            m_labelItemCount = NULL;
+            m_labelItemCount = nullptr;
         } else {
             if (!m_labelItemCount) {
                 m_labelItemCount = createLabel("tab_item_counter", this);
@@ -180,10 +200,11 @@ public:
     }
 
 protected:
-    bool eventFilter(QObject *, QEvent *event)
+    bool eventFilter(QObject *, QEvent *event) override
     {
         if ( event->type() == QEvent::Shortcut ) {
-            foreach ( QTreeWidgetItem *item, m_treeWidget->findItems(QString(), Qt::MatchContains | Qt::MatchRecursive) ) {
+            for ( QTreeWidgetItemIterator it(m_treeWidget->topLevelItem(0)); *it; ++it ) {
+                auto item = *it;
                 if ( m_treeWidget->itemWidget(item, 0) == this ) {
                     m_treeWidget->setCurrentItem(item);
                     return true;
@@ -196,14 +217,26 @@ protected:
 
 private:
     QTreeWidget *m_treeWidget;
+    QLabel *m_label;
     QLabel *m_labelItemCount;
     QHBoxLayout *m_layout;
 };
 
+ItemLabel *itemLabel(QTreeWidgetItem *item)
+{
+    return static_cast<ItemLabel*>( item->treeWidget()->itemWidget(item, 0) );
+}
+
 void labelItem(QTreeWidgetItem *item)
 {
+    ItemLabel *label = itemLabel(item);
+    if (label) {
+        label->updateFromItem(item);
+        return;
+    }
+
     QTreeWidget *parent = item->treeWidget();
-    ItemLabel *label = new ItemLabel(item);
+    label = new ItemLabel(item);
     label->installEventFilter(parent);
     item->setTextAlignment(0, Qt::AlignCenter);
     parent->setItemWidget(item, 0, label);
@@ -211,6 +244,12 @@ void labelItem(QTreeWidgetItem *item)
     setItemWidgetSelected(item);
 }
 
+bool isInside(QWidget *child, QWidget *parent)
+{
+    const QPoint scrollBarPosition = child->mapTo(parent, QPoint(0,0));
+    return parent->contentsRect().contains(scrollBarPosition);
+}
+
 } // namespace
 
 TabTree::TabTree(QWidget *parent)
@@ -228,7 +267,7 @@ TabTree::TabTree(QWidget *parent)
     setHeaderHidden(true);
     setSelectionMode(QAbstractItemView::SingleSelection);
 
-    const int x = iconFontSizePixels() + 4;
+    const int x = smallIconSize();
     setIconSize(QSize(x, x));
 
     setMinimumHeight(fontMetrics().lineSpacing() * 3);
@@ -238,17 +277,92 @@ TabTree::TabTree(QWidget *parent)
 
     connect( this, SIGNAL(itemCollapsed(QTreeWidgetItem*)), SLOT(updateSize()) );
     connect( this, SIGNAL(itemExpanded(QTreeWidgetItem*)), SLOT(updateSize()) );
+
+    initSingleShotTimer( &m_timerUpdate, 0, this, SLOT(doUpdateSize()) );
 }
 
-void TabTree::insertTab(const QString &path, int index, bool selected)
+QString TabTree::getCurrentTabPath() const
+{
+    return getTabPath( currentItem() );
+}
+
+bool TabTree::isTabGroup(const QString &tab) const
+{
+    return isTabGroup( findTreeItem(tab) );
+}
+
+QString TabTree::tabName(int tabIndex) const
+{
+    return getTabPath( findTreeItem(tabIndex) );
+}
+
+void TabTree::setTabName(int tabIndex, const QString &tabName)
+{
+    QTreeWidgetItem *item = findTreeItem(tabIndex);
+    Q_ASSERT(item);
+
+    if (getTabPath(item) == tabName)
+        return;
+
+    const QString itemCount = item->data(0, DataItemCount).toString();
+    insertTab(tabIndex, tabName);
+    if (item == currentItem())
+        setCurrentTab(tabIndex);
+
+    // Remove old item if it's an empty group.
+    m_tabs.removeOne(item);
+    if ( isEmptyTabGroup(item) )
+        deleteItem(item);
+
+    item = findTreeItem(tabIndex);
+    Q_ASSERT(item);
+    Q_ASSERT(getTabPath(item) == tabName);
+
+    if ( !itemCount.isEmpty() )
+        setTabItemCount(tabName, itemCount);
+
+    updateItemSize(item);
+    updateSize();
+}
+
+void TabTree::setTabItemCount(const QString &tabName, const QString &itemCount)
+{
+    QTreeWidgetItem *item = findTreeItem(tabName);
+    if (!item)
+        return;
+
+    item->setData(0, DataItemCount, itemCount);
+
+    ItemLabel *label = itemLabel(item);
+    Q_ASSERT(label);
+    label->setItemCountLabel(itemCount);
+
+    updateItemSize(item);
+    updateSize();
+}
+
+void TabTree::updateTabIcon(const QString &tabName)
+{
+    QTreeWidgetItem *item = findTreeItem(tabName);
+    if (!item)
+        return;
+
+    const QIcon icon = getIconForTabName(tabName);
+    item->setIcon(0, icon);
+    updateItemSize(item);
+    updateSize();
+}
+
+void TabTree::insertTab(int index, const QString &path)
 {
     QStringList pathComponents = path.split('/');
     QTreeWidgetItem *item = findLastTreeItem(*this, &pathComponents);
+    const bool selectTab = topLevelItemCount() == 0;
 
-    foreach (const QString &text, pathComponents) {
+    for (const auto &text : pathComponents) {
         QTreeWidgetItem *parent = item;
 
-        if (parent != NULL) {
+        if (parent != nullptr) {
             int to = 0;
             for ( ; to < parent->childCount(); ++to ) {
                  const int index2 = getTabIndex(parent->child(to));
@@ -273,19 +387,18 @@ void TabTree::insertTab(const QString &path, int index, bool selected)
         }
 
         item->setExpanded(true);
-        item->setData(0, DataIndex, -1);
         item->setData(0, DataText, text);
 
-        const QIcon icon = ConfigurationManager::instance()->getIconForTabName( getTabPath(item) );
+        const QIcon icon = getIconForTabName( getTabPath(item) );
         item->setIcon(0, icon);
 
         labelItem(item);
     }
 
-    Q_ASSERT(item != NULL);
-    item->setData(0, DataIndex, index);
+    Q_ASSERT(item != nullptr);
+    m_tabs.insert(index, item);
 
-    if (selected)
+    if (selectTab)
         setCurrentItem(item);
 
     updateSize();
@@ -294,39 +407,113 @@ void TabTree::insertTab(const QString &path, int index, bool selected)
 void TabTree::removeTab(int index)
 {
     QTreeWidgetItem *item = findTreeItem(index);
-    if (item == NULL)
+    if (item == nullptr)
         return;
 
+    m_tabs.removeOne(item);
     if (item->childCount() == 0)
         deleteItem(item);
-    else
-        item->setData(0, DataIndex, -1);
 
-    shiftIndexesBetween(index + 1);
+    updateSize();
+}
+
+void TabTree::moveTab(int from, int to)
+{
+    if (from == to)
+        return;
+
+    QTreeWidgetItem *item = findTreeItem(from);
+    if (item == nullptr)
+        return;
+
+    m_tabs.removeOne(item);
+    m_tabs.insert(to, item);
+}
+
+void TabTree::updateCollapsedTabs(QStringList *tabs) const
+{
+    tabs->clear();
+    for ( QTreeWidgetItemIterator it(topLevelItem(0)); *it; ++it ) {
+        auto item = *it;
+        if ( isTabGroup(item) && !item->isExpanded() )
+            tabs->append( getTabPath(item) );
+    }
+}
+
+void TabTree::setCollapsedTabs(const QStringList &collapsedPaths)
+{
+    for (const auto &path : collapsedPaths) {
+        QTreeWidgetItem *item = findTreeItem(path);
+        if ( isTabGroup(item) )
+            item->setExpanded(false);
+    }
+}
+
+void TabTree::updateTabIcons()
+{
+    for ( QTreeWidgetItemIterator it(topLevelItem(0)); *it; ++it )
+        updateTabIcon( getTabPath(*it) );
+}
+
+void TabTree::nextTab()
+{
+    QTreeWidgetItem *item = currentItem();
+    if (item != nullptr)
+        item = itemBelow(item);
+
+    if (item == nullptr)
+        item = topLevelItem(0);
+
+    if (item != nullptr)
+        setCurrentItem(item);
+}
+
+void TabTree::previousTab()
+{
+    QTreeWidgetItem *item = currentItem();
+    if (item != nullptr)
+        item = itemAbove(item);
+
+    if (item == nullptr) {
+        item = topLevelItem( topLevelItemCount() - 1 );
+        while ( isTabGroup(item) && item->isExpanded() )
+            item = item->child( item->childCount() - 1 );
+    }
+
+    if (item != nullptr)
+        setCurrentItem(item);
+}
+
+void TabTree::setCurrentTab(int index)
+{
+    if (index < 0)
+        return;
+
+    QTreeWidgetItem *item = findTreeItem(index);
+    if (item != nullptr)
+        setCurrentItem(item);
+}
+
+void TabTree::adjustSize()
+{
     updateSize();
 }
 
 QTreeWidgetItem *TabTree::findTreeItem(int index) const
 {
-    QList<QTreeWidgetItem *> items = findItems(QString(), Qt::MatchContains | Qt::MatchRecursive);
-    for (int i = items.size() - 1; i >= 0; --i) {
-        if ( getTabIndex(items[i]) == index )
-            return items[i];
-    }
-
-    return NULL;
+    return m_tabs.value(index);
 }
 
 QTreeWidgetItem *TabTree::findTreeItem(const QString &path) const
 {
     QStringList pathComponents = path.split('/');
     QTreeWidgetItem *parentItem = findLastTreeItem(*this, &pathComponents);
-    return pathComponents.isEmpty() ? parentItem : NULL;
+    return pathComponents.isEmpty() ? parentItem : nullptr;
 }
 
 int TabTree::getTabIndex(const QTreeWidgetItem *item) const
 {
-    return (item == NULL) ? -1 : item->data(0, DataIndex).toInt();
+    return (item == nullptr) ? -1 : m_tabs.indexOf( const_cast<QTreeWidgetItem*>(item) );
 }
 
 QString TabTree::getTabPath(const QTreeWidgetItem *item) const
@@ -334,7 +521,7 @@ QString TabTree::getTabPath(const QTreeWidgetItem *item) const
     QString result;
     const QTreeWidgetItem *parent = item;
 
-    while (parent != NULL) {
+    while (parent != nullptr) {
         const QString part = parent->data(0, DataText).toString();
         result.prepend('/');
         result.prepend(part);
@@ -348,7 +535,7 @@ QString TabTree::getTabPath(const QTreeWidgetItem *item) const
 
 bool TabTree::isTabGroup(const QTreeWidgetItem *item) const
 {
-    return item != NULL && item->childCount() > 0;
+    return item != nullptr && item->childCount() > 0;
 }
 
 bool TabTree::isEmptyTabGroup(const QTreeWidgetItem *item) const
@@ -356,122 +543,11 @@ bool TabTree::isEmptyTabGroup(const QTreeWidgetItem *item) const
     return item->childCount() == 0 && getTabIndex(item) < 0;
 }
 
-void TabTree::moveTab(int from, int to)
-{
-    if (from == to)
-        return;
-
-    QTreeWidgetItem *item = findTreeItem(from);
-    if (item == NULL)
-        return;
-
-    if (from < to)
-        shiftIndexesBetween(from + 1, to, -1);
-    else
-        shiftIndexesBetween(to, from - 1, 1);
-
-    item->setData(0, DataIndex, to);
-}
-
-void TabTree::setTabText(int tabIndex, const QString &tabText)
-{
-    QTreeWidgetItem *item = findTreeItem(tabIndex);
-    Q_ASSERT(item);
-
-    if (getTabPath(item) == tabText)
-        return;
-
-    const QString itemCount = item->data(0, DataItemCount).toString();
-    bool isCurrent = item == currentItem();
-
-    insertTab(tabText, tabIndex, isCurrent);
-
-    // Remove old item if it's an empty group.
-    item->setData(0, DataIndex, -1);
-    if ( isEmptyTabGroup(item) )
-        deleteItem(item);
-
-    item = findTreeItem(tabIndex);
-    Q_ASSERT(item);
-    Q_ASSERT(getTabPath(item) == tabText);
-
-    if ( !itemCount.isEmpty() )
-        setTabItemCount(tabText, itemCount);
-
-    updateItemSize(item);
-    updateSize();
-}
-
-void TabTree::setTabItemCount(const QString &tabName, const QString &itemCount)
-{
-    QTreeWidgetItem *item = findTreeItem(tabName);
-    if (!item)
-        return;
-
-    item->setData(0, DataItemCount, itemCount);
-
-    ItemLabel *label = static_cast<ItemLabel*>( itemWidget(item, 0) );
-    Q_ASSERT(label);
-    label->setItemCountLabel(itemCount);
-
-    updateItemSize(item);
-    updateSize();
-}
-
-void TabTree::setCollapsedTabs(const QStringList &collapsedPaths)
-{
-    foreach (const QString &path, collapsedPaths) {
-        QTreeWidgetItem *item = findTreeItem(path);
-        if ( isTabGroup(item) )
-            item->setExpanded(false);
-    }
-}
-
-QStringList TabTree::collapsedTabs() const
-{
-    QStringList tabs;
-
-    foreach ( QTreeWidgetItem *item, findItems(QString(), Qt::MatchContains | Qt::MatchRecursive) ) {
-        if ( isTabGroup(item) && !item->isExpanded() )
-            tabs.append( getTabPath(item) );
-    }
-
-    return tabs;
-}
-
 QSize TabTree::sizeHint() const
 {
     return minimumSizeHint();
 }
 
-void TabTree::updateTabIcon(const QString &tabName)
-{
-    QTreeWidgetItem *item = findTreeItem(tabName);
-    if (!item)
-        return;
-
-    const QIcon icon = ConfigurationManager::instance()->getIconForTabName(tabName);
-    item->setIcon(0, icon);
-    updateItemSize(item);
-    updateSize();
-}
-
-void TabTree::updateTabIcons()
-{
-    foreach ( QTreeWidgetItem *item, findItems(QString(), Qt::MatchContains | Qt::MatchRecursive) )
-        updateTabIcon( getTabPath(item) );
-}
-
-void TabTree::mousePressEvent(QMouseEvent *event)
-{
-    if ( event->button() == Qt::RightButton ) {
-        requestTabMenu(event->pos(), event->globalPos());
-        event->accept();
-    } else {
-        QTreeWidget::mousePressEvent(event);
-    }
-}
-
 void TabTree::contextMenuEvent(QContextMenuEvent *event)
 {
     requestTabMenu(event->pos(), event->globalPos());
@@ -480,16 +556,19 @@ void TabTree::contextMenuEvent(QContextMenuEvent *event)
 
 void TabTree::dragEnterEvent(QDragEnterEvent *event)
 {
-    if ( canDrop(*event->mimeData()) )
-        event->acceptProposedAction();
-    else
+    if ( canDropToTab(*event) ) {
+        acceptDrag(event);
+    } else {
         QTreeWidget::dragEnterEvent(event);
+        // Workaround for QTBUG-44939 (Qt 5.4): Don't ignore successive drag move events.
+        event->acceptProposedAction();
+    }
 }
 
 void TabTree::dragMoveEvent(QDragMoveEvent *event)
 {
     if ( dropItemsTarget(*event, *this) )
-        event->acceptProposedAction();
+        acceptDrag(event);
     else if ( itemAt(event->pos()) )
         QTreeWidget::dragMoveEvent(event);
     else
@@ -498,13 +577,14 @@ void TabTree::dragMoveEvent(QDragMoveEvent *event)
 
 void TabTree::dropEvent(QDropEvent *event)
 {
-    QTreeWidgetItem *current = currentItem();
-    if (current == NULL)
+    const auto current = currentItem();
+    if (current == nullptr)
         return;
 
-    QTreeWidgetItem *item = dropItemsTarget(*event, *this);
-    if (item) {
-        emit dropItems( getTabPath(item), event );
+    const auto targetItem = dropItemsTarget(*event, *this);
+    if (targetItem) {
+        acceptDrag(event);
+        emit dropItems( getTabPath(targetItem), event->mimeData() );
     } else if ( itemAt(event->pos()) ) {
         const QString oldPrefix = getTabPath(current);
 
@@ -515,36 +595,39 @@ void TabTree::dropEvent(QDropEvent *event)
         blockSignals(false);
 
         // Rename moved item if non-unique.
-        QStringList tabs;
-        QTreeWidgetItem *parent = current->parent();
+        QStringList uniqueTabNames;
+        const auto parent = current->parent();
         for (int i = 0, count = parent ? parent->childCount() : topLevelItemCount(); i < count; ++i) {
             QTreeWidgetItem *sibling = parent ? parent->child(i) : topLevelItem(i);
             if (sibling != current)
-                tabs.append( getTabPath(sibling) );
+                uniqueTabNames.append( getTabPath(sibling) );
         }
 
-        QString newPrefix = getTabPath(current);
-        if ( tabs.contains(newPrefix) ) {
-            renameToUnique(&newPrefix, tabs);
+        auto newPrefix = getTabPath(current);
+        if ( uniqueTabNames.contains(newPrefix) ) {
+            renameToUnique(&newPrefix, uniqueTabNames);
             const QString text = newPrefix.mid( newPrefix.lastIndexOf(QChar('/')) + 1 );
             current->setData(0, DataText, text);
             labelItem(current);
         }
 
+        QList<QTreeWidgetItem*> newTabs;
         QList<int> indexes;
-        foreach ( QTreeWidgetItem *item, findItems(QString(), Qt::MatchContains | Qt::MatchRecursive) ) {
+        for ( QTreeWidgetItemIterator it(topLevelItem(0)); *it; ++it ) {
+            auto item = *it;
             // Remove empty groups.
             if ( isEmptyTabGroup(item) ) {
                 deleteItem(item);
             } else {
                 const int oldIndex = getTabIndex(item);
-                if (oldIndex >= 0) {
-                    item->setData(0, DataIndex, indexes.size());
+                if (oldIndex != -1) {
+                    newTabs.append(item);
                     indexes.append(oldIndex);
                 }
             }
         }
 
+        m_tabs = std::move(newTabs);
         emit tabsMoved(oldPrefix, newPrefix, indexes);
 
         updateSize();
@@ -567,8 +650,10 @@ void TabTree::rowsInserted(const QModelIndex &parent, int start, int end)
 {
     QTreeWidget::rowsInserted(parent, start, end);
 
+    QList<QTreeWidgetItem *> items;
+
     for (int row = start; row <= end; ++row) {
-        QList<QTreeWidgetItem *> items;
+        items.clear();
         items.append( parent.isValid() ? itemFromIndex(parent.child(row, 0))
                                        : topLevelItem(row) );
         while ( !items.isEmpty() ) {
@@ -598,78 +683,41 @@ void TabTree::onCurrentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *pr
 
 void TabTree::updateSize()
 {
+    m_timerUpdate.start();
+}
+
+void TabTree::doUpdateSize()
+{
+    doItemsLayout();
+
     const QMargins margins = contentsMargins();
     int w = margins.left() + margins.right();
 
-    if ( verticalScrollBar()->isVisible() )
+    // Some styles put scrollbar outside of parent's contents so there is no need
+    // to resize parent widget.
+    if ( verticalScrollBar()->isVisible() && isInside(verticalScrollBar(), this) )
         w += verticalScrollBar()->width();
 
     resizeColumnToContents(0);
-    w += sizeHintForColumn(0) + 4;
+    w += sizeHintForColumn(0);
+    resizeColumnToContents(1);
+    w += sizeHintForColumn(1);
 
     setFixedWidth(w);
 }
 
-void TabTree::requestTabMenu(const QPoint &itemPosition, const QPoint &menuPosition)
+void TabTree::requestTabMenu(QPoint itemPosition, QPoint menuPosition)
 {
     QTreeWidgetItem *item = itemAt(itemPosition);
     QString tabPath = getTabPath(item);
     emit tabMenuRequested(menuPosition, tabPath);
 }
 
-void TabTree::shiftIndexesBetween(int from, int to, int how)
-{
-    foreach ( QTreeWidgetItem *item, findItems(QString(), Qt::MatchContains | Qt::MatchRecursive) ) {
-        const int oldIndex = getTabIndex(item);
-        if (oldIndex >= from && (to == -1 || oldIndex <= to))
-            item->setData(0, DataIndex, oldIndex + how);
-    }
-}
-
-void TabTree::setCurrentTabIndex(int index)
-{
-    if (index < 0)
-        return;
-
-    QTreeWidgetItem *item = findTreeItem(index);
-    if (item != NULL)
-        setCurrentItem(item);
-}
-
-void TabTree::nextTreeItem()
-{
-    QTreeWidgetItem *item = currentItem();
-    if (item != NULL)
-        item = itemBelow(item);
-
-    if (item == NULL)
-        item = topLevelItem(0);
-
-    if (item != NULL)
-        setCurrentItem(item);
-}
-
-void TabTree::previousTreeItem()
-{
-    QTreeWidgetItem *item = currentItem();
-    if (item != NULL)
-        item = itemAbove(item);
-
-    if (item == NULL) {
-        item = topLevelItem( topLevelItemCount() - 1 );
-        while ( isTabGroup(item) && item->isExpanded() )
-            item = item->child( item->childCount() - 1 );
-    }
-
-    if (item != NULL)
-        setCurrentItem(item);
-}
-
 void TabTree::deleteItem(QTreeWidgetItem *item)
 {
     // Recursively remove empty parent item.
     QTreeWidgetItem *parent = item->parent();
-    while (parent != NULL && parent->childCount() == 1 && getTabIndex(parent) < 0) {
+    while (parent != nullptr && parent->childCount() == 1 && getTabIndex(parent) < 0) {
         item = parent;
         parent = item->parent();
     }
