@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -24,6 +24,7 @@
 #include "common/config.h"
 
 #include <QAction>
+#include <QBoxLayout>
 #include <QEvent>
 #include <QMainWindow>
 #include <QPoint>
@@ -42,7 +43,7 @@ void addTabAction(QWidget *widget, const QKeySequence &shortcut,
                   QWidget *receiver, const char *slot,
                   Qt::ShortcutContext context = Qt::WindowShortcut)
 {
-    QAction *act = new QAction(widget);
+    auto act = new QAction(widget);
     act->setShortcut(shortcut);
     act->setShortcutContext(context);
     receiver->connect(act, SIGNAL(triggered()), slot);
@@ -55,9 +56,7 @@ TabWidget::TabWidget(QWidget *parent)
     : QWidget(parent)
     , m_toolBar(new QToolBar(this))
     , m_toolBarTree(new QToolBar(this))
-    , m_tabBar(NULL)
-    , m_tabTree(NULL)
-    , m_stackedWidget(NULL)
+    , m_stackedWidget(nullptr)
     , m_hideTabBar(false)
     , m_showTabItemCount(false)
 {
@@ -72,7 +71,7 @@ TabWidget::TabWidget(QWidget *parent)
     connect( m_toolBar, SIGNAL(orientationChanged(Qt::Orientation)),
              this, SLOT(onToolBarOrientationChanged(Qt::Orientation)) );
 
-    QBoxLayout *layout = new QBoxLayout(QBoxLayout::TopToBottom, this);
+    auto layout = new QBoxLayout(QBoxLayout::TopToBottom, this);
     setLayout(layout);
     layout->setContentsMargins(0, 0, 0, 0);
     layout->setSpacing(0);
@@ -90,17 +89,18 @@ TabWidget::TabWidget(QWidget *parent)
 
 QString TabWidget::getCurrentTabPath() const
 {
-    return isTreeModeEnabled() ? m_tabTree->getTabPath( m_tabTree->currentItem() ) : QString();
+    return m_tabs->getCurrentTabPath();
 }
 
 bool TabWidget::isTabGroup(const QString &tab) const
 {
-    return isTreeModeEnabled() && m_tabTree->isTabGroup( m_tabTree->findTreeItem(tab) );
+    return m_tabs->isTabGroup(tab);
 }
 
-bool TabWidget::isTreeModeEnabled() const
+bool TabWidget::isTabGroupSelected() const
 {
-    return m_tabTree != NULL;
+    QWidget *w = currentWidget();
+    return w != nullptr && w->isHidden();
 }
 
 int TabWidget::currentIndex() const
@@ -108,7 +108,7 @@ int TabWidget::currentIndex() const
     return m_stackedWidget->currentIndex();
 }
 
-QWidget *TabWidget::widget(int tabIndex)
+QWidget *TabWidget::widget(int tabIndex) const
 {
     return m_stackedWidget->widget(tabIndex);
 }
@@ -118,78 +118,67 @@ int TabWidget::count() const
     return m_stackedWidget->count();
 }
 
-QString TabWidget::tabText(int tabIndex) const
+QString TabWidget::tabName(int tabIndex) const
 {
-    return isTreeModeEnabled() ? m_tabTree->getTabPath( m_tabTree->findTreeItem(tabIndex) )
-                               : m_tabBar->tabText(tabIndex);
+    return m_tabs->tabName(tabIndex);
 }
 
-void TabWidget::setTabText(int tabIndex, const QString &tabName)
+void TabWidget::setTabName(int tabIndex, const QString &tabName)
 {
-    const QString oldTabName = tabText(tabIndex);
+    const QString oldTabName = this->tabName(tabIndex);
     if ( m_tabItemCounters.contains(oldTabName) )
         m_tabItemCounters.insert( tabName, m_tabItemCounters.take(oldTabName) );
 
-    if ( isTreeModeEnabled() )
-        m_tabTree->setTabText(tabIndex, tabName);
-    else
-        m_tabBar->setTabText(tabIndex, tabName);
+    m_tabs->setTabName(tabIndex, tabName);
 
-    updateSize();
+    m_tabs->adjustSize();
 }
 
 void TabWidget::setTabItemCountVisible(bool visible)
 {
     m_showTabItemCount = visible;
     for (int i = 0; i < count(); ++i)
-        updateTabItemCount( tabText(i) );
+        updateTabItemCount( tabName(i) );
 
-    updateSize();
+    m_tabs->adjustSize();
 }
 
 void TabWidget::updateTabIcon(const QString &tabName)
 {
-    if ( isTreeModeEnabled() )
-        m_tabTree->updateTabIcon(tabName);
-    else
-        m_tabBar->updateTabIcon(tabName);
+    m_tabs->updateTabIcon(tabName);
 }
 
-void TabWidget::insertTab(int tabIndex, QWidget *widget, const QString &tabText)
+void TabWidget::insertTab(int tabIndex, QWidget *widget, const QString &tabName)
 {
-    bool firstTab = count() == 0;
+    const bool firstTab = count() == 0;
     m_stackedWidget->insertWidget(tabIndex, widget);
 
-    if ( isTreeModeEnabled() ) {
-        m_tabTree->insertTab(tabText, tabIndex, firstTab);
-        m_toolBarTree->layout()->setSizeConstraint(QLayout::SetMinAndMaxSize);
-    } else {
-        m_tabBar->insertTab(tabIndex, tabText);
-        m_toolBar->layout()->setSizeConstraint(QLayout::SetMinAndMaxSize);
-    }
+    m_tabs->insertTab(tabIndex, tabName);
+    m_toolBarCurrent->layout()->setSizeConstraint(QLayout::SetMinAndMaxSize);
 
     if (firstTab)
         emit currentChanged(0, -1);
 
-    updateTabItemCount(tabText);
+    updateTabItemCount(tabName);
     updateToolBar();
 }
 
 void TabWidget::removeTab(int tabIndex)
 {
     if (tabIndex == currentIndex())
-        setCurrentIndex(0);
+        setCurrentIndex(tabIndex == 0 ? 1 : 0);
 
-    m_tabItemCounters.remove( tabText(tabIndex) );
+    const QString tabName = this->tabName(tabIndex);
+    m_tabItemCounters.remove(tabName);
+
+    // Item count must be updated If tab is removed but tab group remains.
+    m_tabs->setTabItemCount(tabName, QString());
 
     QWidget *w = m_stackedWidget->widget(tabIndex);
     m_stackedWidget->removeWidget(w);
     delete w;
 
-    if ( isTreeModeEnabled() )
-        m_tabTree->removeTab(tabIndex);
-    else
-        m_tabBar->removeTab(tabIndex);
+    m_tabs->removeTab(tabIndex);
 
     updateToolBar();
 }
@@ -197,9 +186,9 @@ void TabWidget::removeTab(int tabIndex)
 QStringList TabWidget::tabs() const
 {
     QStringList tabs;
-
+    tabs.reserve( count() );
     for( int i = 0; i < count(); ++i )
-        tabs.append( tabText(i) );
+        tabs.append( tabName(i) );
 
     return tabs;
 }
@@ -211,10 +200,7 @@ void TabWidget::moveTab(int from, int to)
     Q_ASSERT(to < count());
     Q_ASSERT(from < count());
 
-    if ( isTreeModeEnabled() )
-        m_tabTree->moveTab(from, to);
-    else
-        m_tabBar->moveTab(from, to);
+    m_tabs->moveTab(from, to);
 
     bool isCurrent = currentIndex() == from;
 
@@ -233,18 +219,17 @@ void TabWidget::addToolBars(QMainWindow *mainWindow)
 void TabWidget::saveTabInfo()
 {
     QStringList tabs;
+    tabs.reserve( count() );
     for ( int i = 0; i < count(); ++i )
-        tabs.append( tabText(i) );
+        tabs.append( tabName(i) );
 
     QSettings settings(getTabWidgetConfigurationFilePath(), QSettings::IniFormat);
 
-    if ( isTreeModeEnabled() )
-        m_collapsedTabs = m_tabTree->collapsedTabs();
-
+    m_tabs->updateCollapsedTabs(&m_collapsedTabs);
     settings.setValue("TabWidget/collapsed_tabs", m_collapsedTabs);
 
     QVariantMap tabItemCounters;
-    foreach (QString key, tabs) {
+    for (const auto &key : tabs) {
         const int count = m_tabItemCounters.value(key, -1);
         if (count >= 0)
             tabItemCounters[key] = count;
@@ -260,24 +245,23 @@ void TabWidget::loadTabInfo()
 
     QVariantMap tabItemCounters = settings.value("TabWidget/tab_item_counters").toMap();
     m_tabItemCounters.clear();
-    foreach (QString key, tabItemCounters.keys())
-        m_tabItemCounters[key] = tabItemCounters[key].toInt();
+    for (auto it = tabItemCounters.constBegin(); it != tabItemCounters.constEnd(); ++it)
+        m_tabItemCounters[it.key()] = it.value().toInt();
 }
 
 void TabWidget::updateTabs()
 {
-    if (isTreeModeEnabled()) {
-        m_tabTree->setCollapsedTabs(m_collapsedTabs);
-        m_tabTree->updateTabIcons();
-    } else {
-        m_tabBar->updateTabIcons();
-    }
+    m_tabs->setCollapsedTabs(m_collapsedTabs);
+    m_tabs->updateTabIcons();
 }
 
 void TabWidget::setCurrentIndex(int tabIndex)
 {
+    if (m_ignoreCurrentTabChanges)
+        return;
+
     QWidget *w = currentWidget();
-    const int current = (isTreeModeEnabled() && w != NULL && w->isHidden()) ? -1 : currentIndex();
+    const int current = isTabGroupSelected() ? -1 : currentIndex();
 
     if (tabIndex == current)
         return;
@@ -286,20 +270,17 @@ void TabWidget::setCurrentIndex(int tabIndex)
         m_stackedWidget->setCurrentIndex(tabIndex);
 
         w = currentWidget();
-        if (w == NULL)
+        if (w == nullptr)
             return;
 
         w->show();
-        if (isTreeModeEnabled() ? m_tabTree->hasFocus() : m_tabBar->hasFocus())
+        if (m_toolBarCurrent->hasFocus())
             w->setFocus();
 
-        if ( isTreeModeEnabled() )
-            m_tabTree->setCurrentTabIndex(tabIndex);
-        else
-            m_tabBar->setCurrentIndex(tabIndex);
-    } else if (w != NULL) {
+        m_tabs->setCurrentTab(tabIndex);
+    } else if (w != nullptr) {
         if (w->hasFocus())
-            isTreeModeEnabled() ? m_tabTree->setFocus() : m_tabBar->setFocus();
+            m_toolBarCurrent->setFocus();
         w->hide();
     }
 
@@ -308,23 +289,12 @@ void TabWidget::setCurrentIndex(int tabIndex)
 
 void TabWidget::nextTab()
 {
-    if ( isTreeModeEnabled() ) {
-        m_tabTree->nextTreeItem();
-    } else {
-        const int tab = (currentIndex() + 1) % count();
-        setCurrentIndex(tab);
-    }
+    m_tabs->nextTab();
 }
 
 void TabWidget::previousTab()
 {
-    if ( isTreeModeEnabled() ) {
-        m_tabTree->previousTreeItem();
-    } else {
-        const int size = count();
-        const int tab = (size + currentIndex() - 1) % size;
-        setCurrentIndex(tab);
-    }
+    m_tabs->previousTab();
 }
 
 void TabWidget::setTabBarHidden(bool hidden)
@@ -335,38 +305,25 @@ void TabWidget::setTabBarHidden(bool hidden)
 
 void TabWidget::setTreeModeEnabled(bool enabled)
 {
-    if (isTreeModeEnabled() == enabled) {
-        updateToolBar();
-        return;
-    }
-
     const QStringList tabs = this->tabs();
 
-    if (enabled) {
-        delete m_tabBar;
-        m_tabBar = NULL;
-
+    if (enabled)
         createTabTree();
-        for (int i = 0; i < tabs.size(); ++i) {
-            const QString &tabName = tabs[i];
-            m_tabTree->insertTab(tabName, i, i == 0);
-            m_tabTree->setTabItemCount(tabName, itemCountLabel(tabName));
-        }
-
-        m_tabTree->setCollapsedTabs(m_collapsedTabs);
-    } else {
-        m_collapsedTabs = m_tabTree->collapsedTabs();
-
-        delete m_tabTree;
-        m_tabTree = NULL;
-
+    else
         createTabBar();
-        for (int i = 0; i < tabs.size(); ++i) {
-            const QString &tabName = tabs[i];
-            m_tabBar->insertTab(i, tabName);
-            m_tabBar->setTabItemCount(tabName, itemCountLabel(tabName));
-        }
+
+    m_ignoreCurrentTabChanges = true;
+    for (int i = 0; i < tabs.size(); ++i) {
+        const QString &tabName = tabs[i];
+        m_tabs->insertTab(i, tabName);
+        m_tabs->setTabItemCount(tabName, itemCountLabel(tabName));
     }
+    m_tabs->setCurrentTab( currentIndex() );
+    m_ignoreCurrentTabChanges = false;
+
+    m_tabs->setCollapsedTabs(m_collapsedTabs);
+
+    updateToolBar();
 }
 
 void TabWidget::setTabItemCount(const QString &tabName, int itemCount)
@@ -387,20 +344,6 @@ bool TabWidget::eventFilter(QObject *, QEvent *event)
     return false;
 }
 
-void TabWidget::onTreeItemSelected(bool isGroup)
-{
-    QWidget *w = currentWidget();
-    if (w == NULL)
-        return;
-
-    if (isGroup) {
-        w->hide();
-    } else {
-        w->show();
-        w->setFocus();
-    }
-}
-
 void TabWidget::onTabMoved(int from, int to)
 {
     m_stackedWidget->insertWidget(to, m_stackedWidget->widget(from));
@@ -423,7 +366,7 @@ void TabWidget::onTabsMoved(const QString &oldPrefix, const QString &newPrefix,
         m_stackedWidget->removeWidget(w);
     }
 
-    foreach (int index, indexes) {
+    for (int index : indexes) {
         Q_ASSERT(index >= 0);
         Q_ASSERT(index < widgets.count());
         m_stackedWidget->insertWidget(-1, widgets[index]);
@@ -446,60 +389,72 @@ void TabWidget::onToolBarOrientationChanged(Qt::Orientation orientation)
     }
 }
 
+void TabWidget::onTreeItemClicked()
+{
+    auto w = currentWidget();
+    if (w)
+        w->setFocus(Qt::MouseFocusReason);
+}
+
 void TabWidget::createTabBar()
 {
-    m_tabBar = new TabBar(this);
-    m_toolBar->addWidget(m_tabBar);
+    auto tabBar = new TabBar(this);
 
-    m_tabBar->setObjectName("tab_bar");
+    tabBar->setObjectName("tab_bar");
 
-    m_tabBar->setExpanding(false);
-    m_tabBar->setMovable(true);
+    tabBar->setExpanding(false);
+    tabBar->setMovable(true);
 
-    connect( m_tabBar, SIGNAL(tabMenuRequested(QPoint, int)),
-             this, SIGNAL(tabMenuRequested(QPoint, int)) );
-    connect( m_tabBar, SIGNAL(tabRenamed(QString,int)),
+    connect( tabBar, SIGNAL(tabMenuRequested(QPoint,int)),
+             this, SIGNAL(tabMenuRequested(QPoint,int)) );
+    connect( tabBar, SIGNAL(tabRenamed(QString,int)),
              this, SIGNAL(tabRenamed(QString,int)) );
-    connect( m_tabBar, SIGNAL(tabCloseRequested(int)),
+    connect( tabBar, SIGNAL(tabCloseRequested(int)),
              this, SIGNAL(tabCloseRequested(int)) );
-    connect( m_tabBar, SIGNAL(dropItems(QString,QDropEvent*)),
-             this, SIGNAL(dropItems(QString,QDropEvent*)) );
-    connect( m_tabBar, SIGNAL(currentChanged(int)),
+    connect( tabBar, SIGNAL(dropItems(QString,const QMimeData*)),
+             this, SIGNAL(dropItems(QString,const QMimeData*)) );
+    connect( tabBar, SIGNAL(currentChanged(int)),
              this, SLOT(setCurrentIndex(int)) );
-    connect( m_tabBar, SIGNAL(tabMoved(int, int)),
-             this, SLOT(onTabMoved(int, int)) );
+    connect( tabBar, SIGNAL(tabMoved(int,int)),
+             this, SLOT(onTabMoved(int,int)) );
 
-    updateToolBar();
+    delete m_tabs;
+    m_tabs = tabBar;
+    m_tabBar = tabBar;
+
+    m_toolBarCurrent = m_toolBar;
+    m_toolBarCurrent->addWidget(tabBar);
 }
 
 void TabWidget::createTabTree()
 {
-    m_tabTree = new TabTree(this);
-    m_toolBarTree->addWidget(m_tabTree);
+    auto tabTree = new TabTree(this);
+    tabTree->setObjectName("tab_tree");
 
-    m_tabTree->setObjectName("tab_tree");
-
-    connect( m_tabTree, SIGNAL(tabMenuRequested(QPoint,QString)),
+    connect( tabTree, SIGNAL(tabMenuRequested(QPoint,QString)),
              this, SIGNAL(tabMenuRequested(QPoint,QString)) );
-    connect( m_tabTree, SIGNAL(tabsMoved(QString,QString,QList<int>)),
+    connect( tabTree, SIGNAL(tabsMoved(QString,QString,QList<int>)),
              this, SLOT(onTabsMoved(QString,QString,QList<int>)) );
-    connect( m_tabTree, SIGNAL(dropItems(QString,QDropEvent*)),
-             this, SIGNAL(dropItems(QString,QDropEvent*)) );
-    connect( m_tabTree, SIGNAL(currentTabChanged(int)),
+    connect( tabTree, SIGNAL(dropItems(QString,const QMimeData*)),
+             this, SIGNAL(dropItems(QString,const QMimeData*)) );
+    connect( tabTree, SIGNAL(currentTabChanged(int)),
              this, SLOT(setCurrentIndex(int)) );
+    connect( tabTree, SIGNAL(itemClicked(QTreeWidgetItem*,int)),
+             this, SLOT(onTreeItemClicked()) );
 
-    updateToolBar();
+    delete m_tabs;
+    m_tabs = tabTree;
+    m_tabBar = nullptr;
 
-    // Override left and right keys of tab tree.
-    addTabAction(m_tabTree, Qt::Key_Left, this, SLOT(previousTab()), Qt::WidgetShortcut);
-    addTabAction(m_tabTree, Qt::Key_Right, this, SLOT(nextTab()), Qt::WidgetShortcut);
+    m_toolBarCurrent = m_toolBarTree;
+    m_toolBarCurrent->addWidget(tabTree);
 }
 
 void TabWidget::updateToolBar()
 {
     bool forceHide = count() == 1;
-    m_toolBar->setVisible(!forceHide && !m_hideTabBar && !isTreeModeEnabled());
-    m_toolBarTree->setVisible(!forceHide && !m_hideTabBar && isTreeModeEnabled());
+    m_toolBar->setVisible(!forceHide && !m_hideTabBar && m_toolBarCurrent == m_toolBar);
+    m_toolBarTree->setVisible(!forceHide && !m_hideTabBar && m_toolBarCurrent == m_toolBarTree);
 
     if (m_tabBar) {
         QMainWindow *mainWindow = qobject_cast<QMainWindow*>(m_toolBar->window());
@@ -516,25 +471,13 @@ void TabWidget::updateToolBar()
         }
     }
 
-    updateSize();
+    m_tabs->adjustSize();
 }
 
 void TabWidget::updateTabItemCount(const QString &name)
 {
-    if ( isTreeModeEnabled() )
-        m_tabTree->setTabItemCount(name, itemCountLabel(name));
-    else
-        m_tabBar->setTabItemCount(name, itemCountLabel(name));
-
-    updateSize();
-}
-
-void TabWidget::updateSize()
-{
-    if ( isTreeModeEnabled() )
-        m_tabTree->setFixedWidth(m_tabTree->minimumWidth());
-    else
-        m_tabBar->adjustSize();
+    m_tabs->setTabItemCount(name, itemCountLabel(name));
+    m_tabs->adjustSize();
 }
 
 QString TabWidget::itemCountLabel(const QString &name)
