@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -24,6 +24,9 @@
 
 #include <QStringList>
 
+#include <algorithm>
+#include <functional>
+
 namespace {
 
 QList<QPersistentModelIndex> validIndeces(const QModelIndexList &indexList)
@@ -31,7 +34,7 @@ QList<QPersistentModelIndex> validIndeces(const QModelIndexList &indexList)
     QList<QPersistentModelIndex> list;
     list.reserve(indexList.size());
 
-    foreach (const QModelIndex &index, indexList) {
+    for (const auto &index : indexList) {
         if ( index.isValid() )
             list.append(index);
     }
@@ -43,7 +46,7 @@ int topMostRow(const QList<QPersistentModelIndex> &indexList)
 {
     int row = indexList.value(0).row();
 
-    foreach (const QPersistentModelIndex &index, indexList)
+    for (const auto &index : indexList)
         row = qMin(row, index.row());
 
     return row;
@@ -51,12 +54,22 @@ int topMostRow(const QList<QPersistentModelIndex> &indexList)
 
 } // namespace
 
+void ClipboardItemList::move(int from, int count, int to)
+{
+    if (to < from) {
+        std::swap(from, to);
+        to += count;
+        count = to - from - count;
+    }
+
+    const auto start1 = std::begin(m_items) + from;
+    const auto start2 = start1 + count;
+    const auto end2 = std::begin(m_items) + to;
+    std::rotate(start1, start2, end2);
+}
+
 ClipboardModel::ClipboardModel(QObject *parent)
     : QAbstractListModel(parent)
-    , m_max(100)
-    , m_clipboardList(m_max)
-    , m_disabled(false)
-    , m_tabName()
 {
 }
 
@@ -128,8 +141,29 @@ void ClipboardModel::insertItem(const QVariantMap &data, int row)
     endInsertRows();
 }
 
+void ClipboardModel::insertItems(const QList<QVariantMap> &dataList, int row)
+{
+    if ( dataList.isEmpty() )
+        return;
+
+    int targetRow = row;
+    m_clipboardList.reserve( m_clipboardList.size() + dataList.size() );
+
+    beginInsertRows(QModelIndex(), row, row + dataList.size() - 1);
+
+    for ( auto it = std::begin(dataList); it != std::end(dataList); ++it ) {
+        m_clipboardList.insert(targetRow, ClipboardItem(*it));
+        ++targetRow;
+    }
+
+    endInsertRows();
+}
+
 bool ClipboardModel::insertRows(int position, int rows, const QModelIndex&)
 {
+    if ( rows <= 0 || position < 0 )
+        return false;
+
     beginInsertRows(QModelIndex(), position, position + rows - 1);
 
     for (int row = 0; row < rows; ++row)
@@ -142,7 +176,7 @@ bool ClipboardModel::insertRows(int position, int rows, const QModelIndex&)
 
 bool ClipboardModel::removeRows(int position, int rows, const QModelIndex&)
 {
-    if ( position < 0 || position >= rowCount() )
+    if ( rows <= 0 || position < 0 || position + rows > rowCount() )
         return false;
 
     int last = qMin( position + rows, rowCount() ) - 1;
@@ -156,127 +190,38 @@ bool ClipboardModel::removeRows(int position, int rows, const QModelIndex&)
     return true;
 }
 
-int ClipboardModel::getRowNumber(int row, bool cycle) const
+bool ClipboardModel::moveRows(
+        const QModelIndex &sourceParent, int sourceRow, int rows,
+        const QModelIndex &destinationParent, int destinationRow)
 {
-    int n = rowCount();
-    if (n == 0)
-        return -1;
+    if (sourceParent.isValid() || destinationParent.isValid())
+        return false;
 
-    if (row >= n)
-        return cycle ? 0 : n - 1;
-
-    if (row < 0)
-        return cycle ? n - 1 : 0;
-
-    return row;
-}
-
-void ClipboardModel::unloadItems()
-{
-    emit unloaded();
-    removeRows(0, rowCount());
-}
-
-void ClipboardModel::setMaxItems(int max)
-{
-    m_max = qMax(0, max);
-
-    if ( m_max < m_clipboardList.size() ) {
-        beginRemoveRows(QModelIndex(), m_max + 1, m_clipboardList.size() - 1);
-        m_clipboardList.resize(m_max);
-        endRemoveRows();
-    } else {
-        m_clipboardList.reserve(m_max);
+    if (sourceRow < 0 || destinationRow < 0 || destinationRow > rowCount()
+            || rows <= 0 || sourceRow + rows > rowCount())
+    {
+        return false;
     }
-}
 
-void ClipboardModel::setTabName(const QString &tabName)
-{
-    Q_ASSERT( !tabName.isEmpty() );
-
-    if (m_tabName == tabName)
-        return;
-
-    m_tabName = tabName;
-    emit tabNameChanged(m_tabName);
-}
-
-bool ClipboardModel::move(int pos, int newpos)
-{
-    int from = getRowNumber(pos,true);
-    int to   = getRowNumber(newpos,true);
-
-    if( from == -1 || to == -1 )
+    const int last = sourceRow + rows - 1;
+    if (sourceRow <= destinationRow && destinationRow <= last + 1)
         return false;
 
-    const int sourceRow = from;
-    const int targetRow = to;
-    if (from < to)
-        ++to;
-
-    if ( !beginMoveRows(QModelIndex(), from, from, QModelIndex(), to) )
-        return false;
-
-    m_clipboardList.move(sourceRow, targetRow);
-
+    beginMoveRows(sourceParent, sourceRow, last, destinationParent, destinationRow);
+    m_clipboardList.move(sourceRow, rows, destinationRow);
     endMoveRows();
 
     return true;
 }
 
-bool ClipboardModel::moveItemsWithKeyboard(QModelIndexList indexList, int key, int count) {
-    int from, to;
-    bool res = false;
-
-    QList<int> list;
-    for ( int i = 0; i < indexList.length(); ++i )
-        list.append( indexList.at(i).row() );
-
-    if ( key == Qt::Key_Down || key == Qt::Key_End )
-        qSort( list.begin(), list.end(), qGreater<int>() );
-    else
-        qSort( list.begin(), list.end(), qLess<int>() );
-
-    for ( int i = 0, d = 0; i<list.length(); ++i ) {
-        from = list.at(i) + d;
-
-        switch (key) {
-        case Qt::Key_Down:
-            to = from + count;
-            break;
-        case Qt::Key_Up:
-            to = from - count;
-            break;
-        case Qt::Key_End:
-            to = rowCount()-i-1;
-            break;
-        default:
-            to = 0+i;
-            break;
-        }
-
-        if ( to < 0 )
-            --d;
-        else if (to >= rowCount() )
-            ++d;
-
-        if ( !move(from, to) )
-            return false;
-        if (!res)
-            res = to==0 || from==0 || to == rowCount();
-    }
-
-    return res;
-}
-
 void ClipboardModel::sortItems(const QModelIndexList &indexList, CompareItems *compare)
 {
     QList<QPersistentModelIndex> list = validIndeces(indexList);
-    qSort( list.begin(), list.end(), compare );
+    std::sort( list.begin(), list.end(), compare );
 
     int targetRow = topMostRow(list);
 
-    foreach (const QPersistentModelIndex &ind, list) {
+    for (const auto &ind : list) {
         if (ind.isValid()) {
             const int sourceRow = ind.row();
 
@@ -296,10 +241,10 @@ void ClipboardModel::sortItems(const QModelIndexList &indexList, CompareItems *c
     }
 }
 
-int ClipboardModel::findItem(uint item_hash) const
+int ClipboardModel::findItem(uint itemHash) const
 {
     for (int i = 0; i < m_clipboardList.size(); ++i) {
-        if ( m_clipboardList[i].dataHash() == item_hash )
+        if ( m_clipboardList[i].dataHash() == itemHash )
             return i;
     }
 
