@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -23,19 +23,27 @@
 #include "tests/testinterface.h"
 
 #include <QRegExp>
-#include <QSharedPointer>
 #include <QStringList>
 #include <QtPlugin>
 #include <QVariantMap>
 
+#include <memory>
+
 class QAbstractItemModel;
 class QTextEdit;
-class QFile;
+class QIODevice;
 class QFont;
 class QModelIndex;
 class QPalette;
 class QRegExp;
 class QWidget;
+struct Command;
+
+class ItemLoaderInterface;
+using ItemLoaderPtr = std::shared_ptr<ItemLoaderInterface>;
+
+class ItemSaverInterface;
+using ItemSaverPtr = std::shared_ptr<ItemSaverInterface>;
 
 #define COPYQ_PLUGIN_ITEM_LOADER_ID "org.CopyQ.ItemPlugin.ItemLoader/1.0"
 
@@ -58,7 +66,7 @@ class ItemWidget
 public:
     explicit ItemWidget(QWidget *widget);
 
-    virtual ~ItemWidget() {}
+    virtual ~ItemWidget() = default;
 
     /**
      * Set search and selections highlight color and font.
@@ -100,25 +108,35 @@ public:
     /**
      * Create external editor for @a index.
      *
-     * Default implementation returns NULL.
+     * Default implementation returns nullptr.
      *
      * @param index  index for which the editor is opened
      *
      * @return Editor object -- see documentation for public signals and slots of ItemEditor class --
-     *         NULL so default text editor is opened.
+     *         nullptr so default text editor is opened.
      */
     virtual QObject *createExternalEditor(const QModelIndex &index, QWidget *parent) const;
 
     /**
      * Size of widget needs to be updated (because maximum size chaged).
      */
-    virtual void updateSize(const QSize &maximumSize);
+    virtual void updateSize(QSize maximumSize, int idealWidth);
 
     /**
      * Called if widget is set or unset as current.
      */
     virtual void setCurrent(bool current);
 
+    /**
+     * Mark item as tagged/untagged.
+     *
+     * Used to hide unimportant data when notes or tags are present.
+     */
+    virtual void setTagged(bool) {}
+
+    ItemWidget(const ItemWidget &) = delete;
+    ItemWidget &operator=(const ItemWidget &) = delete;
+
 protected:
     /**
      * Highlight matching text with given font and color.
@@ -145,12 +163,117 @@ private:
     QWidget *m_widget;
 };
 
+/**
+ * Adss functions and variables accessible from scripts/commands.
+ *
+ * New functions and variables will be available in scripts under `plugins.<PLUGIN_ID>`.
+ *
+ * Public slots will be available as functions.
+ *
+ * Public QObject properties will be available as variables.
+ *
+ * Important: The slots and properties are used in different than main thread.
+ */
+class ItemScriptable : public QObject
+{
+    Q_OBJECT
+public:
+    explicit ItemScriptable(QObject *parent) : QObject(parent) {}
+
+    /** Return scriptable object (nullptr before start() is called). */
+    QObject *scriptable() const { return m_scriptable; }
+
+    /** Set scriptable (used internally before scripts starts). */
+    void setScriptable(QObject *scriptable) { m_scriptable = scriptable; }
+
+    /** Called before script starts (and scriptable is valid). */
+    virtual void start() {}
+
+protected:
+    /**
+     * Call scriptable function by name with given @a arguments.
+     *
+     * Shouldn't be called before start().
+     */
+    QVariant call(const QString &method, const QVariantList &arguments = QVariantList());
+
+    /**
+     * Evaluate script.
+     *
+     * Shouldn't be called before start().
+     */
+    QVariant eval(const QString &script);
+
+    /**
+     * Return arguments passed to slots from script.
+     *
+     * Shouldn't be called before start().
+     */
+    QVariantList currentArguments();
+
+private:
+    QObject *m_scriptable = nullptr;
+};
+
+/**
+ * Handles saving and modifying item models.
+ */
+class ItemSaverInterface
+{
+public:
+    ItemSaverInterface() = default;
+
+    virtual ~ItemSaverInterface() = default;
+
+    /**
+     * Save items.
+     * @return true only if items were saved
+     */
+    virtual bool saveItems(const QString &tabName, const QAbstractItemModel &model, QIODevice *file);
+
+    /**
+     * Called before items are deleted by user.
+     * @return true if items can be removed, false to cancel the removal
+     */
+    virtual bool canRemoveItems(const QList<QModelIndex> &indexList, QString *error);
+
+    /**
+     * Called before items are moved out of list (i.e. deleted) by user.
+     * @return true if items can be moved, false to cancel the removal
+     */
+    virtual bool canMoveItems(const QList<QModelIndex> &);
+
+    /**
+     * Called when items are being deleted by user.
+     */
+    virtual void itemsRemovedByUser(const QList<QModelIndex> &indexList);
+
+    /**
+     * Return copy of items data.
+     */
+    virtual QVariantMap copyItem(const QAbstractItemModel &model, const QVariantMap &itemData);
+
+    ItemSaverInterface(const ItemSaverInterface &) = delete;
+    ItemSaverInterface &operator=(const ItemSaverInterface &) = delete;
+};
+
+/**
+ * Main class for plugins (needs to be implemented).
+ *
+ * Identifies the plugin (id, name, author, description) and optionally:
+ * - loads items from file (creates ItemSaverInterface instance),
+ * - creates item widgets (creates ItemWidget instance),
+ * - adds scripting capabilities (creates ItemScriptable instance),
+ * - matches/filters items,
+ * - provides commands for Command dialog,
+ * - provides settings widget for Configuration dialog.
+ */
 class ItemLoaderInterface
 {
 public:
-    ItemLoaderInterface() {}
+    ItemLoaderInterface() = default;
 
-    virtual ~ItemLoaderInterface() {}
+    virtual ~ItemLoaderInterface() = default;
 
     /**
      * Return priority.
@@ -162,17 +285,21 @@ public:
     /**
      * Create ItemWidget instance from index data.
      *
-     * @return NULL if index hasn't appropriate data
+     * @return nullptr if index hasn't appropriate data
      */
-    virtual ItemWidget *create(const QModelIndex &index, QWidget *parent) const;
+    virtual ItemWidget *create(const QModelIndex &index, QWidget *parent, bool preview) const;
 
     /**
-     * Simple ID of plugin (e.g. part of plugin file name).
+     * Simple ID of plugin.
+     *
+     * Name configuration sub-section and plugin object in scripts (`plugins.<PLUGIN_ID>`).
+     *
+     * Should be short and contain only lower case letters.
      */
     virtual QString id() const = 0;
 
     /**
-     * Return descriptive name.
+     * Return descriptive name (translatable).
      */
     virtual QString name() const = 0;
 
@@ -182,14 +309,16 @@ public:
     virtual QString author() const = 0;
 
     /**
-     * Return plugin description.
+     * Return plugin description (translatable).
      */
     virtual QString description() const = 0;
 
     /**
      * Return icon representing the plugin.
      *
-     * Return value can be QIcon or UInt (to render a character from icon font). Default is no icon.
+     * Return value can be QIcon or UInt (to render a character from icon font).
+     *
+     * Default is no icon.
      */
     virtual QVariant icon() const { return QVariant(); }
 
@@ -213,69 +342,42 @@ public:
     /**
      * Create settings widget.
      */
-    virtual QWidget *createSettingsWidget(QWidget *) { return NULL; }
+    virtual QWidget *createSettingsWidget(QWidget *) { return nullptr; }
 
     /**
      * @return true only if items can be loaded
      */
-    virtual bool canLoadItems(QFile *file);
+    virtual bool canLoadItems(QIODevice *file) const;
 
     /**
      * @return true only if items can be saved
      */
-    virtual bool canSaveItems(const QAbstractItemModel &model);
+    virtual bool canSaveItems(const QString &tabName) const;
 
     /**
      * Load items.
      * @return true only if items were saved by this plugin (or just not to load them any further)
      */
-    virtual bool loadItems(QAbstractItemModel *model, QFile *file);
-
-    /**
-     * Save items.
-     * @return true only if items were saved
-     */
-    virtual bool saveItems(const QAbstractItemModel &model, QFile *file);
+    virtual ItemSaverPtr loadItems(
+            const QString &tabName, QAbstractItemModel *model, QIODevice *file, int maxItems);
 
     /**
      * Initialize tab (tab was not yet saved or loaded).
      * @return true only if successful
      */
-    virtual bool initializeTab(QAbstractItemModel *model);
-
-    /**
-     * Uninitialize tab (tab will be handled by different plugin).
-     */
-    virtual void uninitializeTab(QAbstractItemModel *model);
+    virtual ItemSaverPtr initializeTab(const QString &tabName, QAbstractItemModel *model, int maxItems);
 
     /**
      * Allow to transform item widget (wrap around a new widget).
-     * By default returns NULL not to wrap the widget.
+     * By default returns nullptr not to wrap the widget.
      * New ItemWidget must take care of deleting the old one!
      */
     virtual ItemWidget *transform(ItemWidget *itemWidget, const QModelIndex &index);
 
     /**
-     * Called before items are deleted by user.
-     * @return true if items can be removed, false to cancel the removal
+     * Transform loader.
      */
-    virtual bool canRemoveItems(const QList<QModelIndex> &indexList);
-
-    /**
-     * Called before items are moved out of list (i.e. deleted) by user.
-     * @return true if items can be moved, false to cancel the removal
-     */
-    virtual bool canMoveItems(const QList<QModelIndex> &);
-
-    /**
-     * Called when items are being deleted by user.
-     */
-    virtual void itemsRemovedByUser(const QList<QModelIndex> &indexList);
-
-    /**
-     * Return copy of items data.
-     */
-    virtual QVariantMap copyItem(const QAbstractItemModel &model, const QVariantMap &itemData);
+    virtual ItemSaverPtr transformSaver(const ItemSaverPtr &saver, QAbstractItemModel *model);
 
     /**
      * Return true if regular expression matches items content.
@@ -295,12 +397,27 @@ public:
 
     /**
      * Return QObject instance with signals (by default null pointer).
-     * Returned QObject must have signal error(QString) for signaling errors.
+     *
+     * Returned QObject can have signal error(QString) for signaling errors
+     * and addCommands(QList<Command>) to open and add commands to Command dialog.
      */
     virtual const QObject *signaler() const;
-};
 
-typedef QSharedPointer<ItemLoaderInterface> ItemLoaderInterfacePtr;
+    /**
+     * Return scriptable object that can be used in scripts.
+     *
+     * Object will be available as `plugins.<PLUGIN_ID>`.
+     */
+    virtual ItemScriptable *scriptableObject(QObject *parent);
+
+    /**
+     * Adds commands from scripts for command dialog.
+     */
+    virtual QList<Command> commands() const;
+
+    ItemLoaderInterface(const ItemLoaderInterface &) = delete;
+    ItemLoaderInterface &operator=(const ItemLoaderInterface &) = delete;
+};
 
 Q_DECLARE_INTERFACE(ItemLoaderInterface, COPYQ_PLUGIN_ITEM_LOADER_ID)
 
