@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -18,57 +18,118 @@
 */
 
 #include "app/app.h"
+#include "app/applicationexceptionhandler.h"
 #include "app/clipboardclient.h"
 #include "app/clipboardmonitor.h"
 #include "app/clipboardserver.h"
+#include "common/commandstatus.h"
 #include "common/log.h"
+#include "common/messagehandlerforqt.h"
+#include "common/textdata.h"
 #include "platform/platformnativeinterface.h"
 #include "scriptable/scriptable.h"
 
-#include <QCoreApplication>
+#include <QApplication>
 #include <QFile>
 #include <QScriptEngine>
+#include <QSettings>
 
 #ifdef HAS_TESTS
 #  include "tests/tests.h"
 #endif // HAS_TESTS
 
+#include <exception>
+
 Q_DECLARE_METATYPE(QByteArray*)
 
 namespace {
 
-int evaluate(const QString &functionName, const QStringList &arguments, int argc, char **argv)
+int evaluate(
+        const QString &functionName,
+        const QStringList &arguments, int argc, char **argv,
+        const QString &sessionName)
 {
-    App app( new QCoreApplication(argc, argv) );
+    App app( "Prompt", createPlatformNativeInterface()->createConsoleApplication(argc, argv), sessionName );
 
-    Scriptable scriptable(NULL, NULL);
     QScriptEngine engine;
-    engine.setGlobalObject( engine.newQObject(&scriptable) );
+    Scriptable scriptable(&engine, nullptr);
 
     QScriptValue function = engine.globalObject().property(functionName);
     QScriptValueList functionArguments;
 
-    foreach (const QString &argument, arguments)
+    functionArguments.reserve( arguments.size() );
+    for (const auto &argument : arguments)
         functionArguments.append(argument);
 
     const QScriptValue result = function.call( QScriptValue(), functionArguments );
 
-    QFile f;
-    f.open(stdout, QIODevice::WriteOnly);
-    f.write( scriptable.fromString(result.toString()) );
+    const auto output = scriptable.fromString(result.toString());
+    if ( !output.isEmpty() ) {
+        QFile f;
+        if ( engine.hasUncaughtException() )
+            f.open(stderr, QIODevice::WriteOnly);
+        else
+            f.open(stdout, QIODevice::WriteOnly);
 
-    return engine.hasUncaughtException() ? 1 : 0;
+        f.write(output);
+        if ( !output.endsWith("\n") )
+            f.write("\n");
+        f.close();
+    }
+
+    const int exitCode = engine.hasUncaughtException() ? CommandException : 0;
+    app.exit(exitCode);
+    return exitCode;
 }
 
-int startServer(int argc, char *argv[], const QString &sessionName)
+bool containsOnlyValidCharacters(const QString &sessionName)
 {
-    ClipboardServer app(argc, argv, sessionName);
+    for (const auto &c : sessionName) {
+        if ( !c.isLetterOrNumber() && c != '-' && c != '_' )
+            return false;
+    }
+
+    return true;
+}
+
+bool isValidSessionName(const QString &sessionName)
+{
+    return !sessionName.isNull() &&
+           sessionName.length() < 16 &&
+           containsOnlyValidCharacters(sessionName);
+}
+
+QString restoreSessionName(const QString &sessionId)
+{
+    QSettings settings(QSettings::IniFormat, QSettings::UserScope, "copyq", "copyq_no_session");
+    const auto sessionNameKey = "session_" + sessionId;
+    const auto sessionName = settings.value(sessionNameKey).toString();
+    settings.remove(sessionNameKey);
+    return sessionName;
+}
+
+int startServer(int argc, char *argv[], QString sessionName)
+{
+    auto qapp = createPlatformNativeInterface()->createServerApplication(argc, argv);
+    if ( qapp->isSessionRestored() ) {
+        const auto sessionId = qapp->sessionId();
+        sessionName = restoreSessionName(sessionId);
+        COPYQ_LOG( QString("Restoring session ID \"%1\", session name \"%2\"")
+                   .arg(sessionId)
+                   .arg(sessionName) );
+        if ( !sessionName.isEmpty() && !isValidSessionName(sessionName) ) {
+            log("Failed to restore session name", LogError);
+            return 2;
+        }
+    }
+
+    ClipboardServer app(qapp, sessionName);
     return app.exec();
 }
 
-int startMonitor(int argc, char *argv[])
+int startMonitor(int argc, char *argv[], const QString &serverName, const QString &sessionName)
 {
-    ClipboardMonitor app(argc, argv);
+    ClipboardMonitor app(argc, argv, serverName, sessionName);
     return app.exec();
 }
 
@@ -92,6 +153,12 @@ bool needsVersion(const QString &arg)
            arg == "version";
 }
 
+bool needsInfo(const QString &arg)
+{
+    return arg == "--info" ||
+           arg == "info";
+}
+
 #ifdef HAS_TESTS
 bool needsTests(const QString &arg)
 {
@@ -100,23 +167,6 @@ bool needsTests(const QString &arg)
 }
 #endif
 
-bool containsOnlyValidCharacters(const QString &sessionName)
-{
-    foreach (const QChar &c, sessionName) {
-        if ( !c.isLetterOrNumber() && c != '-' && c != '_' )
-            return false;
-    }
-
-    return true;
-}
-
-bool isValidSessionName(const QString &sessionName)
-{
-    return !sessionName.isNull() &&
-           sessionName.length() < 16 &&
-           containsOnlyValidCharacters(sessionName);
-}
-
 QString getSessionName(const QStringList &arguments, int *skipArguments)
 {
     const QString firstArgument = arguments.value(0);
@@ -132,32 +182,20 @@ QString getSessionName(const QStringList &arguments, int *skipArguments)
         return firstArgument.mid( firstArgument.indexOf('=') + 1 );
     }
 
-    return QString::fromUtf8( qgetenv("COPYQ_SESSION_NAME") );
+    // Skip session arguments passed from session manager.
+    if (arguments.size() == 2 && firstArgument == "-session")
+        *skipArguments = 2;
+
+    return getTextData( qgetenv("COPYQ_SESSION_NAME") );
 }
 
-} // namespace
-
-int main(int argc, char **argv)
+int startApplication(int argc, char **argv)
 {
+    installMessageHandlerForQt();
+
     const QStringList arguments =
             createPlatformNativeInterface()->getCommandLineArguments(argc, argv);
 
-    // Print version, help or run tests.
-    if ( !arguments.isEmpty() ) {
-        if ( needsVersion(arguments.first()) )
-            return evaluate( "version", QStringList(), argc, argv );
-
-        if ( needsHelp(arguments.first()) )
-            return evaluate( "help", arguments.mid(1), argc, argv );
-
-#ifdef HAS_TESTS
-        if ( needsTests(arguments.first()) ) {
-            // Skip the "tests" argument and pass the rest to tests.
-            return runTests(argc - 1, argv + 1);
-        }
-#endif
-    }
-
     // Get session name (default is empty).
     int skipArguments;
     const QString sessionName = getSessionName(arguments, &skipArguments);
@@ -168,6 +206,27 @@ int main(int argc, char **argv)
         return 2;
     }
 
+    // Print version, help or run tests.
+    if ( arguments.size() > skipArguments ) {
+        const auto arg = arguments[skipArguments];
+
+        if ( needsVersion(arg) )
+            return evaluate( "version", QStringList(), argc, argv, sessionName );
+
+        if ( needsHelp(arg) )
+            return evaluate( "help", arguments.mid(skipArguments + 1), argc, argv, sessionName );
+
+        if ( needsInfo(arg) )
+            return evaluate( "info", arguments.mid(skipArguments + 1), argc, argv, sessionName );
+
+#ifdef HAS_TESTS
+        if ( needsTests(arg) ) {
+            // Skip the "tests" argument and pass the rest to tests.
+            return runTests(argc - skipArguments - 1, argv + skipArguments + 1);
+        }
+#endif
+    }
+
     // If server hasn't been run yet and no argument were specified
     // then run this process as server.
     if ( arguments.size() - skipArguments == 0 )
@@ -175,10 +234,25 @@ int main(int argc, char **argv)
 
     // If first argument is "monitor" (second is monitor server name/ID)
     // then run this process as clipboard monitor.
-    if ( arguments.size() == 2 && arguments[0] == "monitor" )
-        return startMonitor(argc, argv);
+    if ( arguments.size() == 4 && arguments[2] == "monitor" )
+        return startMonitor(argc, argv, arguments[3], sessionName);
 
     // If argument was specified and server is running
     // then run this process as client.
     return startClient(argc, argv, skipArguments, sessionName);
 }
+
+} // namespace
+
+int main(int argc, char **argv)
+{
+    try {
+        return startApplication(argc, argv);
+    } catch (const std::exception &e) {
+        logException(e.what());
+        throw;
+    } catch (...) {
+        logException();
+        throw;
+    }
+}
