@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -21,30 +21,65 @@
 
 #include "dummyclipboard.h"
 
+#include "app/applicationexceptionhandler.h"
+
 #include <QApplication>
 #include <QCoreApplication>
+#include <QDir>
+#include <QStringList>
 
 PlatformPtr createPlatformNativeInterface()
 {
     return PlatformPtr(new DummyPlatform);
 }
 
+QCoreApplication *DummyPlatform::createConsoleApplication(int &argc, char **argv)
+{
+    return new ApplicationExceptionHandler<QCoreApplication>(argc, argv);
+}
+
 QApplication *DummyPlatform::createServerApplication(int &argc, char **argv)
 {
-    return new QApplication(argc, argv);
+    return new ApplicationExceptionHandler<QApplication>(argc, argv);
 }
 
 QApplication *DummyPlatform::createMonitorApplication(int &argc, char **argv)
 {
-    return new QApplication(argc, argv);
+    return new ApplicationExceptionHandler<QApplication>(argc, argv);
 }
 
 QCoreApplication *DummyPlatform::createClientApplication(int &argc, char **argv)
 {
-    return new QCoreApplication(argc, argv);
+    return new ApplicationExceptionHandler<QCoreApplication>(argc, argv);
 }
 
 PlatformClipboardPtr DummyPlatform::clipboard()
 {
     return PlatformClipboardPtr(new DummyClipboard());
 }
+
+bool DummyPlatform::findPluginDir(QDir *pluginsDir)
+{
+    pluginsDir->setPath( qApp->applicationDirPath() );
+    return pluginsDir->cd("plugins");
+}
+
+QString DummyPlatform::defaultEditorCommand()
+{
+    return "gedit %1";
+}
+
+QString DummyPlatform::translationPrefix()
+{
+    return QString();
+}
+
+QStringList DummyPlatform::getCommandLineArguments(int argc, char **argv)
+{
+    QStringList arguments;
+
+    for (int i = 1; i < argc; ++i)
+        arguments.append( QString::fromUtf8(argv[i]) );
+
+    return arguments;
+}
