@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -42,8 +42,6 @@ public:
      * Load settings from GUI.
      *
      * Settings keys are:
-     *   - "copy_clipboard" (bool)  - copy new clipboard data to X11 selection
-     *   - "copy_selection" (bool)  - copy new X11 selection data to clipboard
      *   - "check_selection" (bool) - emit changed() when X11 selection changes
      *   - "formats" (QStringList)  - string list with MIME formats to store
      *     (formats are also passed to data() method call from ClipboardMonitor)
@@ -59,17 +57,6 @@ public:
      * Set data to clipboard.
      */
     virtual void setData(Mode mode, const QVariantMap &dataMap) = 0;
-
-    /**
-     * Triggered by "ignore()" script command.
-     *
-     * Some applications reset clipboard so that passwords and sensitive data are kept
-     * in clipboard only while really needed.
-     *
-     * On X11 clipboard/selection synchronization should be canceled and previously
-     * synchronized content should be cleared.
-     */
-    virtual void ignoreCurrentData() = 0;
 };
 
 #endif // PLATFORMCLIPBOARD_H
