@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -30,7 +30,9 @@ class QString;
 class PlatformWindow
 {
 public:
-    virtual ~PlatformWindow() {}
+    PlatformWindow() = default;
+
+    virtual ~PlatformWindow() = default;
 
     /**
      * Return window title text.
@@ -51,6 +53,9 @@ public:
      * Copy to clipboard from window.
      */
     virtual void copy() = 0;
+
+    PlatformWindow(const PlatformWindow &) = delete;
+    PlatformWindow &operator=(const PlatformWindow &) = delete;
 };
 
 #endif // PLATFORMWINDOW_H
