@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -20,6 +20,8 @@
 #include "platform/platformcommon.h"
 #include "winplatformwindow.h"
 
+#include "common/log.h"
+
 #include <QApplication>
 #include <QElapsedTimer>
 #include <QString>
@@ -27,6 +29,13 @@
 
 namespace {
 
+QString windowTitle(HWND window)
+{
+    WCHAR buf[1024];
+    GetWindowTextW(window, buf, 1024);
+    return QString::fromUtf16(reinterpret_cast<ushort *>(buf));
+}
+
 INPUT createInput(WORD key, DWORD flags = 0)
 {
     INPUT input;
@@ -41,20 +50,65 @@ INPUT createInput(WORD key, DWORD flags = 0)
     return input;
 }
 
-bool raiseWindow(HWND window)
+QString windowLogText(QString text, HWND window)
 {
-    if (!IsWindowVisible(window))
-        return false;
+    const QString windowInfo =
+            QString("%1").arg(reinterpret_cast<quintptr>(window))
+            + " \"" + windowTitle(window) + "\"";
 
-    if (!SetForegroundWindow(window))
+    text.prepend("Window " + windowInfo + ": ");
+
+    const DWORD lastError = GetLastError();
+    if (lastError != 0)
+        text.append( QString(" (last error is %1)").arg(GetLastError()) );
+
+    return text;
+}
+
+void logWindow(const char *text, HWND window)
+{
+    COPYQ_LOG( windowLogText(text, window) );
+}
+
+bool raiseWindowHelper(HWND window)
+{
+    if (!SetForegroundWindow(window)) {
+        logWindow("Failed to raise:  SetForegroundWindow() == false", window);
         return false;
+    }
 
     SetWindowPos(window, HWND_TOP, 0, 0, 0, 0,
                  SWP_DRAWFRAME | SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW);
 
+    logWindow("Raised", window);
+
     return true;
 }
 
+bool raiseWindow(HWND window)
+{
+    if (!IsWindowVisible(window)) {
+        logWindow("Failed to raise: IsWindowVisible() == false", window);
+        return false;
+    }
+
+    // WORKAROUND: Set foreground window if even this process is not in foreground.
+    const auto thisThreadId = GetCurrentThreadId();
+    const auto foregroundThreadId = GetWindowThreadProcessId(GetForegroundWindow(), NULL);
+    if (thisThreadId != foregroundThreadId) {
+        if ( AttachThreadInput(thisThreadId, foregroundThreadId, true) ) {
+            logWindow("Attached foreground thread", window);
+            const bool result = raiseWindowHelper(window);
+            AttachThreadInput(thisThreadId, foregroundThreadId, false);
+            return result;
+        }
+
+        logWindow("Failed to attach foreground thread", window);
+    }
+
+    return raiseWindowHelper(window);
+}
+
 bool isKeyPressed(int key)
 {
     return GetKeyState(key) & 0x8000;
@@ -69,9 +123,7 @@ WinPlatformWindow::WinPlatformWindow(HWND window)
 
 QString WinPlatformWindow::getTitle()
 {
-    WCHAR buf[1024];
-    GetWindowTextW(m_window, buf, 1024);
-    return QString::fromUtf16(reinterpret_cast<ushort *>(buf));
+    return windowTitle(m_window);
 }
 
 void WinPlatformWindow::raise()
@@ -119,7 +171,7 @@ void WinPlatformWindow::sendKeyPress(WORD modifier, WORD key)
             << VK_MENU;
 
     // Release all modifiers first to send just Shift+Insert.
-    foreach (int mod, mods) {
+    for (int mod : mods) {
         if ( isKeyPressed(mod) ) {
             input1 << createInput(mod, KEYEVENTF_KEYUP);
             input2 << createInput(mod); // Press again at the end.
@@ -132,5 +184,10 @@ void WinPlatformWindow::sendKeyPress(WORD modifier, WORD key)
            << createInput(modifier, KEYEVENTF_KEYUP);
 
     QVector<INPUT> input = input1 + input2;
-    SendInput( input.size(), input.data(), sizeof(INPUT) );
+    const UINT numberOfAddedEvents = SendInput( input.size(), input.data(), sizeof(INPUT) );
+
+    if (numberOfAddedEvents == 0u)
+        logWindow("Failed to paste", m_window);
+    else
+        logWindow("Paste successful", m_window);
 }
