@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -23,31 +23,33 @@
 #include "platform/dummy/dummyclipboard.h"
 
 #include <QClipboard>
-#include <QSharedPointer>
 #include <QStringList>
 #include <QTimer>
 
+#include <memory>
+
 class X11DisplayGuard;
 
 class X11PlatformClipboard : public DummyClipboard
 {
     Q_OBJECT
 public:
-    X11PlatformClipboard(const QSharedPointer<X11DisplayGuard> &d);
+    explicit X11PlatformClipboard(const std::shared_ptr<X11DisplayGuard> &d);
 
-    void loadSettings(const QVariantMap &settings);
+    void loadSettings(const QVariantMap &settings) override;
 
-    QVariantMap data(Mode mode, const QStringList &formats) const;
+    QVariantMap data(Mode mode, const QStringList &formats) const override;
 
-    void setData(Mode mode, const QVariantMap &dataMap);
-
-    void ignoreCurrentData();
+    void setData(Mode mode, const QVariantMap &dataMap) override;
 
 private slots:
-    void onChanged(QClipboard::Mode mode);
-    void checkSelectionComplete();
+    void onChanged(QClipboard::Mode mode) override;
+
+    void onClipboardChanged();
+    void onSelectionChanged();
+
     void resetClipboard();
-    void synchronize();
+    void resetSelection();
 
 private:
     bool waitIfSelectionIncomplete();
@@ -57,24 +59,17 @@ private:
      *
      * @return return true if clipboard/selection has no owner and will be reset
      */
-    bool maybeResetClipboard(QClipboard::Mode mode);
+    bool maybeResetClipboard();
+    bool maybeResetSelection();
 
-    void syncFrom(QClipboard::Mode mode);
+    std::shared_ptr<X11DisplayGuard> d;
 
-    QSharedPointer<X11DisplayGuard> d;
-
-    bool m_copyclip;
-    bool m_checksel;
-    bool m_copysel;
     QStringList m_formats;
 
-    bool m_resetClipboard;
-    bool m_resetSelection;
-    bool m_syncFromClipboard;
-
-    QTimer m_timerIncompleteSelection;
-    QTimer m_timerReset;
-    QTimer m_timerSync;
+    QTimer m_timerCheckClipboard;
+    QTimer m_timerCheckSelection;
+    QTimer m_timerResetClipboard;
+    QTimer m_timerResetSelection;
 
     QVariantMap m_clipboardData;
     QVariantMap m_selectionData;
