@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2014, Lukas Holecek <hluk@email.cz>
+    Copyright (c) 2017, Lukas Holecek <hluk@email.cz>
 
     This file is part of CopyQ.
 
@@ -19,6 +19,7 @@
 
 #include "commandhelp.h"
 
+#include "common/common.h"
 #include "scriptable/scriptable.h"
 
 CommandHelp::CommandHelp()
@@ -57,7 +58,7 @@ QString CommandHelp::toString() const
 
 QList<CommandHelp> commandHelp()
 {
-    static QList<CommandHelp> help = QList<CommandHelp>()
+    return QList<CommandHelp>()
             << CommandHelp("show",
                            Scriptable::tr("Show main window and optionally open tab with given name."))
                .addArg("[" + Scriptable::tr("NAME") + "]")
@@ -75,7 +76,7 @@ QList<CommandHelp> commandHelp()
             << CommandHelp("clipboard",
                            Scriptable::tr("Print clipboard content."))
                .addArg("[" + Scriptable::tr("MIME") + "]")
-           #ifdef COPYQ_WS_X11
+           #ifdef HAS_MOUSE_SELECTIONS
             << CommandHelp("selection",
                            Scriptable::tr("Print X11 selection content."))
                .addArg("[" + Scriptable::tr("MIME") + "]")
@@ -90,8 +91,8 @@ QList<CommandHelp> commandHelp()
                .addArg(Scriptable::tr("DATA"))
                .addArg("[" + Scriptable::tr("MIME") + " " + Scriptable::tr("DATA") + "]...")
             << CommandHelp()
-            << CommandHelp("length, count, size",
-                           Scriptable::tr("Print number of items in history."))
+            << CommandHelp("count",
+                           Scriptable::tr("Print amount of items in current tab."))
             << CommandHelp("select",
                            Scriptable::tr("Copy item in the row to clipboard."))
                .addArg("[" + Scriptable::tr("ROW") + "=0]")
@@ -112,7 +113,7 @@ QList<CommandHelp> commandHelp()
             << CommandHelp("edit",
                            Scriptable::tr("Edit items or edit new one.\n"
                                           "Value -1 is for current text in clipboard."))
-               .addArg("[" + Scriptable::tr("ROWS") + "...]")
+               .addArg("[" + Scriptable::tr("ROW") + "=-1...]")
             << CommandHelp()
             << CommandHelp("separator",
                            Scriptable::tr("Set separator for items on output."))
@@ -175,7 +176,7 @@ QList<CommandHelp> commandHelp()
             << CommandHelp()
             << CommandHelp("eval, -e",
                            Scriptable::tr("\nEvaluate ECMAScript program.\n"
-                                          "Arguments are accessible using with \"arguments(0..N)\"."))
+                                          "Arguments are accessible using with \"arguments[0..N]\"."))
                .addArg("[" + Scriptable::tr("SCRIPT") + "]")
                .addArg("[" + Scriptable::tr("ARGUMENTS") + "]...")
             << CommandHelp("session, -s, --session",
@@ -188,12 +189,7 @@ QList<CommandHelp> commandHelp()
                            Scriptable::tr("\nPrint version of program and libraries."))
 #ifdef HAS_TESTS
             << CommandHelp("tests, --tests",
-                           QString("Run tests (append --help argument for more info)."))
-            << CommandHelp("keys",
-                           QString("Pass keys to the main window (used in tests)."))
-               .addArg(QString("KEYS") + "...")
+                           Scriptable::tr("Run application tests (append --help argument for more info)."))
 #endif
                ;
-
-    return help;
 }
